package yomichan

import (
	"regexp"
	"strings"

	zig "git.foosoft.net/alex/zero-epwing-go"
)

type shougakukan2Extractor struct {
	partsExp     *regexp.Regexp
	pitchExp     *regexp.Regexp
	readGroupExp *regexp.Regexp
	optExp       *regexp.Regexp
	kanaExp      *regexp.Regexp
	exprExp      *regexp.Regexp
}

func makeShougakukan2Extractor() epwingExtractor {
	return &shougakukan2Extractor{
		partsExp:     regexp.MustCompile(`^([^【]+?)(?:\(\d\))?(?:【(.*)】)?$`),
		pitchExp:     regexp.MustCompile(`発音図`),
		readGroupExp: regexp.MustCompile(`[-＊\s]`),
		optExp:       regexp.MustCompile(`（([^）]*)）`),
		kanaExp:      regexp.MustCompile(`^[\x{3040}-\x{30FF}]+$`),
		exprExp:      regexp.MustCompile(`^[\p{Han}\x{3040}-\x{30FF}]+$`),
	}
}

func (e *shougakukan2Extractor) extractTerms(entry zig.BookEntry, sequence int) []dbTerm {
	matches := e.partsExp.FindStringSubmatch(entry.Heading)
	if matches == nil {
		return nil
	}

	// skip pitch accent entries
	if e.pitchExp.MatchString(entry.Text) {
		return nil
	}

	reading := matches[1]
	if len(reading) == 0 {
		return nil
	}

	reading = e.readGroupExp.ReplaceAllLiteralString(reading, "")
	expression := matches[2]

	// handle parens, which indicate optional parts of the word
	type readingExpression struct{ reading, expression string }
	noOptional := readingExpression{
		reading:    e.optExp.ReplaceAllLiteralString(reading, ""),
		expression: e.optExp.ReplaceAllLiteralString(expression, ""),
	}
	readingExpressions := []readingExpression{noOptional}
	includedOptional := readingExpression{
		reading:    e.optExp.ReplaceAllString(reading, "$1"),
		expression: e.optExp.ReplaceAllString(expression, "$1"),
	}
	if includedOptional != noOptional {
		readingExpressions = append(readingExpressions, includedOptional)
	}

	var terms []dbTerm
	for _, re := range readingExpressions {
		reading := re.reading

		// skip C-J entries (some are legitimate J-C entries, but they have
		// romaji or kanji in the expressions without any readings)
		if !e.kanaExp.MatchString(reading) {
			continue
		}

		var expressions []string
		if expression := re.expression; len(expression) > 0 {
			for _, split := range strings.Split(expression, "・") {
				// some expressions are incomplete (they skip katakana parts of
				// the reading) or have romaji or punctuation in them
				if !e.exprExp.MatchString(split) {
					continue
				}

				expressions = append(expressions, split)
			}
		}

		if len(expressions) == 0 {
			expressions = []string{reading}
			reading = ""
		}

		for _, expression := range expressions {
			terms = append(terms, dbTerm{
				Expression: expression,
				Reading:    reading,
				Glossary:   []any{entry.Text},
				Sequence:   sequence,
			})
		}
	}

	return terms
}

func (*shougakukan2Extractor) extractKanji(entry zig.BookEntry) []dbKanji {
	return nil
}

func (*shougakukan2Extractor) getRevision() string {
	return "shougakukan2"
}

func (*shougakukan2Extractor) getFontNarrow() map[int]string {
	return map[int]string{
		0xA121: "\u00A9",
		0xA122: "\u00B7",
		0xA123: "\u00C0",
		0xA124: "\u00C1",
		0xA125: "\u00C2",
		0xA126: "\u00C3",
		0xA127: "\u00C4",
		0xA128: "\u00C5",
		0xA129: "\u00C8",
		0xA12A: "\u00C9",
		0xA12B: "\u00CA",
		0xA12C: "\u00CB",
		0xA12D: "\u00CC",
		0xA12E: "\u00CD",
		0xA12F: "\u00CE",
		0xA130: "\u00CF",
		0xA131: "\u00D2",
		0xA132: "\u00D3",
		0xA133: "\u00D4",
		0xA134: "\u00D5",
		0xA135: "\u00D6",
		0xA136: "\u00D9",
		0xA137: "\u00DA",
		0xA138: "\u00DB",
		0xA139: "\u00DC",
		0xA13A: "\u00E0",
		0xA13B: "\u00E1",
		0xA13C: "\u00E2",
		0xA13D: "\u00E3",
		0xA13E: "\u00E4",
		0xA13F: "\u00E5",
		0xA140: "\u00E8",
		0xA141: "\u00E9",
		0xA142: "\u00EA",
		0xA143: "\u00EB",
		0xA144: "\u00EC",
		0xA145: "\u00ED",
		0xA146: "\u00EE",
		0xA147: "\u00EF",
		0xA148: "\u00F2",
		0xA149: "\u00F3",
		0xA14A: "\u00F4",
		0xA14B: "\u00F5",
		0xA14C: "\u00F6",
		0xA14D: "\u00F9",
		0xA14E: "\u00FA",
		0xA14F: "\u00FB",
		0xA150: "\u00FC",
		0xA151: "\u0100",
		0xA152: "\u0101",
		0xA153: "\u0112",
		0xA154: "\u0113",
		0xA155: "\u011A",
		0xA156: "\u011B",
		0xA157: "\u012A",
		0xA158: "\u012B",
		0xA159: "\u0144",
		0xA15A: "\u0148",
		0xA15B: "\u014C",
		0xA15C: "\u014D",
		0xA15D: "\u016A",
		0xA15E: "\u016B",
		0xA15F: "\u01CD",
		0xA160: "\u01CE",
		0xA161: "\u01CF",
		0xA162: "\u01D0",
		0xA163: "\u01D1",
		0xA164: "\u01D2",
		0xA165: "\u01D3",
		0xA166: "\u01D4",
		0xA167: "\u01D5",
		0xA168: "\u01D6",
		0xA169: "\u01D7",
		0xA16A: "\u01D8",
		0xA16B: "\u01D9",
		0xA16C: "\u01DA",
		0xA16D: "\u01DB",
		0xA16E: "\u01DC",
		0xA170: "\u01F8",
		0xA171: "\u01F9",
		0xA172: "\uE000",
	}
}

func (*shougakukan2Extractor) getFontWide() map[int]string {
	return map[int]string{
		0xA221: "\u2153",
		0xA222: "\u24D0",
		0xA223: "\u24D1",
		0xA224: "\u24D2",
		0xA225: "\u2630",
		0xA226: "\u2631",
		0xA227: "\u2632",
		0xA228: "\u2633",
		0xA229: "\u2634",
		0xA22A: "\u2635",
		0xA22B: "\u2636",
		0xA22C: "\u2637",
		0xA22D: "\u3021",
		0xA22E: "\u3022",
		0xA22F: "\u3023",
		0xA230: "\u3024",
		0xA231: "\u3025",
		0xA232: "\u3026",
		0xA233: "\u3027",
		0xA234: "\u3028",
		0xA235: "\u3029",
		0xA236: "\u3030",
		0xA237: "\u3119",
		0xA238: "\u3397",
		0xA239: "\u4E0C",
		0xA23A: "\u4E0F",
		0xA23B: "\u4E13",
		0xA23C: "\u4E1A",
		0xA23D: "\u4E1B",
		0xA23E: "\u4E1C",
		0xA23F: "\u4E1D",
		0xA240: "\u4E1F",
		0xA241: "\u4E22",
		0xA242: "\u4E24",
		0xA243: "\u4E25",
		0xA244: "\u4E27",
		0xA245: "\u4E28",
		0xA246: "\u4E2B",
		0xA247: "\u4E30",
		0xA248: "\u4E34",
		0xA249: "\u4E3A",
		0xA24A: "\u4E3D",
		0xA24B: "\u4E3E",
		0xA24C: "\u4E47",
		0xA24D: "\u4E48",
		0xA24E: "\u4E49",
		0xA24F: "\u4E4C",
		0xA250: "\u4E50",
		0xA251: "\u4E52",
		0xA252: "\u4E53",
		0xA253: "\u4E54",
		0xA254: "\u4E5A",
		0xA255: "\u4E5C",
		0xA256: "\u4E60",
		0xA257: "\u4E61",
		0xA258: "\u4E66",
		0xA259: "\u4E69",
		0xA25A: "\u4E70",
		0xA25B: "\u4E78",
		0xA25C: "\u4E8D",
		0xA25D: "\u4E8F",
		0xA25E: "\u4E93",
		0xA25F: "\u4E9A",
		0xA260: "\u4EA7",
		0xA261: "\u4EA9",
		0xA262: "\u4EB1",
		0xA263: "\u4EB2",
		0xA264: "\u4EB5",
		0xA265: "\u4EB7",
		0xA266: "\u4EB8",
		0xA267: "\u4EB9",
		0xA268: "\u4EBB",
		0xA269: "\u4EBE",
		0xA26A: "\u4EBF",
		0xA26B: "\u4EC3",
		0xA26C: "\u4EC5",
		0xA26D: "\u4EC9",
		0xA26E: "\u4ED1",
		0xA26F: "\u4ED3",
		0xA270: "\u4EDB",
		0xA271: "\u4EE1",
		0xA272: "\u4EE8",
		0xA273: "\u4EEA",
		0xA274: "\u4EEB",
		0xA275: "\u4EEC",
		0xA276: "\u4EF3",
		0xA277: "\u4EF5",
		0xA278: "\u4EFD",
		0xA279: "\u4EFF",
		0xA27A: "\u4F00",
		0xA27B: "\u4F03",
		0xA27C: "\u4F08",
		0xA27D: "\u4F0B",
		0xA27E: "\u4F15",
		0xA321: "\u4F17",
		0xA322: "\u4F18",
		0xA323: "\u4F19",
		0xA324: "\u4F1B",
		0xA325: "\u4F1E",
		0xA326: "\u4F1F",
		0xA327: "\u4F20",
		0xA328: "\u4F21",
		0xA329: "\u4F22",
		0xA32A: "\u4F24",
		0xA32B: "\u4F25",
		0xA32C: "\u4F26",
		0xA32D: "\u4F27",
		0xA32E: "\u4F2A",
		0xA32F: "\u4F2B",
		0xA330: "\u4F32",
		0xA331: "\u4F37",
		0xA332: "\u4F3E",
		0xA333: "\u4F40",
		0xA334: "\u4F48",
		0xA335: "\u4F49",
		0xA336: "\u4F4A",
		0xA337: "\u4F54",
		0xA338: "\u4F58",
		0xA339: "\u4F5F",
		0xA33A: "\u4F60",
		0xA33B: "\u4F62",
		0xA33C: "\u4F63",
		0xA33D: "\u4F64",
		0xA33E: "\u4F65",
		0xA33F: "\u4F67",
		0xA340: "\u4F6B",
		0xA341: "\u4F6C",
		0xA342: "\u4F74",
		0xA343: "\u4F79",
		0xA344: "\u4F7A",
		0xA345: "\u4F7D",
		0xA346: "\u4F7E",
		0xA347: "\u4F81",
		0xA348: "\u4F82",
		0xA349: "\u4F84",
		0xA34A: "\u4F89",
		0xA34B: "\u4F94",
		0xA34C: "\u4F97",
		0xA34D: "\u4F9C",
		0xA34E: "\u4FA3",
		0xA34F: "\u4FA5",
		0xA350: "\u4FA6",
		0xA351: "\u4FA7",
		0xA352: "\u4FA8",
		0xA353: "\u4FA9",
		0xA354: "\u4FAA",
		0xA355: "\u4FAC",
		0xA356: "\u4FB4",
		0xA357: "\u4FB7",
		0xA358: "\u4FB9",
		0xA359: "\u4FBB",
		0xA35A: "\u4FC5",
		0xA35B: "\u4FCD",
		0xA35C: "\u4FCF",
		0xA35D: "\u4FDC",
		0xA35E: "\u4FDE",
		0xA35F: "\u4FE0",
		0xA360: "\u4FE6",
		0xA361: "\u4FE8",
		0xA362: "\u4FE9",
		0xA363: "\u4FEA",
		0xA364: "\u4FED",
		0xA365: "\u4FF1",
		0xA366: "\u4FFB",
		0xA367: "\u5000",
		0xA368: "\u5008",
		0xA369: "\u500C",
		0xA36A: "\u5010",
		0xA36B: "\u5013",
		0xA36C: "\u5018",
		0xA36D: "\u501C",
		0xA36E: "\u501E",
		0xA36F: "\u5022",
		0xA370: "\u502E",
		0xA371: "\u5033",
		0xA372: "\u5034",
		0xA373: "\u5037",
		0xA374: "\u5038",
		0xA375: "\u503A",
		0xA376: "\u503B",
		0xA377: "\u503C",
		0xA378: "\u503E",
		0xA379: "\u5041",
		0xA37A: "\u504A",
		0xA37B: "\u504C",
		0xA37C: "\u504E",
		0xA37D: "\u5053",
		0xA37E: "\u5062",
		0xA421: "\u506A",
		0xA422: "\u5070",
		0xA423: "\u5077",
		0xA424: "\u507A",
		0xA425: "\u507B",
		0xA426: "\u507E",
		0xA427: "\u507F",
		0xA428: "\u5088",
		0xA429: "\u5089",
		0xA42A: "\u508E",
		0xA42B: "\u5092",
		0xA42C: "\u5095",
		0xA42D: "\u5096",
		0xA42E: "\u509C",
		0xA42F: "\u50A2",
		0xA430: "\u50A3",
		0xA431: "\u50A4",
		0xA432: "\u50A5",
		0xA433: "\u50A7",
		0xA434: "\u50A8",
		0xA435: "\u50A9",
		0xA436: "\u50AF",
		0xA437: "\u50BA",
		0xA438: "\u50BB",
		0xA439: "\u50C4",
		0xA43A: "\u50C7",
		0xA43B: "\u50DD",
		0xA43C: "\u50E1",
		0xA43D: "\u50E2",
		0xA43E: "\u50E6",
		0xA43F: "\u50EC",
		0xA440: "\u50F0",
		0xA441: "\u50F1",
		0xA442: "\u50F3",
		0xA443: "\u50F6",
		0xA444: "\u50FD",
		0xA445: "\u50FE",
		0xA446: "\u5106",
		0xA447: "\u5107",
		0xA448: "\u5108",
		0xA449: "\u510B",
		0xA44A: "\u510C",
		0xA44B: "\u510D",
		0xA44C: "\u5110",
		0xA44D: "\u5113",
		0xA44E: "\u5117",
		0xA44F: "\u512D",
		0xA450: "\u5131",
		0xA451: "\u5134",
		0xA452: "\u5135",
		0xA453: "\u5138",
		0xA454: "\u5139",
		0xA455: "\u513D",
		0xA456: "\u5151",
		0xA457: "\u5155",
		0xA458: "\u5156",
		0xA459: "\u5170",
		0xA45A: "\u5173",
		0xA45B: "\u5174",
		0xA45C: "\u5179",
		0xA45D: "\u517B",
		0xA45E: "\u517D",
		0xA45F: "\u5181",
		0xA460: "\u5184",
		0xA461: "\u5187",
		0xA462: "\u5188",
		0xA463: "\u519B",
		0xA464: "\u519C",
		0xA465: "\u51AE",
		0xA466: "\u51AF",
		0xA467: "\u51BB",
		0xA468: "\u51BC",
		0xA469: "\u51C0",
		0xA46A: "\u51C3",
		0xA46B: "\u51C7",
		0xA46C: "\u51C8",
		0xA46D: "\u51CA",
		0xA46E: "\u51CF",
		0xA46F: "\u51D1",
		0xA470: "\u51D3",
		0xA471: "\u51D4",
		0xA472: "\u51D8",
		0xA473: "\u51D9",
		0xA474: "\u51DF",
		0xA475: "\u51E2",
		0xA476: "\u51E4",
		0xA477: "\u51EB",
		0xA478: "\u51EF",
		0xA479: "\u51F3",
		0xA47A: "\u51F4",
		0xA47B: "\u51F7",
		0xA47C: "\u51FB",
		0xA47D: "\u51FC",
		0xA47E: "\u51FF",
		0xA521: "\u5201",
		0xA522: "\u5202",
		0xA523: "\u520C",
		0xA524: "\u520D",
		0xA525: "\u5212",
		0xA526: "\u5213",
		0xA527: "\u5216",
		0xA528: "\u5218",
		0xA529: "\u5219",
		0xA52A: "\u521A",
		0xA52B: "\u521B",
		0xA52C: "\u521C",
		0xA52D: "\u5220",
		0xA52E: "\u5226",
		0xA52F: "\u5228",
		0xA530: "\u522B",
		0xA531: "\u522C",
		0xA532: "\u522D",
		0xA533: "\u5232",
		0xA534: "\u5234",
		0xA535: "\u5235",
		0xA536: "\u523C",
		0xA537: "\u523D",
		0xA538: "\u523F",
		0xA539: "\u5240",
		0xA53A: "\u5241",
		0xA53B: "\u5242",
		0xA53C: "\u5245",
		0xA53D: "\u5249",
		0xA53E: "\u5250",
		0xA53F: "\u5251",
		0xA540: "\u5252",
		0xA541: "\u5255",
		0xA542: "\u5257",
		0xA543: "\u5259",
		0xA544: "\u525A",
		0xA545: "\u525C",
		0xA546: "\u525F",
		0xA547: "\u5260",
		0xA548: "\u5261",
		0xA549: "\u5267",
		0xA54A: "\u5268",
		0xA54B: "\u526E",
		0xA54C: "\u5277",
		0xA54D: "\u527A",
		0xA54E: "\u5280",
		0xA54F: "\u5281",
		0xA550: "\u5282",
		0xA551: "\u5284",
		0xA552: "\u528A",
		0xA553: "\u528C",
		0xA554: "\u5290",
		0xA555: "\u5293",
		0xA556: "\u5298",
		0xA557: "\u5299",
		0xA558: "\u529A",
		0xA559: "\u529D",
		0xA55A: "\u529E",
		0xA55B: "\u52A1",
		0xA55C: "\u52A2",
		0xA55D: "\u52A4",
		0xA55E: "\u52A8",
		0xA55F: "\u52B2",
		0xA560: "\u52B3",
		0xA561: "\u52BB",
		0xA562: "\u52BF",
		0xA563: "\u52CB",
		0xA564: "\u52D0",
		0xA565: "\u52D1",
		0xA566: "\u52D4",
		0xA567: "\u52D6",
		0xA568: "\u52DA",
		0xA569: "\u52E5",
		0xA56A: "\u52F0",
		0xA56B: "\u52F1",
		0xA56C: "\u52F7",
		0xA56D: "\u52FC",
		0xA56E: "\u5300",
		0xA56F: "\u5304",
		0xA570: "\u5309",
		0xA571: "\u530A",
		0xA572: "\u530B",
		0xA573: "\u531C",
		0xA574: "\u5326",
		0xA575: "\u532E",
		0xA576: "\u5332",
		0xA577: "\u5335",
		0xA578: "\u533C",
		0xA579: "\u533E",
		0xA57A: "\u534B",
		0xA57B: "\u534C",
		0xA57C: "\u534E",
		0xA57D: "\u534F",
		0xA57E: "\u5350",
		0xA621: "\u5355",
		0xA622: "\u5356",
		0xA623: "\u535D",
		0xA624: "\u535F",
		0xA625: "\u5361",
		0xA626: "\u5362",
		0xA627: "\u5363",
		0xA628: "\u5364",
		0xA629: "\u5367",
		0xA62A: "\u5368",
		0xA62B: "\u536B",
		0xA62C: "\u536C",
		0xA62D: "\u5372",
		0xA62E: "\u5379",
		0xA62F: "\u537A",
		0xA630: "\u5385",
		0xA631: "\u5386",
		0xA632: "\u5389",
		0xA633: "\u538B",
		0xA634: "\u538C",
		0xA635: "\u538D",
		0xA636: "\u5393",
		0xA637: "\u5394",
		0xA638: "\u5395",
		0xA639: "\u5399",
		0xA63A: "\u539B",
		0xA63B: "\u539D",
		0xA63C: "\u53A2",
		0xA63D: "\u53A3",
		0xA63E: "\u53A4",
		0xA63F: "\u53AA",
		0xA640: "\u53AB",
		0xA641: "\u53B2",
		0xA642: "\u53B4",
		0xA643: "\u53BE",
		0xA644: "\u53BF",
		0xA645: "\u53C1",
		0xA646: "\u53C6",
		0xA647: "\u53C7",
		0xA648: "\u53D1",
		0xA649: "\u53D8",
		0xA64A: "\u53DC",
		0xA64B: "\u53E0",
		0xA64C: "\u53E6",
		0xA64D: "\u53F5",
		0xA64E: "\u53F9",
		0xA64F: "\u53FB",
		0xA650: "\u53FC",
		0xA651: "\u53FD",
		0xA652: "\u5406",
		0xA653: "\u5412",
		0xA654: "\u5413",
		0xA655: "\u5415",
		0xA656: "\u5416",
		0xA657: "\u5417",
		0xA658: "\u541A",
		0xA659: "\u541E",
		0xA65A: "\u5421",
		0xA65B: "\u5422",
		0xA65C: "\u5423",
		0xA65D: "\u5425",
		0xA65E: "\u5427",
		0xA65F: "\u5428",
		0xA660: "\u542A",
		0xA661: "\u542F",
		0xA662: "\u5430",
		0xA663: "\u5431",
		0xA664: "\u5432",
		0xA665: "\u5433",
		0xA666: "\u5434",
		0xA667: "\u5435",
		0xA668: "\u5443",
		0xA669: "\u544B",
		0xA66A: "\u544C",
		0xA66B: "\u544D",
		0xA66C: "\u5450",
		0xA66D: "\u5452",
		0xA66E: "\u5453",
		0xA66F: "\u5454",
		0xA670: "\u5455",
		0xA671: "\u5456",
		0xA672: "\u5457",
		0xA673: "\u5458",
		0xA674: "\u5459",
		0xA675: "\u545B",
		0xA676: "\u545C",
		0xA677: "\u545D",
		0xA678: "\u5462",
		0xA679: "\u5463",
		0xA67A: "\u5464",
		0xA67B: "\u5466",
		0xA67C: "\u5472",
		0xA67D: "\u5478",
		0xA67E: "\u5482",
		0xA721: "\u5488",
		0xA722: "\u548D",
		0xA723: "\u5491",
		0xA724: "\u5494",
		0xA725: "\u5495",
		0xA726: "\u5496",
		0xA727: "\u5499",
		0xA728: "\u549A",
		0xA729: "\u549B",
		0xA72A: "\u549D",
		0xA72B: "\u54A3",
		0xA72C: "\u54A6",
		0xA72D: "\u54A7",
		0xA72E: "\u54A9",
		0xA72F: "\u54AA",
		0xA730: "\u54AD",
		0xA731: "\u54AE",
		0xA732: "\u54B1",
		0xA733: "\u54B4",
		0xA734: "\u54B5",
		0xA735: "\u54B7",
		0xA736: "\u54B9",
		0xA737: "\u54BB",
		0xA738: "\u54BF",
		0xA739: "\u54C5",
		0xA73A: "\u54C6",
		0xA73B: "\u54CC",
		0xA73C: "\u54CD",
		0xA73D: "\u54CE",
		0xA73E: "\u54CF",
		0xA73F: "\u54D0",
		0xA740: "\u54D1",
		0xA741: "\u54D2",
		0xA742: "\u54D3",
		0xA743: "\u54D4",
		0xA744: "\u54D5",
		0xA745: "\u54D7",
		0xA746: "\u54D9",
		0xA747: "\u54DA",
		0xA748: "\u54DC",
		0xA749: "\u54DD",
		0xA74A: "\u54DE",
		0xA74B: "\u54DF",
		0xA74C: "\u54E7",
		0xA74D: "\u54EA",
		0xA74E: "\u54F1",
		0xA74F: "\u54F3",
		0xA750: "\u54FC",
		0xA751: "\u54FF",
		0xA752: "\u5501",
		0xA753: "\u5508",
		0xA754: "\u5509",
		0xA755: "\u5511",
		0xA756: "\u5515",
		0xA757: "\u5517",
		0xA758: "\u5519",
		0xA759: "\u551A",
		0xA75A: "\u551B",
		0xA75B: "\u551D",
		0xA75C: "\u5520",
		0xA75D: "\u5521",
		0xA75E: "\u5522",
		0xA75F: "\u5523",
		0xA760: "\u5524",
		0xA761: "\u5527",
		0xA762: "\u552A",
		0xA763: "\u552B",
		0xA764: "\u552C",
		0xA765: "\u5530",
		0xA766: "\u5535",
		0xA767: "\u5537",
		0xA768: "\u553B",
		0xA769: "\u553C",
		0xA76A: "\u553F",
		0xA76B: "\u5541",
		0xA76C: "\u5543",
		0xA76D: "\u5548",
		0xA76E: "\u5549",
		0xA76F: "\u554A",
		0xA770: "\u5550",
		0xA771: "\u5551",
		0xA772: "\u5555",
		0xA773: "\u5559",
		0xA774: "\u555E",
		0xA775: "\u555F",
		0xA776: "\u5561",
		0xA777: "\u5562",
		0xA778: "\u5564",
		0xA779: "\u5565",
		0xA77A: "\u5566",
		0xA77B: "\u5567",
		0xA77C: "\u556A",
		0xA77D: "\u556C",
		0xA77E: "\u556D",
		0xA821: "\u556E",
		0xA822: "\u556F",
		0xA823: "\u5570",
		0xA824: "\u5574",
		0xA825: "\u5575",
		0xA826: "\u5576",
		0xA827: "\u5577",
		0xA828: "\u5578",
		0xA829: "\u5581",
		0xA82A: "\u5582",
		0xA82B: "\u5588",
		0xA82C: "\u558C",
		0xA82D: "\u558E",
		0xA82E: "\u558F",
		0xA82F: "\u5591",
		0xA830: "\u5592",
		0xA831: "\u5593",
		0xA832: "\u5594",
		0xA833: "\u55A4",
		0xA834: "\u55AD",
		0xA835: "\u55B1",
		0xA836: "\u55B3",
		0xA837: "\u55B5",
		0xA838: "\u55B7",
		0xA839: "\u55B9",
		0xA83A: "\u55BB",
		0xA83B: "\u55BD",
		0xA83C: "\u55BE",
		0xA83D: "\u55C1",
		0xA83E: "\u55C6",
		0xA83F: "\u55C8",
		0xA840: "\u55C9",
		0xA841: "\u55CC",
		0xA842: "\u55CD",
		0xA843: "\u55D0",
		0xA844: "\u55D1",
		0xA845: "\u55D2",
		0xA846: "\u55D3",
		0xA847: "\u55D6",
		0xA848: "\u55D9",
		0xA849: "\u55DB",
		0xA84A: "\u55DD",
		0xA84B: "\u55DE",
		0xA84C: "\u55E1",
		0xA84D: "\u55E5",
		0xA84E: "\u55E6",
		0xA84F: "\u55E8",
		0xA850: "\u55EA",
		0xA851: "\u55EB",
		0xA852: "\u55EC",
		0xA853: "\u55EF",
		0xA854: "\u55F2",
		0xA855: "\u55F3",
		0xA856: "\u55F5",
		0xA857: "\u55F6",
		0xA858: "\u55FB",
		0xA859: "\u5600",
		0xA85A: "\u5601",
		0xA85B: "\u5605",
		0xA85C: "\u5608",
		0xA85D: "\u560C",
		0xA85E: "\u560D",
		0xA85F: "\u560E",
		0xA860: "\u560F",
		0xA861: "\u5611",
		0xA862: "\u5612",
		0xA863: "\u5613",
		0xA864: "\u561A",
		0xA865: "\u561D",
		0xA866: "\u561E",
		0xA867: "\u561F",
		0xA868: "\u5621",
		0xA869: "\u5623",
		0xA86A: "\u5624",
		0xA86B: "\u5627",
		0xA86C: "\u562C",
		0xA86D: "\u562D",
		0xA86E: "\u562E",
		0xA86F: "\u5630",
		0xA870: "\u5635",
		0xA871: "\u5637",
		0xA872: "\u5639",
		0xA873: "\u563B",
		0xA874: "\u563D",
		0xA875: "\u563F",
		0xA876: "\u5640",
		0xA877: "\u5641",
		0xA878: "\u5646",
		0xA879: "\u5647",
		0xA87A: "\u5649",
		0xA87B: "\u564D",
		0xA87C: "\u564F",
		0xA87D: "\u5654",
		0xA87E: "\u5657",
		0xA921: "\u5658",
		0xA922: "\u5659",
		0xA923: "\u565A",
		0xA924: "\u565C",
		0xA925: "\u565D",
		0xA926: "\u565E",
		0xA927: "\u5660",
		0xA928: "\u5662",
		0xA929: "\u5665",
		0xA92A: "\u5666",
		0xA92B: "\u5669",
		0xA92C: "\u566D",
		0xA92D: "\u566F",
		0xA92E: "\u5670",
		0xA92F: "\u5671",
		0xA930: "\u5672",
		0xA931: "\u5676",
		0xA932: "\u5677",
		0xA933: "\u5679",
		0xA934: "\u567B",
		0xA935: "\u567C",
		0xA936: "\u567D",
		0xA937: "\u5683",
		0xA938: "\u5684",
		0xA939: "\u5685",
		0xA93A: "\u568C",
		0xA93B: "\u568E",
		0xA93C: "\u5690",
		0xA93D: "\u5693",
		0xA93E: "\u569A",
		0xA93F: "\u569C",
		0xA940: "\u56A3",
		0xA941: "\u56A6",
		0xA942: "\u56A8",
		0xA943: "\u56AB",
		0xA944: "\u56AD",
		0xA945: "\u56AF",
		0xA946: "\u56B2",
		0xA947: "\u56B3",
		0xA948: "\u56B7",
		0xA949: "\u56BB",
		0xA94A: "\u56BE",
		0xA94B: "\u56C5",
		0xA94C: "\u56C9",
		0xA94D: "\u56CA",
		0xA94E: "\u56CC",
		0xA94F: "\u56CD",
		0xA950: "\u56D4",
		0xA951: "\u56DD",
		0xA952: "\u56DF",
		0xA953: "\u56E1",
		0xA954: "\u56E2",
		0xA955: "\u56E4",
		0xA956: "\u56E5",
		0xA957: "\u56EB",
		0xA958: "\u56EC",
		0xA959: "\u56ED",
		0xA95A: "\u56EF",
		0xA95B: "\u56F1",
		0xA95C: "\u56F4",
		0xA95D: "\u56F5",
		0xA95E: "\u56F7",
		0xA95F: "\u56FE",
		0xA960: "\u5702",
		0xA961: "\u5705",
		0xA962: "\u5706",
		0xA963: "\u5707",
		0xA964: "\u570A",
		0xA965: "\u570C",
		0xA966: "\u5710",
		0xA967: "\u5715",
		0xA968: "\u5719",
		0xA969: "\u571D",
		0xA96A: "\u571E",
		0xA96B: "\u5723",
		0xA96C: "\u5729",
		0xA96D: "\u572A",
		0xA96E: "\u572C",
		0xA96F: "\u572E",
		0xA970: "\u572F",
		0xA971: "\u5733",
		0xA972: "\u5739",
		0xA973: "\u573A",
		0xA974: "\u573E",
		0xA975: "\u574B",
		0xA976: "\u574C",
		0xA977: "\u574D",
		0xA978: "\u5757",
		0xA979: "\u575A",
		0xA97A: "\u575B",
		0xA97B: "\u575C",
		0xA97C: "\u575D",
		0xA97D: "\u575E",
		0xA97E: "\u575F",
		0xAA21: "\u5760",
		0xAA22: "\u5762",
		0xAA23: "\u5768",
		0xAA24: "\u576B",
		0xAA25: "\u576D",
		0xAA26: "\u576F",
		0xAA27: "\u5770",
		0xAA28: "\u5773",
		0xAA29: "\u5775",
		0xAA2A: "\u5776",
		0xAA2B: "\u5777",
		0xAA2C: "\u577B",
		0xAA2D: "\u577C",
		0xAA2E: "\u5783",
		0xAA2F: "\u5784",
		0xAA30: "\u5786",
		0xAA31: "\u5787",
		0xAA32: "\u578C",
		0xAA33: "\u578D",
		0xAA34: "\u5792",
		0xAA35: "\u5795",
		0xAA36: "\u579A",
		0xAA37: "\u579B",
		0xAA38: "\u579C",
		0xAA39: "\u579D",
		0xAA3A: "\u579E",
		0xAA3B: "\u579F",
		0xAA3C: "\u57A1",
		0xAA3D: "\u57A6",
		0xAA3E: "\u57A7",
		0xAA3F: "\u57A9",
		0xAA40: "\u57AB",
		0xAA41: "\u57AD",
		0xAA42: "\u57AE",
		0xAA43: "\u57AF",
		0xAA44: "\u57B1",
		0xAA45: "\u57B2",
		0xAA46: "\u57B4",
		0xAA47: "\u57B5",
		0xAA48: "\u57B8",
		0xAA49: "\u57BD",
		0xAA4A: "\u57BE",
		0xAA4B: "\u57BF",
		0xAA4C: "\u57C2",
		0xAA4D: "\u57C7",
		0xAA4E: "\u57CC",
		0xAA4F: "\u57CF",
		0xAA50: "\u57D5",
		0xAA51: "\u57D8",
		0xAA52: "\u57D9",
		0xAA53: "\u57DA",
		0xAA54: "\u57DD",
		0xAA55: "\u57E1",
		0xAA56: "\u57E4",
		0xAA57: "\u57E8",
		0xAA58: "\u57ED",
		0xAA59: "\u57EF",
		0xAA5A: "\u57F0",
		0xAA5B: "\u57F3",
		0xAA5C: "\u57F5",
		0xAA5D: "\u57F8",
		0xAA5E: "\u57FB",
		0xAA5F: "\u57FD",
		0xAA60: "\u5803",
		0xAA61: "\u5804",
		0xAA62: "\u5807",
		0xAA63: "\u5809",
		0xAA64: "\u580C",
		0xAA65: "\u580D",
		0xAA66: "\u580E",
		0xAA67: "\u5811",
		0xAA68: "\u5816",
		0xAA69: "\u5818",
		0xAA6A: "\u581E",
		0xAA6B: "\u5820",
		0xAA6C: "\u5826",
		0xAA6D: "\u5827",
		0xAA6E: "\u5829",
		0xAA6F: "\u5836",
		0xAA70: "\u583E",
		0xAA71: "\u583F",
		0xAA72: "\u5843",
		0xAA73: "\u5844",
		0xAA74: "\u5845",
		0xAA75: "\u5846",
		0xAA76: "\u5848",
		0xAA77: "\u584C",
		0xAA78: "\u584D",
		0xAA79: "\u584F",
		0xAA7A: "\u5855",
		0xAA7B: "\u585D",
		0xAA7C: "\u5865",
		0xAA7D: "\u5868",
		0xAA7E: "\u586C",
		0xAB21: "\u586E",
		0xAB22: "\u5871",
		0xAB23: "\u587C",
		0xAB24: "\u587D",
		0xAB25: "\u5880",
		0xAB26: "\u5881",
		0xAB27: "\u5888",
		0xAB28: "\u5889",
		0xAB29: "\u588A",
		0xAB2A: "\u588B",
		0xAB2B: "\u5890",
		0xAB2C: "\u5892",
		0xAB2D: "\u5895",
		0xAB2E: "\u5896",
		0xAB2F: "\u5898",
		0xAB30: "\u5899",
		0xAB31: "\u589A",
		0xAB32: "\u589D",
		0xAB33: "\u589E",
		0xAB34: "\u58A0",
		0xAB35: "\u58A1",
		0xAB36: "\u58A6",
		0xAB37: "\u58A9",
		0xAB38: "\u58AC",
		0xAB39: "\u58B0",
		0xAB3A: "\u58B6",
		0xAB3B: "\u58BC",
		0xAB3C: "\u58C4",
		0xAB3D: "\u58C8",
		0xAB3E: "\u58CB",
		0xAB3F: "\u58CE",
		0xAB40: "\u58D4",
		0xAB41: "\u58D6",
		0xAB42: "\u58DA",
		0xAB43: "\u58E0",
		0xAB44: "\u58E2",
		0xAB45: "\u58E9",
		0xAB46: "\u58EA",
		0xAB47: "\u58F3",
		0xAB48: "\u58F6",
		0xAB49: "\u58F8",
		0xAB4A: "\u5900",
		0xAB4B: "\u5904",
		0xAB4C: "\u5907",
		0xAB4D: "\u590D",
		0xAB4E: "\u5912",
		0xAB4F: "\u5914",
		0xAB50: "\u591F",
		0xAB51: "\u5920",
		0xAB52: "\u5924",
		0xAB53: "\u592F",
		0xAB54: "\u5934",
		0xAB55: "\u5939",
		0xAB56: "\u593A",
		0xAB57: "\u593C",
		0xAB58: "\u593F",
		0xAB59: "\u5940",
		0xAB5A: "\u5941",
		0xAB5B: "\u5942",
		0xAB5C: "\u5945",
		0xAB5D: "\u594B",
		0xAB5E: "\u5953",
		0xAB5F: "\u5956",
		0xAB60: "\u5961",
		0xAB61: "\u5964",
		0xAB62: "\u596D",
		0xAB63: "\u5970",
		0xAB64: "\u5972",
		0xAB65: "\u5976",
		0xAB66: "\u5979",
		0xAB67: "\u597C",
		0xAB68: "\u5986",
		0xAB69: "\u5987",
		0xAB6A: "\u5988",
		0xAB6B: "\u5992",
		0xAB6C: "\u5997",
		0xAB6D: "\u5998",
		0xAB6E: "\u599E",
		0xAB6F: "\u59A4",
		0xAB70: "\u59A9",
		0xAB71: "\u59AA",
		0xAB72: "\u59AB",
		0xAB73: "\u59AE",
		0xAB74: "\u59AF",
		0xAB75: "\u59B3",
		0xAB76: "\u59BA",
		0xAB77: "\u59C1",
		0xAB78: "\u59CA",
		0xAB79: "\u59CD",
		0xAB7A: "\u59D2",
		0xAB7B: "\u59D7",
		0xAB7C: "\u59D8",
		0xAB7D: "\u59DD",
		0xAB7E: "\u59DE",
		0xAC21: "\u59E3",
		0xAC22: "\u59E4",
		0xAC23: "\u59EC",
		0xAC24: "\u59EE",
		0xAC25: "\u59F1",
		0xAC26: "\u59F8",
		0xAC27: "\u59F9",
		0xAC28: "\u59FD",
		0xAC29: "\u5A00",
		0xAC2A: "\u5A04",
		0xAC2B: "\u5A05",
		0xAC2C: "\u5A06",
		0xAC2D: "\u5A07",
		0xAC2E: "\u5A08",
		0xAC2F: "\u5A0C",
		0xAC30: "\u5A12",
		0xAC31: "\u5A13",
		0xAC32: "\u5A16",
		0xAC33: "\u5A23",
		0xAC34: "\u5A2C",
		0xAC35: "\u5A2D",
		0xAC36: "\u5A31",
		0xAC37: "\u5A32",
		0xAC38: "\u5A34",
		0xAC39: "\u5A3F",
		0xAC3A: "\u5A4A",
		0xAC3B: "\u5A55",
		0xAC3C: "\u5A57",
		0xAC3D: "\u5A5E",
		0xAC3E: "\u5A63",
		0xAC3F: "\u5A65",
		0xAC40: "\u5A67",
		0xAC41: "\u5A6D",
		0xAC42: "\u5A73",
		0xAC43: "\u5A74",
		0xAC44: "\u5A75",
		0xAC45: "\u5A76",
		0xAC46: "\u5A77",
		0xAC47: "\u5A7A",
		0xAC48: "\u5A7C",
		0xAC49: "\u5A95",
		0xAC4A: "\u5A96",
		0xAC4B: "\u5A9F",
		0xAC4C: "\u5AA2",
		0xAC4D: "\u5AA7",
		0xAC4E: "\u5AAA",
		0xAC4F: "\u5AAD",
		0xAC50: "\u5AAE",
		0xAC51: "\u5AAF",
		0xAC52: "\u5AB2",
		0xAC53: "\u5AB3",
		0xAC54: "\u5AB5",
		0xAC55: "\u5AB8",
		0xAC56: "\u5ABF",
		0xAC57: "\u5AC4",
		0xAC58: "\u5ACF",
		0xAC59: "\u5AD2",
		0xAC5A: "\u5AD4",
		0xAC5B: "\u5AD5",
		0xAC5C: "\u5AD8",
		0xAC5D: "\u5ADA",
		0xAC5E: "\u5ADB",
		0xAC5F: "\u5ADC",
		0xAC60: "\u5AE0",
		0xAC61: "\u5AEA",
		0xAC62: "\u5AEB",
		0xAC63: "\u5AF0",
		0xAC64: "\u5AF1",
		0xAC65: "\u5AF3",
		0xAC66: "\u5AF5",
		0xAC67: "\u5AFD",
		0xAC68: "\u5AFF",
		0xAC69: "\u5B00",
		0xAC6A: "\u5B08",
		0xAC6B: "\u5B0E",
		0xAC6C: "\u5B17",
		0xAC6D: "\u5B19",
		0xAC6E: "\u5B1B",
		0xAC6F: "\u5B1D",
		0xAC70: "\u5B21",
		0xAC71: "\u5B2D",
		0xAC72: "\u5B34",
		0xAC73: "\u5B37",
		0xAC74: "\u5B38",
		0xAC75: "\u5B3F",
		0xAC76: "\u5B4C",
		0xAC77: "\u5B53",
		0xAC78: "\u5B56",
		0xAC79: "\u5B59",
		0xAC7A: "\u5B62",
		0xAC7B: "\u5B6A",
		0xAC7C: "\u5B6C",
		0xAC7D: "\u5B7C",
		0xAC7E: "\u5B7D",
		0xAD21: "\u5B7F",
		0xAD22: "\u5B81",
		0xAD23: "\u5B82",
		0xAD24: "\u5B84",
		0xAD25: "\u5B93",
		0xAD26: "\u5B9E",
		0xAD27: "\u5BA0",
		0xAD28: "\u5BA1",
		0xAD29: "\u5BA7",
		0xAD2A: "\u5BAA",
		0xAD2B: "\u5BAB",
		0xAD2C: "\u5BAC",
		0xAD2D: "\u5BAD",
		0xAD2E: "\u5BB7",
		0xAD2F: "\u5BBD",
		0xAD30: "\u5BBE",
		0xAD31: "\u5BD6",
		0xAD32: "\u5BD8",
		0xAD33: "\u5BEC",
		0xAD34: "\u5BEF",
		0xAD35: "\u5BF1",
		0xAD36: "\u5BF9",
		0xAD37: "\u5BFB",
		0xAD38: "\u5BFC",
		0xAD39: "\u5C14",
		0xAD3A: "\u5C15",
		0xAD3B: "\u5C17",
		0xAD3C: "\u5C18",
		0xAD3D: "\u5C1C",
		0xAD3E: "\u5C1D",
		0xAD3F: "\u5C1F",
		0xAD40: "\u5C25",
		0xAD41: "\u5C27",
		0xAD42: "\u5C2C",
		0xAD43: "\u5C32",
		0xAD44: "\u5C34",
		0xAD45: "\u5C37",
		0xAD46: "\u5C42",
		0xAD47: "\u5C43",
		0xAD48: "\u5C44",
		0xAD49: "\u5C49",
		0xAD4A: "\u5C4C",
		0xAD4B: "\u5C59",
		0xAD4C: "\u5C5C",
		0xAD4D: "\u5C5F",
		0xAD4E: "\u5C62",
		0xAD4F: "\u5C63",
		0xAD50: "\u5C66",
		0xAD51: "\u5C67",
		0xAD52: "\u5C68",
		0xAD53: "\u5C69",
		0xAD54: "\u5C6D",
		0xAD55: "\u5C7A",
		0xAD56: "\u5C7C",
		0xAD57: "\u5C7E",
		0xAD58: "\u5C7F",
		0xAD59: "\u5C81",
		0xAD5A: "\u5C82",
		0xAD5B: "\u5C88",
		0xAD5C: "\u5C8D",
		0xAD5D: "\u5C96",
		0xAD5E: "\u5C97",
		0xAD5F: "\u5C98",
		0xAD60: "\u5C99",
		0xAD61: "\u5C9A",
		0xAD62: "\u5C9B",
		0xAD63: "\u5C9C",
		0xAD64: "\u5C9E",
		0xAD65: "\u5CA2",
		0xAD66: "\u5CA3",
		0xAD67: "\u5CA7",
		0xAD68: "\u5CAD",
		0xAD69: "\u5CB5",
		0xAD6A: "\u5CB9",
		0xAD6B: "\u5CBD",
		0xAD6C: "\u5CBF",
		0xAD6D: "\u5CC1",
		0xAD6E: "\u5CC2",
		0xAD6F: "\u5CC3",
		0xAD70: "\u5CC4",
		0xAD71: "\u5CCB",
		0xAD72: "\u5CD2",
		0xAD73: "\u5CE3",
		0xAD74: "\u5CE4",
		0xAD75: "\u5CE5",
		0xAD76: "\u5CE6",
		0xAD77: "\u5CE7",
		0xAD78: "\u5CEC",
		0xAD79: "\u5CF1",
		0xAD7A: "\u5D00",
		0xAD7B: "\u5D01",
		0xAD7C: "\u5D02",
		0xAD7D: "\u5D03",
		0xAD7E: "\u5D04",
		0xAE21: "\u5D06",
		0xAE22: "\u5D0D",
		0xAE23: "\u5D12",
		0xAE24: "\u5D1E",
		0xAE25: "\u5D24",
		0xAE26: "\u5D26",
		0xAE27: "\u5D27",
		0xAE28: "\u5D2A",
		0xAE29: "\u5D2C",
		0xAE2A: "\u5D2D",
		0xAE2B: "\u5D2E",
		0xAE2C: "\u5D2F",
		0xAE2D: "\u5D34",
		0xAE2E: "\u5D3D",
		0xAE2F: "\u5D3F",
		0xAE30: "\u5D46",
		0xAE31: "\u5D47",
		0xAE32: "\u5D4A",
		0xAE33: "\u5D56",
		0xAE34: "\u5D57",
		0xAE35: "\u5D58",
		0xAE36: "\u5D5A",
		0xAE37: "\u5D5B",
		0xAE38: "\u5D5D",
		0xAE39: "\u5D6B",
		0xAE3A: "\u5D72",
		0xAE3B: "\u5D74",
		0xAE3C: "\u5D7D",
		0xAE3D: "\u5D81",
		0xAE3E: "\u5D8D",
		0xAE3F: "\u5D92",
		0xAE40: "\u5D93",
		0xAE41: "\u5D95",
		0xAE42: "\u5D97",
		0xAE43: "\u5D99",
		0xAE44: "\u5DA0",
		0xAE45: "\u5DA7",
		0xAE46: "\u5DA8",
		0xAE47: "\u5DA9",
		0xAE48: "\u5DB0",
		0xAE49: "\u5DB2",
		0xAE4A: "\u5DB4",
		0xAE4B: "\u5DB8",
		0xAE4C: "\u5DC5",
		0xAE4D: "\u5DC7",
		0xAE4E: "\u5DCB",
		0xAE4F: "\u5DCE",
		0xAE50: "\u5DD8",
		0xAE51: "\u5DE0",
		0xAE52: "\u5DE2",
		0xAE53: "\u5DE9",
		0xAE54: "\u5DEF",
		0xAE55: "\u5DF0",
		0xAE56: "\u5DFF",
		0xAE57: "\u5E00",
		0xAE58: "\u5E01",
		0xAE59: "\u5E05",
		0xAE5A: "\u5E08",
		0xAE5B: "\u5E0A",
		0xAE5C: "\u5E0F",
		0xAE5D: "\u5E10",
		0xAE5E: "\u5E14",
		0xAE5F: "\u5E15",
		0xAE60: "\u5E18",
		0xAE61: "\u5E1C",
		0xAE62: "\u5E21",
		0xAE63: "\u5E23",
		0xAE64: "\u5E26",
		0xAE65: "\u5E27",
		0xAE66: "\u5E28",
		0xAE67: "\u5E29",
		0xAE68: "\u5E2C",
		0xAE69: "\u5E2E",
		0xAE6A: "\u5E31",
		0xAE6B: "\u5E35",
		0xAE6C: "\u5E3B",
		0xAE6D: "\u5E3C",
		0xAE6E: "\u5E42",
		0xAE6F: "\u5E48",
		0xAE70: "\u5E56",
		0xAE71: "\u5E5B",
		0xAE72: "\u5E5E",
		0xAE73: "\u5E60",
		0xAE74: "\u5E67",
		0xAE75: "\u5E6A",
		0xAE76: "\u5E6B",
		0xAE77: "\u5E6C",
		0xAE78: "\u5E6E",
		0xAE79: "\u5E70",
		0xAE7A: "\u5E71",
		0xAE7B: "\u5E80",
		0xAE7C: "\u5E86",
		0xAE7D: "\u5E8B",
		0xAE7E: "\u5E90",
		0xAF21: "\u5E91",
		0xAF22: "\u5E93",
		0xAF23: "\u5E94",
		0xAF24: "\u5E99",
		0xAF25: "\u5E9E",
		0xAF26: "\u5E9F",
		0xAF27: "\u5EA4",
		0xAF28: "\u5EA5",
		0xAF29: "\u5EAA",
		0xAF2A: "\u5EAF",
		0xAF2B: "\u5EB3",
		0xAF2C: "\u5EB9",
		0xAF2D: "\u5EBC",
		0xAF2E: "\u5EBD",
		0xAF2F: "\u5EBE",
		0xAF30: "\u5EC4",
		0xAF31: "\u5EC6",
		0xAF32: "\u5ECB",
		0xAF33: "\u5ED1",
		0xAF34: "\u5ED2",
		0xAF35: "\u5ED5",
		0xAF36: "\u5ED9",
		0xAF37: "\u5EEA",
		0xAF38: "\u5EF5",
		0xAF39: "\u5EF9",
		0xAF3A: "\u5F00",
		0xAF3B: "\u5F02",
		0xAF3C: "\u5F06",
		0xAF3D: "\u5F07",
		0xAF3E: "\u5F08",
		0xAF3F: "\u5F0E",
		0xAF40: "\u5F1D",
		0xAF41: "\u5F20",
		0xAF42: "\u5F22",
		0xAF43: "\u5F28",
		0xAF44: "\u5F2A",
		0xAF45: "\u5F2E",
		0xAF46: "\u5F33",
		0xAF47: "\u5F36",
		0xAF48: "\u5F39",
		0xAF49: "\u5F3A",
		0xAF4A: "\u5F3B",
		0xAF4B: "\u5F40",
		0xAF4C: "\u5F44",
		0xAF4D: "\u5F46",
		0xAF4E: "\u5F49",
		0xAF4F: "\u5F4D",
		0xAF50: "\u5F50",
		0xAF51: "\u5F52",
		0xAF52: "\u5F55",
		0xAF53: "\u5F58",
		0xAF54: "\u5F5F",
		0xAF55: "\u5F60",
		0xAF56: "\u5F64",
		0xAF57: "\u5F67",
		0xAF58: "\u5F7B",
		0xAF59: "\u5F89",
		0xAF5A: "\u5F95",
		0xAF5B: "\u5F9B",
		0xAF5C: "\u5F9C",
		0xAF5D: "\u5FA7",
		0xAF5E: "\u5FAC",
		0xAF5F: "\u5FAF",
		0xAF60: "\u5FB5",
		0xAF61: "\u5FB7",
		0xAF62: "\u5FC4",
		0xAF63: "\u5FC6",
		0xAF64: "\u5FC9",
		0xAF65: "\u5FCF",
		0xAF66: "\u5FD0",
		0xAF67: "\u5FD1",
		0xAF68: "\u5FD2",
		0xAF69: "\u5FD3",
		0xAF6A: "\u5FDE",
		0xAF6B: "\u5FE1",
		0xAF6C: "\u5FE3",
		0xAF6D: "\u5FE7",
		0xAF6E: "\u5FE8",
		0xAF6F: "\u5FEA",
		0xAF70: "\u5FED",
		0xAF71: "\u5FEE",
		0xAF72: "\u5FF3",
		0xAF73: "\u5FF7",
		0xAF74: "\u5FFA",
		0xAF75: "\u5FFC",
		0xAF76: "\u5FFE",
		0xAF77: "\u6000",
		0xAF78: "\u6001",
		0xAF79: "\u6002",
		0xAF7A: "\u6003",
		0xAF7B: "\u6004",
		0xAF7C: "\u6005",
		0xAF7D: "\u6006",
		0xAF7E: "\u600A",
		0xB021: "\u600D",
		0xB022: "\u6014",
		0xB023: "\u6017",
		0xB024: "\u6033",
		0xB025: "\u6035",
		0xB026: "\u6039",
		0xB027: "\u603B",
		0xB028: "\u603C",
		0xB029: "\u603F",
		0xB02A: "\u6047",
		0xB02B: "\u6049",
		0xB02C: "\u6053",
		0xB02D: "\u605D",
		0xB02E: "\u6067",
		0xB02F: "\u6073",
		0xB030: "\u6076",
		0xB031: "\u6078",
		0xB032: "\u6079",
		0xB033: "\u607A",
		0xB034: "\u607B",
		0xB035: "\u607C",
		0xB036: "\u607D",
		0xB037: "\u607F",
		0xB038: "\u6082",
		0xB039: "\u6095",
		0xB03A: "\u609D",
		0xB03B: "\u609E",
		0xB03C: "\u60A2",
		0xB03D: "\u60A4",
		0xB03E: "\u60A8",
		0xB03F: "\u60AB",
		0xB040: "\u60AC",
		0xB041: "\u60AD",
		0xB042: "\u60AF",
		0xB043: "\u60B0",
		0xB044: "\u60B1",
		0xB045: "\u60BB",
		0xB046: "\u60BE",
		0xB047: "\u60C4",
		0xB048: "\u60CA",
		0xB049: "\u60CB",
		0xB04A: "\u60CE",
		0xB04B: "\u60D5",
		0xB04C: "\u60D9",
		0xB04D: "\u60DB",
		0xB04E: "\u60DD",
		0xB04F: "\u60E6",
		0xB050: "\u60E9",
		0xB051: "\u60EA",
		0xB052: "\u60EB",
		0xB053: "\u60EC",
		0xB054: "\u60ED",
		0xB055: "\u60EE",
		0xB056: "\u60EF",
		0xB057: "\u60F2",
		0xB058: "\u60F5",
		0xB059: "\u60F8",
		0xB05A: "\u60FC",
		0xB05B: "\u60FD",
		0xB05C: "\u610A",
		0xB05D: "\u6110",
		0xB05E: "\u6112",
		0xB05F: "\u6114",
		0xB060: "\u6119",
		0xB061: "\u611C",
		0xB062: "\u6120",
		0xB063: "\u6123",
		0xB064: "\u6124",
		0xB065: "\u6126",
		0xB066: "\u612B",
		0xB067: "\u6137",
		0xB068: "\u6141",
		0xB069: "\u6151",
		0xB06A: "\u615C",
		0xB06B: "\u6160",
		0xB06C: "\u6164",
		0xB06D: "\u616A",
		0xB06E: "\u616D",
		0xB06F: "\u617C",
		0xB070: "\u6183",
		0xB071: "\u618B",
		0xB072: "\u6193",
		0xB073: "\u619D",
		0xB074: "\u61A8",
		0xB075: "\u61AD",
		0xB076: "\u61AF",
		0xB077: "\u61B1",
		0xB078: "\u61B7",
		0xB079: "\u61B9",
		0xB07A: "\u61C1",
		0xB07B: "\u61C2",
		0xB07C: "\u61D1",
		0xB07D: "\u61D2",
		0xB07E: "\u61D4",
		0xB121: "\u61D5",
		0xB122: "\u61DC",
		0xB123: "\u61DF",
		0xB124: "\u61E4",
		0xB125: "\u61E8",
		0xB126: "\u61F5",
		0xB127: "\u6204",
		0xB128: "\u6206",
		0xB129: "\u620B",
		0xB12A: "\u620F",
		0xB12B: "\u6215",
		0xB12C: "\u6217",
		0xB12D: "\u6218",
		0xB12E: "\u6222",
		0xB12F: "\u6223",
		0xB130: "\u6224",
		0xB131: "\u6225",
		0xB132: "\u6227",
		0xB133: "\u622C",
		0xB134: "\u6237",
		0xB135: "\u6239",
		0xB136: "\u623D",
		0xB137: "\u623E",
		0xB138: "\u6243",
		0xB139: "\u6245",
		0xB13A: "\u6246",
		0xB13B: "\u624A",
		0xB13C: "\u624C",
		0xB13D: "\u6251",
		0xB13E: "\u6252",
		0xB13F: "\u6254",
		0xB140: "\u6264",
		0xB141: "\u6266",
		0xB142: "\u6267",
		0xB143: "\u6269",
		0xB144: "\u626A",
		0xB145: "\u626B",
		0xB146: "\u626C",
		0xB147: "\u626D",
		0xB148: "\u626F",
		0xB149: "\u6270",
		0xB14A: "\u6273",
		0xB14B: "\u627A",
		0xB14C: "\u627D",
		0xB14D: "\u6286",
		0xB14E: "\u6288",
		0xB14F: "\u628F",
		0xB150: "\u629A",
		0xB151: "\u629D",
		0xB152: "\u629F",
		0xB153: "\u62A0",
		0xB154: "\u62A1",
		0xB155: "\u62A2",
		0xB156: "\u62A4",
		0xB157: "\u62A5",
		0xB158: "\u62A8",
		0xB159: "\u62B4",
		0xB15A: "\u62B6",
		0xB15B: "\u62BF",
		0xB15C: "\u62C3",
		0xB15D: "\u62C4",
		0xB15E: "\u62CE",
		0xB15F: "\u62D5",
		0xB160: "\u62D6",
		0xB161: "\u62DA",
		0xB162: "\u62DF",
		0xB163: "\u62E2",
		0xB164: "\u62E3",
		0xB165: "\u62E4",
		0xB166: "\u62E5",
		0xB167: "\u62E6",
		0xB168: "\u62E7",
		0xB169: "\u62E8",
		0xB16A: "\u62E9",
		0xB16B: "\u62F4",
		0xB16C: "\u62FC",
		0xB16D: "\u62FD",
		0xB16E: "\u630E",
		0xB16F: "\u6310",
		0xB170: "\u6313",
		0xB171: "\u6316",
		0xB172: "\u631A",
		0xB173: "\u631B",
		0xB174: "\u631C",
		0xB175: "\u631D",
		0xB176: "\u631E",
		0xB177: "\u6320",
		0xB178: "\u6321",
		0xB179: "\u6322",
		0xB17A: "\u6323",
		0xB17B: "\u6324",
		0xB17C: "\u6325",
		0xB17D: "\u6326",
		0xB17E: "\u632A",
		0xB221: "\u6331",
		0xB222: "\u6332",
		0xB223: "\u6335",
		0xB224: "\u6336",
		0xB225: "\u6339",
		0xB226: "\u633C",
		0xB227: "\u6342",
		0xB228: "\u6343",
		0xB229: "\u6344",
		0xB22A: "\u6345",
		0xB22B: "\u6346",
		0xB22C: "\u634B",
		0xB22D: "\u634E",
		0xB22E: "\u6358",
		0xB22F: "\u635D",
		0xB230: "\u635E",
		0xB231: "\u635F",
		0xB232: "\u6361",
		0xB233: "\u6362",
		0xB234: "\u6363",
		0xB235: "\u6365",
		0xB236: "\u6366",
		0xB237: "\u636D",
		0xB238: "\u636F",
		0xB239: "\u6371",
		0xB23A: "\u6374",
		0xB23B: "\u6375",
		0xB23C: "\u637D",
		0xB23D: "\u6382",
		0xB23E: "\u6384",
		0xB23F: "\u6386",
		0xB240: "\u6387",
		0xB241: "\u638A",
		0xB242: "\u6390",
		0xB243: "\u6397",
		0xB244: "\u639E",
		0xB245: "\u63AD",
		0xB246: "\u63AE",
		0xB247: "\u63AF",
		0xB248: "\u63B0",
		0xB249: "\u63B1",
		0xB24A: "\u63B3",
		0xB24B: "\u63B7",
		0xB24C: "\u63B8",
		0xB24D: "\u63BA",
		0xB24E: "\u63BC",
		0xB24F: "\u63BD",
		0xB250: "\u63C5",
		0xB251: "\u63C8",
		0xB252: "\u63CC",
		0xB253: "\u63CD",
		0xB254: "\u63CE",
		0xB255: "\u63D1",
		0xB256: "\u63D5",
		0xB257: "\u63DC",
		0xB258: "\u63DE",
		0xB259: "\u63E0",
		0xB25A: "\u63E2",
		0xB25B: "\u63E6",
		0xB25C: "\u63EA",
		0xB25D: "\u63ED",
		0xB25E: "\u63F2",
		0xB25F: "\u63F3",
		0xB260: "\u63F8",
		0xB261: "\u63F9",
		0xB262: "\u63FB",
		0xB263: "\u63FD",
		0xB264: "\u63FE",
		0xB265: "\u63FF",
		0xB266: "\u6400",
		0xB267: "\u6401",
		0xB268: "\u6402",
		0xB269: "\u6405",
		0xB26A: "\u6407",
		0xB26B: "\u6409",
		0xB26C: "\u640A",
		0xB26D: "\u640B",
		0xB26E: "\u640C",
		0xB26F: "\u6410",
		0xB270: "\u6412",
		0xB271: "\u6414",
		0xB272: "\u6415",
		0xB273: "\u641B",
		0xB274: "\u641E",
		0xB275: "\u6420",
		0xB276: "\u6421",
		0xB277: "\u6422",
		0xB278: "\u6424",
		0xB279: "\u6425",
		0xB27A: "\u6427",
		0xB27B: "\u6429",
		0xB27C: "\u642A",
		0xB27D: "\u642F",
		0xB27E: "\u6430",
		0xB321: "\u6433",
		0xB322: "\u643D",
		0xB323: "\u643F",
		0xB324: "\u6440",
		0xB325: "\u6441",
		0xB326: "\u6444",
		0xB327: "\u6445",
		0xB328: "\u6446",
		0xB329: "\u6447",
		0xB32A: "\u6448",
		0xB32B: "\u644A",
		0xB32C: "\u644B",
		0xB32D: "\u6450",
		0xB32E: "\u6451",
		0xB32F: "\u6452",
		0xB330: "\u6454",
		0xB331: "\u645E",
		0xB332: "\u645F",
		0xB333: "\u6460",
		0xB334: "\u6463",
		0xB335: "\u646D",
		0xB336: "\u6473",
		0xB337: "\u6474",
		0xB338: "\u6479",
		0xB339: "\u647B",
		0xB33A: "\u647D",
		0xB33B: "\u6482",
		0xB33C: "\u6484",
		0xB33D: "\u6485",
		0xB33E: "\u6487",
		0xB33F: "\u6489",
		0xB340: "\u648F",
		0xB341: "\u6490",
		0xB342: "\u6491",
		0xB343: "\u6496",
		0xB344: "\u6499",
		0xB345: "\u649D",
		0xB346: "\u649F",
		0xB347: "\u64A2",
		0xB348: "\u64A3",
		0xB349: "\u64A6",
		0xB34A: "\u64A7",
		0xB34B: "\u64AC",
		0xB34C: "\u64B4",
		0xB34D: "\u64B5",
		0xB34E: "\u64B7",
		0xB34F: "\u64B8",
		0xB350: "\u64BA",
		0xB351: "\u64BD",
		0xB352: "\u64BE",
		0xB353: "\u64BF",
		0xB354: "\u64C0",
		0xB355: "\u64C4",
		0xB356: "\u64CA",
		0xB357: "\u64CB",
		0xB358: "\u64CE",
		0xB359: "\u64D0",
		0xB35A: "\u64D3",
		0xB35B: "\u64D5",
		0xB35C: "\u64D6",
		0xB35D: "\u64D7",
		0xB35E: "\u64DE",
		0xB35F: "\u64E4",
		0xB360: "\u64E9",
		0xB361: "\u64F0",
		0xB362: "\u64FB",
		0xB363: "\u64FF",
		0xB364: "\u6504",
		0xB365: "\u6509",
		0xB366: "\u650F",
		0xB367: "\u6512",
		0xB368: "\u6514",
		0xB369: "\u6519",
		0xB36A: "\u651B",
		0xB36B: "\u651F",
		0xB36C: "\u6525",
		0xB36D: "\u6529",
		0xB36E: "\u652E",
		0xB36F: "\u6532",
		0xB370: "\u653D",
		0xB371: "\u6541",
		0xB372: "\u6542",
		0xB373: "\u6543",
		0xB374: "\u6549",
		0xB375: "\u654C",
		0xB376: "\u6554",
		0xB377: "\u655A",
		0xB378: "\u655B",
		0xB379: "\u6560",
		0xB37A: "\u656B",
		0xB37B: "\u656D",
		0xB37C: "\u657A",
		0xB37D: "\u6585",
		0xB37E: "\u658B",
		0xB421: "\u6592",
		0xB422: "\u6593",
		0xB423: "\u659A",
		0xB424: "\u659D",
		0xB425: "\u65A0",
		0xB426: "\u65A2",
		0xB427: "\u65A8",
		0xB428: "\u65A9",
		0xB429: "\u65AE",
		0xB42A: "\u65B2",
		0xB42B: "\u65B6",
		0xB42C: "\u65B8",
		0xB42D: "\u65BE",
		0xB42E: "\u65BF",
		0xB42F: "\u65C2",
		0xB430: "\u65CE",
		0xB431: "\u65D0",
		0xB432: "\u65D6",
		0xB433: "\u65DF",
		0xB434: "\u65EE",
		0xB435: "\u65EF",
		0xB436: "\u65F0",
		0xB437: "\u65F6",
		0xB438: "\u65F7",
		0xB439: "\u65F8",
		0xB43A: "\u65F9",
		0xB43B: "\u65FE",
		0xB43C: "\u6600",
		0xB43D: "\u6609",
		0xB43E: "\u6610",
		0xB43F: "\u6615",
		0xB440: "\u6619",
		0xB441: "\u661A",
		0xB442: "\u661D",
		0xB443: "\u661E",
		0xB444: "\u6621",
		0xB445: "\u662A",
		0xB446: "\u662B",
		0xB447: "\u6630",
		0xB448: "\u6631",
		0xB449: "\u6633",
		0xB44A: "\u663A",
		0xB44B: "\u663D",
		0xB44C: "\u663E",
		0xB44D: "\u6645",
		0xB44E: "\u664C",
		0xB44F: "\u6653",
		0xB450: "\u6654",
		0xB451: "\u6655",
		0xB452: "\u6656",
		0xB453: "\u6657",
		0xB454: "\u665A",
		0xB455: "\u6661",
		0xB456: "\u6663",
		0xB457: "\u666C",
		0xB458: "\u6673",
		0xB459: "\u6677",
		0xB45A: "\u6679",
		0xB45B: "\u667B",
		0xB45C: "\u667E",
		0xB45D: "\u6682",
		0xB45E: "\u6685",
		0xB45F: "\u668B",
		0xB460: "\u668C",
		0xB461: "\u668D",
		0xB462: "\u6690",
		0xB463: "\u6695",
		0xB464: "\u66A0",
		0xB465: "\u66A7",
		0xB466: "\u66A8",
		0xB467: "\u66B1",
		0xB468: "\u66B5",
		0xB469: "\u66C0",
		0xB46A: "\u66C6",
		0xB46B: "\u66C8",
		0xB46C: "\u66CC",
		0xB46D: "\u66CF",
		0xB46E: "\u66DB",
		0xB46F: "\u66E1",
		0xB470: "\u66E8",
		0xB471: "\u66EC",
		0xB472: "\u66F6",
		0xB473: "\u6705",
		0xB474: "\u6706",
		0xB475: "\u670A",
		0xB476: "\u6710",
		0xB477: "\u6712",
		0xB478: "\u6713",
		0xB479: "\u6718",
		0xB47A: "\u6723",
		0xB47B: "\u672F",
		0xB47C: "\u6733",
		0xB47D: "\u6735",
		0xB47E: "\u6740",
		0xB521: "\u6742",
		0xB522: "\u6743",
		0xB523: "\u6747",
		0xB524: "\u6748",
		0xB525: "\u674C",
		0xB526: "\u6755",
		0xB527: "\u675D",
		0xB528: "\u6767",
		0xB529: "\u6768",
		0xB52A: "\u6769",
		0xB52B: "\u6774",
		0xB52C: "\u677B",
		0xB52D: "\u6781",
		0xB52E: "\u6784",
		0xB52F: "\u678A",
		0xB530: "\u6791",
		0xB531: "\u6792",
		0xB532: "\u6793",
		0xB533: "\u6798",
		0xB534: "\u679E",
		0xB535: "\u67A3",
		0xB536: "\u67A5",
		0xB537: "\u67A7",
		0xB538: "\u67A8",
		0xB539: "\u67AA",
		0xB53A: "\u67AB",
		0xB53B: "\u67AD",
		0xB53C: "\u67B0",
		0xB53D: "\u67B1",
		0xB53E: "\u67B2",
		0xB53F: "\u67B5",
		0xB540: "\u67BB",
		0xB541: "\u67C3",
		0xB542: "\u67C8",
		0xB543: "\u67D2",
		0xB544: "\u67D9",
		0xB545: "\u67DC",
		0xB546: "\u67DF",
		0xB547: "\u67E0",
		0xB548: "\u67E5",
		0xB549: "\u67F0",
		0xB54A: "\u67F7",
		0xB54B: "\u67F9",
		0xB54C: "\u67FA",
		0xB54D: "\u67FD",
		0xB54E: "\u6800",
		0xB54F: "\u6805",
		0xB550: "\u6807",
		0xB551: "\u6808",
		0xB552: "\u6809",
		0xB553: "\u680A",
		0xB554: "\u680B",
		0xB555: "\u680C",
		0xB556: "\u680E",
		0xB557: "\u680F",
		0xB558: "\u6811",
		0xB559: "\u6812",
		0xB55A: "\u6814",
		0xB55B: "\u6818",
		0xB55C: "\u681D",
		0xB55D: "\u681F",
		0xB55E: "\u6827",
		0xB55F: "\u6830",
		0xB560: "\u6831",
		0xB561: "\u6833",
		0xB562: "\u6837",
		0xB563: "\u683B",
		0xB564: "\u683E",
		0xB565: "\u683F",
		0xB566: "\u6844",
		0xB567: "\u6845",
		0xB568: "\u6849",
		0xB569: "\u684A",
		0xB56A: "\u684C",
		0xB56B: "\u6855",
		0xB56C: "\u685A",
		0xB56D: "\u685B",
		0xB56E: "\u6860",
		0xB56F: "\u6861",
		0xB570: "\u6862",
		0xB571: "\u6864",
		0xB572: "\u6865",
		0xB573: "\u6866",
		0xB574: "\u6868",
		0xB575: "\u6869",
		0xB576: "\u686B",
		0xB577: "\u686E",
		0xB578: "\u686F",
		0xB579: "\u6872",
		0xB57A: "\u6879",
		0xB57B: "\u687C",
		0xB57C: "\u6886",
		0xB57D: "\u6890",
		0xB57E: "\u6898",
		0xB621: "\u689C",
		0xB622: "\u68A3",
		0xB623: "\u68A9",
		0xB624: "\u68AB",
		0xB625: "\u68AE",
		0xB626: "\u68B2",
		0xB627: "\u68BD",
		0xB628: "\u68BE",
		0xB629: "\u68BF",
		0xB62A: "\u68C0",
		0xB62B: "\u68C1",
		0xB62C: "\u68C2",
		0xB62D: "\u68C3",
		0xB62E: "\u68D0",
		0xB62F: "\u68D1",
		0xB630: "\u68D3",
		0xB631: "\u68D6",
		0xB632: "\u68E8",
		0xB633: "\u68EA",
		0xB634: "\u68EB",
		0xB635: "\u68EC",
		0xB636: "\u68F0",
		0xB637: "\u68F1",
		0xB638: "\u68F5",
		0xB639: "\u68F6",
		0xB63A: "\u68FB",
		0xB63B: "\u68FC",
		0xB63C: "\u68FD",
		0xB63D: "\u6907",
		0xB63E: "\u6909",
		0xB63F: "\u6910",
		0xB640: "\u6911",
		0xB641: "\u6913",
		0xB642: "\u6917",
		0xB643: "\u691F",
		0xB644: "\u6920",
		0xB645: "\u6924",
		0xB646: "\u692D",
		0xB647: "\u6933",
		0xB648: "\u6938",
		0xB649: "\u693E",
		0xB64A: "\u6942",
		0xB64B: "\u6952",
		0xB64C: "\u6957",
		0xB64D: "\u695B",
		0xB64E: "\u6963",
		0xB64F: "\u6964",
		0xB650: "\u6965",
		0xB651: "\u6966",
		0xB652: "\u6969",
		0xB653: "\u696C",
		0xB654: "\u6980",
		0xB655: "\u6983",
		0xB656: "\u6984",
		0xB657: "\u6985",
		0xB658: "\u6986",
		0xB659: "\u6987",
		0xB65A: "\u6988",
		0xB65B: "\u6989",
		0xB65C: "\u698D",
		0xB65D: "\u6996",
		0xB65E: "\u6998",
		0xB65F: "\u69A4",
		0xB660: "\u69A5",
		0xB661: "\u69A6",
		0xB662: "\u69A8",
		0xB663: "\u69AB",
		0xB664: "\u69AD",
		0xB665: "\u69B0",
		0xB666: "\u69B7",
		0xB667: "\u69BC",
		0xB668: "\u69C0",
		0xB669: "\u69C5",
		0xB66A: "\u69D1",
		0xB66B: "\u69D4",
		0xB66C: "\u69D6",
		0xB66D: "\u69DA",
		0xB66E: "\u69DB",
		0xB66F: "\u69DC",
		0xB670: "\u69DF",
		0xB671: "\u69E0",
		0xB672: "\u69E5",
		0xB673: "\u69F3",
		0xB674: "\u69F7",
		0xB675: "\u69FA",
		0xB676: "\u69FC",
		0xB677: "\u69FE",
		0xB678: "\u6A00",
		0xB679: "\u6A01",
		0xB67A: "\u6A0F",
		0xB67B: "\u6A11",
		0xB67C: "\u6A18",
		0xB67D: "\u6A1D",
		0xB67E: "\u6A28",
		0xB721: "\u6A2F",
		0xB722: "\u6A31",
		0xB723: "\u6A37",
		0xB724: "\u6A3E",
		0xB725: "\u6A45",
		0xB726: "\u6A50",
		0xB727: "\u6A5B",
		0xB728: "\u6A5C",
		0xB729: "\u6A64",
		0xB72A: "\u6A65",
		0xB72B: "\u6A70",
		0xB72C: "\u6A71",
		0xB72D: "\u6A79",
		0xB72E: "\u6A7C",
		0xB72F: "\u6A81",
		0xB730: "\u6A83",
		0xB731: "\u6A87",
		0xB732: "\u6A89",
		0xB733: "\u6A91",
		0xB734: "\u6A94",
		0xB735: "\u6A9B",
		0xB736: "\u6A9E",
		0xB737: "\u6AA5",
		0xB738: "\u6AA9",
		0xB739: "\u6AAB",
		0xB73A: "\u6AAF",
		0xB73B: "\u6AB5",
		0xB73C: "\u6ABE",
		0xB73D: "\u6AC6",
		0xB73E: "\u6AC8",
		0xB73F: "\u6ADD",
		0xB740: "\u6AE5",
		0xB741: "\u6AEB",
		0xB742: "\u6AEC",
		0xB743: "\u6AF3",
		0xB744: "\u6AF8",
		0xB745: "\u6AFC",
		0xB746: "\u6AFD",
		0xB747: "\u6B02",
		0xB748: "\u6B0F",
		0xB749: "\u6B11",
		0xB74A: "\u6B13",
		0xB74B: "\u6B19",
		0xB74C: "\u6B1B",
		0xB74D: "\u6B1E",
		0xB74E: "\u6B22",
		0xB74F: "\u6B24",
		0xB750: "\u6B2C",
		0xB751: "\u6B35",
		0xB752: "\u6B3B",
		0xB753: "\u6B3F",
		0xB754: "\u6B45",
		0xB755: "\u6B46",
		0xB756: "\u6B48",
		0xB757: "\u6B57",
		0xB758: "\u6B58",
		0xB759: "\u6B5C",
		0xB75A: "\u6B60",
		0xB75B: "\u6B65",
		0xB75C: "\u6B67",
		0xB75D: "\u6B72",
		0xB75E: "\u6B77",
		0xB75F: "\u6B7C",
		0xB760: "\u6B81",
		0xB761: "\u6B82",
		0xB762: "\u6B87",
		0xB763: "\u6B92",
		0xB764: "\u6B93",
		0xB765: "\u6B9A",
		0xB766: "\u6B9B",
		0xB767: "\u6BA0",
		0xB768: "\u6BA1",
		0xB769: "\u6BA3",
		0xB76A: "\u6BAD",
		0xB76B: "\u6BAE",
		0xB76C: "\u6BBD",
		0xB76D: "\u6BC1",
		0xB76E: "\u6BC2",
		0xB76F: "\u6BC9",
		0xB770: "\u6BCC",
		0xB771: "\u6BCF",
		0xB772: "\u6BD0",
		0xB773: "\u6BD1",
		0xB774: "\u6BD5",
		0xB775: "\u6BD6",
		0xB776: "\u6BD7",
		0xB777: "\u6BD9",
		0xB778: "\u6BE1",
		0xB779: "\u6BE7",
		0xB77A: "\u6BE8",
		0xB77B: "\u6BEA",
		0xB77C: "\u6BF0",
		0xB77D: "\u6BF5",
		0xB77E: "\u6BF7",
		0xB821: "\u6BF8",
		0xB822: "\u6BF9",
		0xB823: "\u6BFD",
		0xB824: "\u6BFF",
		0xB825: "\u6C02",
		0xB826: "\u6C04",
		0xB827: "\u6C05",
		0xB828: "\u6C06",
		0xB829: "\u6C07",
		0xB82A: "\u6C09",
		0xB82B: "\u6C0A",
		0xB82C: "\u6C0D",
		0xB82D: "\u6C10",
		0xB82E: "\u6C15",
		0xB82F: "\u6C16",
		0xB830: "\u6C18",
		0xB831: "\u6C19",
		0xB832: "\u6C1A",
		0xB833: "\u6C1F",
		0xB834: "\u6C21",
		0xB835: "\u6C22",
		0xB836: "\u6C25",
		0xB837: "\u6C26",
		0xB838: "\u6C27",
		0xB839: "\u6C28",
		0xB83A: "\u6C29",
		0xB83B: "\u6C2A",
		0xB83C: "\u6C2B",
		0xB83D: "\u6C2C",
		0xB83E: "\u6C2E",
		0xB83F: "\u6C2F",
		0xB840: "\u6C30",
		0xB841: "\u6C32",
		0xB842: "\u6C35",
		0xB843: "\u6C39",
		0xB844: "\u6C3D",
		0xB845: "\u6C3F",
		0xB846: "\u6C46",
		0xB847: "\u6C47",
		0xB848: "\u6C48",
		0xB849: "\u6C49",
		0xB84A: "\u6C4A",
		0xB84B: "\u6C4D",
		0xB84C: "\u6C4F",
		0xB84D: "\u6C54",
		0xB84E: "\u6C59",
		0xB84F: "\u6C5B",
		0xB850: "\u6C5C",
		0xB851: "\u6C61",
		0xB852: "\u6C64",
		0xB853: "\u6C67",
		0xB854: "\u6C69",
		0xB855: "\u6C6B",
		0xB856: "\u6C6D",
		0xB857: "\u6C74",
		0xB858: "\u6C76",
		0xB859: "\u6C79",
		0xB85A: "\u6C84",
		0xB85B: "\u6C85",
		0xB85C: "\u6C86",
		0xB85D: "\u6C89",
		0xB85E: "\u6C8F",
		0xB85F: "\u6C94",
		0xB860: "\u6C98",
		0xB861: "\u6C9F",
		0xB862: "\u6CA3",
		0xB863: "\u6CA4",
		0xB864: "\u6CA5",
		0xB865: "\u6CA6",
		0xB866: "\u6CA7",
		0xB867: "\u6CA8",
		0xB868: "\u6CA9",
		0xB869: "\u6CAA",
		0xB86A: "\u6CAC",
		0xB86B: "\u6CAD",
		0xB86C: "\u6CB4",
		0xB86D: "\u6CB5",
		0xB86E: "\u6CC2",
		0xB86F: "\u6CC3",
		0xB870: "\u6CC6",
		0xB871: "\u6CD0",
		0xB872: "\u6CD4",
		0xB873: "\u6CD6",
		0xB874: "\u6CDA",
		0xB875: "\u6CDC",
		0xB876: "\u6CDE",
		0xB877: "\u6CE0",
		0xB878: "\u6CEB",
		0xB879: "\u6CEE",
		0xB87A: "\u6CF5",
		0xB87B: "\u6CF7",
		0xB87C: "\u6CF8",
		0xB87D: "\u6CFA",
		0xB87E: "\u6CFB",
		0xB921: "\u6CFC",
		0xB922: "\u6CFD",
		0xB923: "\u6CFE",
		0xB924: "\u6D01",
		0xB925: "\u6D04",
		0xB926: "\u6D07",
		0xB927: "\u6D08",
		0xB928: "\u6D0E",
		0xB929: "\u6D0F",
		0xB92A: "\u6D11",
		0xB92B: "\u6D1A",
		0xB92C: "\u6D22",
		0xB92D: "\u6D23",
		0xB92E: "\u6D27",
		0xB92F: "\u6D28",
		0xB930: "\u6D2D",
		0xB931: "\u6D2E",
		0xB932: "\u6D31",
		0xB933: "\u6D34",
		0xB934: "\u6D39",
		0xB935: "\u6D3A",
		0xB936: "\u6D3C",
		0xB937: "\u6D3F",
		0xB938: "\u6D43",
		0xB939: "\u6D46",
		0xB93A: "\u6D47",
		0xB93B: "\u6D48",
		0xB93C: "\u6D49",
		0xB93D: "\u6D4A",
		0xB93E: "\u6D4B",
		0xB93F: "\u6D4D",
		0xB940: "\u6D4E",
		0xB941: "\u6D4F",
		0xB942: "\u6D50",
		0xB943: "\u6D51",
		0xB944: "\u6D52",
		0xB945: "\u6D53",
		0xB946: "\u6D54",
		0xB947: "\u6D55",
		0xB948: "\u6D5B",
		0xB949: "\u6D5E",
		0xB94A: "\u6D60",
		0xB94B: "\u6D61",
		0xB94C: "\u6D65",
		0xB94D: "\u6D6F",
		0xB94E: "\u6D7C",
		0xB94F: "\u6D82",
		0xB950: "\u6D87",
		0xB951: "\u6D89",
		0xB952: "\u6D8A",
		0xB953: "\u6D90",
		0xB954: "\u6D91",
		0xB955: "\u6D94",
		0xB956: "\u6D96",
		0xB957: "\u6D98",
		0xB958: "\u6D9D",
		0xB959: "\u6D9E",
		0xB95A: "\u6D9F",
		0xB95B: "\u6DA0",
		0xB95C: "\u6DA1",
		0xB95D: "\u6DA2",
		0xB95E: "\u6DA3",
		0xB95F: "\u6DA4",
		0xB960: "\u6DA6",
		0xB961: "\u6DA7",
		0xB962: "\u6DA8",
		0xB963: "\u6DA9",
		0xB964: "\u6DAA",
		0xB965: "\u6DAB",
		0xB966: "\u6DAE",
		0xB967: "\u6DB4",
		0xB968: "\u6DBF",
		0xB969: "\u6DC4",
		0xB96A: "\u6DCA",
		0xB96B: "\u6DCF",
		0xB96C: "\u6DD6",
		0xB96D: "\u6DDA",
		0xB96E: "\u6DDB",
		0xB96F: "\u6DDD",
		0xB970: "\u6DDF",
		0xB971: "\u6DE0",
		0xB972: "\u6DE9",
		0xB973: "\u6DEF",
		0xB974: "\u6DF4",
		0xB975: "\u6DF6",
		0xB976: "\u6DFC",
		0xB977: "\u6E0C",
		0xB978: "\u6E0D",
		0xB979: "\u6E0E",
		0xB97A: "\u6E10",
		0xB97B: "\u6E11",
		0xB97C: "\u6E14",
		0xB97D: "\u6E17",
		0xB97E: "\u6E32",
		0xBA21: "\u6E34",
		0xBA22: "\u6E3C",
		0xBA23: "\u6E44",
		0xBA24: "\u6E49",
		0xBA25: "\u6E4B",
		0xBA26: "\u6E4C",
		0xBA27: "\u6E51",
		0xBA28: "\u6E53",
		0xBA29: "\u6E54",
		0xBA2A: "\u6E5C",
		0xBA2B: "\u6E5D",
		0xBA2C: "\u6E62",
		0xBA2D: "\u6E63",
		0xBA2E: "\u6E66",
		0xBA2F: "\u6E68",
		0xBA30: "\u6E74",
		0xBA31: "\u6E81",
		0xBA32: "\u6E83",
		0xBA33: "\u6E85",
		0xBA34: "\u6E86",
		0xBA35: "\u6E87",
		0xBA36: "\u6E88",
		0xBA37: "\u6E89",
		0xBA38: "\u6E8D",
		0xBA39: "\u6E9A",
		0xBA3A: "\u6EA0",
		0xBA3B: "\u6EA6",
		0xBA3C: "\u6EA7",
		0xBA3D: "\u6EAE",
		0xBA3E: "\u6EB1",
		0xBA3F: "\u6EB4",
		0xBA40: "\u6EB5",
		0xBA41: "\u6EBB",
		0xBA42: "\u6EBC",
		0xBA43: "\u6EC0",
		0xBA44: "\u6EC1",
		0xBA45: "\u6EC3",
		0xBA46: "\u6EC6",
		0xBA47: "\u6EC7",
		0xBA48: "\u6EC8",
		0xBA49: "\u6ECD",
		0xBA4A: "\u6ECE",
		0xBA4B: "\u6ECF",
		0xBA4C: "\u6ED7",
		0xBA4D: "\u6ED8",
		0xBA4E: "\u6ED9",
		0xBA4F: "\u6EDA",
		0xBA50: "\u6EDF",
		0xBA51: "\u6EE0",
		0xBA52: "\u6EE1",
		0xBA53: "\u6EE2",
		0xBA54: "\u6EE4",
		0xBA55: "\u6EE5",
		0xBA56: "\u6EE6",
		0xBA57: "\u6EE8",
		0xBA58: "\u6EE9",
		0xBA59: "\u6EEA",
		0xBA5A: "\u6EEB",
		0xBA5B: "\u6EEE",
		0xBA5C: "\u6EF9",
		0xBA5D: "\u6EFB",
		0xBA5E: "\u6F08",
		0xBA5F: "\u6F0A",
		0xBA60: "\u6F0D",
		0xBA61: "\u6F16",
		0xBA62: "\u6F18",
		0xBA63: "\u6F19",
		0xBA64: "\u6F1A",
		0xBA65: "\u6F24",
		0xBA66: "\u6F26",
		0xBA67: "\u6F29",
		0xBA68: "\u6F2A",
		0xBA69: "\u6F2D",
		0xBA6A: "\u6F2F",
		0xBA6B: "\u6F33",
		0xBA6C: "\u6F36",
		0xBA6D: "\u6F37",
		0xBA6E: "\u6F3B",
		0xBA6F: "\u6F3C",
		0xBA70: "\u6F46",
		0xBA71: "\u6F47",
		0xBA72: "\u6F4B",
		0xBA73: "\u6F4D",
		0xBA74: "\u6F4F",
		0xBA75: "\u6F51",
		0xBA76: "\u6F53",
		0xBA77: "\u6F55",
		0xBA78: "\u6F56",
		0xBA79: "\u6F59",
		0xBA7A: "\u6F5E",
		0xBA7B: "\u6F60",
		0xBA7C: "\u6F62",
		0xBA7D: "\u6F69",
		0xBA7E: "\u6F72",
		0xBB21: "\u6F75",
		0xBB22: "\u6F77",
		0xBB23: "\u6F7D",
		0xBB24: "\u6F7E",
		0xBB25: "\u6F7F",
		0xBB26: "\u6F87",
		0xBB27: "\u6F88",
		0xBB28: "\u6F89",
		0xBB29: "\u6F8C",
		0xBB2A: "\u6F8D",
		0xBB2B: "\u6F90",
		0xBB2C: "\u6F93",
		0xBB2D: "\u6F94",
		0xBB2E: "\u6F98",
		0xBB2F: "\u6F9C",
		0xBB30: "\u6FA0",
		0xBB31: "\u6FA5",
		0xBB32: "\u6FA7",
		0xBB33: "\u6FA8",
		0xBB34: "\u6FAB",
		0xBB35: "\u6FAE",
		0xBB36: "\u6FB4",
		0xBB37: "\u6FB6",
		0xBB38: "\u6FBC",
		0xBB39: "\u6FBD",
		0xBB3A: "\u6FBE",
		0xBB3B: "\u6FC7",
		0xBB3C: "\u6FC9",
		0xBB3D: "\u6FD1",
		0xBB3E: "\u6FD2",
		0xBB3F: "\u6FDA",
		0xBB40: "\u6FDC",
		0xBB41: "\u6FDE",
		0xBB42: "\u6FF0",
		0xBB43: "\u6FFC",
		0xBB44: "\u7005",
		0xBB45: "\u7006",
		0xBB46: "\u700C",
		0xBB47: "\u700D",
		0xBB48: "\u7014",
		0xBB49: "\u7020",
		0xBB4A: "\u7023",
		0xBB4B: "\u7035",
		0xBB4C: "\u7039",
		0xBB4D: "\u703C",
		0xBB4E: "\u703D",
		0xBB4F: "\u7043",
		0xBB50: "\u7044",
		0xBB51: "\u704A",
		0xBB52: "\u704B",
		0xBB53: "\u704F",
		0xBB54: "\u7055",
		0xBB55: "\u705E",
		0xBB56: "\u7060",
		0xBB57: "\u7064",
		0xBB58: "\u7067",
		0xBB59: "\u7068",
		0xBB5A: "\u706D",
		0xBB5B: "\u7075",
		0xBB5C: "\u7076",
		0xBB5D: "\u707A",
		0xBB5E: "\u707E",
		0xBB5F: "\u707F",
		0xBB60: "\u7080",
		0xBB61: "\u7081",
		0xBB62: "\u7085",
		0xBB63: "\u7086",
		0xBB64: "\u7094",
		0xBB65: "\u7095",
		0xBB66: "\u7096",
		0xBB67: "\u7098",
		0xBB68: "\u709C",
		0xBB69: "\u709D",
		0xBB6A: "\u709F",
		0xBB6B: "\u70A4",
		0xBB6C: "\u70A7",
		0xBB6D: "\u70AB",
		0xBB6E: "\u70B1",
		0xBB6F: "\u70B7",
		0xBB70: "\u70BB",
		0xBB71: "\u70BC",
		0xBB72: "\u70BD",
		0xBB73: "\u70C0",
		0xBB74: "\u70C1",
		0xBB75: "\u70C2",
		0xBB76: "\u70C3",
		0xBB77: "\u70CA",
		0xBB78: "\u70D4",
		0xBB79: "\u70D6",
		0xBB7A: "\u70D8",
		0xBB7B: "\u70DB",
		0xBB7C: "\u70DC",
		0xBB7D: "\u70E4",
		0xBB7E: "\u70E6",
		0xBC21: "\u70E7",
		0xBC22: "\u70E8",
		0xBC23: "\u70E9",
		0xBC24: "\u70EB",
		0xBC25: "\u70EC",
		0xBC26: "\u70ED",
		0xBC27: "\u70EF",
		0xBC28: "\u70F4",
		0xBC29: "\u70F7",
		0xBC2A: "\u70FA",
		0xBC2B: "\u7105",
		0xBC2C: "\u710A",
		0xBC2D: "\u710C",
		0xBC2E: "\u7110",
		0xBC2F: "\u7113",
		0xBC30: "\u7115",
		0xBC31: "\u7116",
		0xBC32: "\u7117",
		0xBC33: "\u7118",
		0xBC34: "\u7120",
		0xBC35: "\u712B",
		0xBC36: "\u712E",
		0xBC37: "\u712F",
		0xBC38: "\u7130",
		0xBC39: "\u7131",
		0xBC3A: "\u7145",
		0xBC3B: "\u7147",
		0xBC3C: "\u714A",
		0xBC3D: "\u714F",
		0xBC3E: "\u7150",
		0xBC3F: "\u7152",
		0xBC40: "\u7157",
		0xBC41: "\u715A",
		0xBC42: "\u715C",
		0xBC43: "\u715E",
		0xBC44: "\u7160",
		0xBC45: "\u7163",
		0xBC46: "\u7168",
		0xBC47: "\u7172",
		0xBC48: "\u7173",
		0xBC49: "\u7174",
		0xBC4A: "\u7178",
		0xBC4B: "\u717A",
		0xBC4C: "\u7187",
		0xBC4D: "\u7192",
		0xBC4E: "\u7197",
		0xBC4F: "\u7198",
		0xBC50: "\u719B",
		0xBC51: "\u719C",
		0xBC52: "\u71A0",
		0xBC53: "\u71A5",
		0xBC54: "\u71AF",
		0xBC55: "\u71B0",
		0xBC56: "\u71B3",
		0xBC57: "\u71B5",
		0xBC58: "\u71BA",
		0xBC59: "\u71C1",
		0xBC5A: "\u71C2",
		0xBC5B: "\u71C4",
		0xBC5C: "\u71CA",
		0xBC5D: "\u71CF",
		0xBC5E: "\u71D9",
		0xBC5F: "\u71DA",
		0xBC60: "\u71F4",
		0xBC61: "\u71FE",
		0xBC62: "\u7207",
		0xBC63: "\u720A",
		0xBC64: "\u7214",
		0xBC65: "\u7215",
		0xBC66: "\u7217",
		0xBC67: "\u721A",
		0xBC68: "\u721D",
		0xBC69: "\u7231",
		0xBC6A: "\u7237",
		0xBC6B: "\u7238",
		0xBC6C: "\u7239",
		0xBC6D: "\u7241",
		0xBC6E: "\u7242",
		0xBC6F: "\u724D",
		0xBC70: "\u724E",
		0xBC71: "\u7250",
		0xBC72: "\u7253",
		0xBC73: "\u7255",
		0xBC74: "\u7256",
		0xBC75: "\u725A",
		0xBC76: "\u7260",
		0xBC77: "\u7263",
		0xBC78: "\u7264",
		0xBC79: "\u7266",
		0xBC7A: "\u726E",
		0xBC7B: "\u726F",
		0xBC7C: "\u7275",
		0xBC7D: "\u7277",
		0xBC7E: "\u7278",
		0xBD21: "\u727A",
		0xBD22: "\u727B",
		0xBD23: "\u727F",
		0xBD24: "\u7284",
		0xBD25: "\u728A",
		0xBD26: "\u728B",
		0xBD27: "\u728D",
		0xBD28: "\u728F",
		0xBD29: "\u729B",
		0xBD2A: "\u729F",
		0xBD2B: "\u72A8",
		0xBD2C: "\u72AD",
		0xBD2D: "\u72B0",
		0xBD2E: "\u72B4",
		0xBD2F: "\u72B7",
		0xBD30: "\u72B8",
		0xBD31: "\u72BC",
		0xBD32: "\u72C0",
		0xBD33: "\u72C1",
		0xBD34: "\u72C8",
		0xBD35: "\u72C9",
		0xBD36: "\u72CC",
		0xBD37: "\u72CD",
		0xBD38: "\u72D3",
		0xBD39: "\u72D6",
		0xBD3A: "\u72DD",
		0xBD3B: "\u72DE",
		0xBD3C: "\u72E5",
		0xBD3D: "\u72EE",
		0xBD3E: "\u72EF",
		0xBD3F: "\u72F0",
		0xBD40: "\u72F1",
		0xBD41: "\u72F2",
		0xBD42: "\u72F3",
		0xBD43: "\u72F4",
		0xBD44: "\u72FA",
		0xBD45: "\u72FB",
		0xBD46: "\u72FE",
		0xBD47: "\u7301",
		0xBD48: "\u7303",
		0xBD49: "\u7304",
		0xBD4A: "\u7307",
		0xBD4B: "\u730B",
		0xBD4C: "\u730E",
		0xBD4D: "\u7315",
		0xBD4E: "\u7318",
		0xBD4F: "\u731E",
		0xBD50: "\u7321",
		0xBD51: "\u7322",
		0xBD52: "\u7328",
		0xBD53: "\u732C",
		0xBD54: "\u7330",
		0xBD55: "\u7331",
		0xBD56: "\u7332",
		0xBD57: "\u7338",
		0xBD58: "\u7339",
		0xBD59: "\u733A",
		0xBD5A: "\u733B",
		0xBD5B: "\u7343",
		0xBD5C: "\u7349",
		0xBD5D: "\u734D",
		0xBD5E: "\u7350",
		0xBD5F: "\u7352",
		0xBD60: "\u7355",
		0xBD61: "\u7358",
		0xBD62: "\u7360",
		0xBD63: "\u7367",
		0xBD64: "\u736B",
		0xBD65: "\u736C",
		0xBD66: "\u736D",
		0xBD67: "\u736E",
		0xBD68: "\u736F",
		0xBD69: "\u7374",
		0xBD6A: "\u7377",
		0xBD6B: "\u737C",
		0xBD6C: "\u737E",
		0xBD6D: "\u7380",
		0xBD6E: "\u7381",
		0xBD6F: "\u7385",
		0xBD70: "\u738D",
		0xBD71: "\u738E",
		0xBD72: "\u738F",
		0xBD73: "\u7391",
		0xBD74: "\u7393",
		0xBD75: "\u7395",
		0xBD76: "\u7398",
		0xBD77: "\u7399",
		0xBD78: "\u739A",
		0xBD79: "\u739B",
		0xBD7A: "\u739E",
		0xBD7B: "\u73A0",
		0xBD7C: "\u73A1",
		0xBD7D: "\u73A2",
		0xBD7E: "\u73A5",
		0xBE21: "\u73A6",
		0xBE22: "\u73AB",
		0xBE23: "\u73AD",
		0xBE24: "\u73AE",
		0xBE25: "\u73AF",
		0xBE26: "\u73B0",
		0xBE27: "\u73B1",
		0xBE28: "\u73B7",
		0xBE29: "\u73BA",
		0xBE2A: "\u73C5",
		0xBE2B: "\u73C9",
		0xBE2C: "\u73CC",
		0xBE2D: "\u73CF",
		0xBE2E: "\u73D0",
		0xBE2F: "\u73D1",
		0xBE30: "\u73D3",
		0xBE31: "\u73D9",
		0xBE32: "\u73E3",
		0xBE33: "\u73E6",
		0xBE34: "\u73E7",
		0xBE35: "\u73E9",
		0xBE36: "\u73F0",
		0xBE37: "\u73F2",
		0xBE38: "\u73F7",
		0xBE39: "\u73FA",
		0xBE3A: "\u73FD",
		0xBE3B: "\u7400",
		0xBE3C: "\u7407",
		0xBE3D: "\u740A",
		0xBE3E: "\u740E",
		0xBE3F: "\u740F",
		0xBE40: "\u7410",
		0xBE41: "\u7416",
		0xBE42: "\u741A",
		0xBE43: "\u741B",
		0xBE44: "\u7424",
		0xBE45: "\u7426",
		0xBE46: "\u7428",
		0xBE47: "\u742A",
		0xBE48: "\u742B",
		0xBE49: "\u742C",
		0xBE4A: "\u742D",
		0xBE4B: "\u742E",
		0xBE4C: "\u742F",
		0xBE4D: "\u7430",
		0xBE4E: "\u7431",
		0xBE4F: "\u7439",
		0xBE50: "\u743C",
		0xBE51: "\u7440",
		0xBE52: "\u7444",
		0xBE53: "\u7447",
		0xBE54: "\u7449",
		0xBE55: "\u744A",
		0xBE56: "\u744B",
		0xBE57: "\u7451",
		0xBE58: "\u7452",
		0xBE59: "\u7457",
		0xBE5A: "\u7462",
		0xBE5B: "\u7468",
		0xBE5C: "\u746D",
		0xBE5D: "\u7471",
		0xBE5E: "\u7472",
		0xBE5F: "\u7474",
		0xBE60: "\u7477",
		0xBE61: "\u747D",
		0xBE62: "\u7480",
		0xBE63: "\u7481",
		0xBE64: "\u7485",
		0xBE65: "\u7486",
		0xBE66: "\u7487",
		0xBE67: "\u7488",
		0xBE68: "\u748A",
		0xBE69: "\u748E",
		0xBE6A: "\u7490",
		0xBE6B: "\u7495",
		0xBE6C: "\u7498",
		0xBE6D: "\u749C",
		0xBE6E: "\u749F",
		0xBE6F: "\u74A0",
		0xBE70: "\u74A1",
		0xBE71: "\u74A3",
		0xBE72: "\u74A6",
		0xBE73: "\u74A8",
		0xBE74: "\u74A9",
		0xBE75: "\u74AA",
		0xBE76: "\u74AB",
		0xBE77: "\u74B5",
		0xBE78: "\u74BA",
		0xBE79: "\u74BF",
		0xBE7A: "\u74C5",
		0xBE7B: "\u74C8",
		0xBE7C: "\u74CC",
		0xBE7D: "\u74D2",
		0xBE7E: "\u74D6",
		0xBF21: "\u74D8",
		0xBF22: "\u74DB",
		0xBF23: "\u74DE",
		0xBF24: "\u74E4",
		0xBF25: "\u74EF",
		0xBF26: "\u74F4",
		0xBF27: "\u74FB",
		0xBF28: "\u74FF",
		0xBF29: "\u7506",
		0xBF2A: "\u750F",
		0xBF2B: "\u7514",
		0xBF2C: "\u7516",
		0xBF2D: "\u7517",
		0xBF2E: "\u7519",
		0xBF2F: "\u7521",
		0xBF30: "\u7529",
		0xBF31: "\u752A",
		0xBF32: "\u752D",
		0xBF33: "\u752E",
		0xBF34: "\u752F",
		0xBF35: "\u7535",
		0xBF36: "\u753D",
		0xBF37: "\u753E",
		0xBF38: "\u753F",
		0xBF39: "\u7540",
		0xBF3A: "\u7545",
		0xBF3B: "\u7547",
		0xBF3C: "\u7548",
		0xBF3D: "\u754E",
		0xBF3E: "\u7556",
		0xBF3F: "\u756C",
		0xBF40: "\u756F",
		0xBF41: "\u7571",
		0xBF42: "\u7572",
		0xBF43: "\u7579",
		0xBF44: "\u7583",
		0xBF45: "\u758C",
		0xBF46: "\u758D",
		0xBF47: "\u7590",
		0xBF48: "\u7592",
		0xBF49: "\u7596",
		0xBF4A: "\u7597",
		0xBF4B: "\u7599",
		0xBF4C: "\u759F",
		0xBF4D: "\u75A0",
		0xBF4E: "\u75A1",
		0xBF4F: "\u75A2",
		0xBF50: "\u75A4",
		0xBF51: "\u75A7",
		0xBF52: "\u75AC",
		0xBF53: "\u75AD",
		0xBF54: "\u75AE",
		0xBF55: "\u75AF",
		0xBF56: "\u75B0",
		0xBF57: "\u75B4",
		0xBF58: "\u75BF",
		0xBF59: "\u75C0",
		0xBF5A: "\u75C1",
		0xBF5B: "\u75C4",
		0xBF5C: "\u75C8",
		0xBF5D: "\u75C9",
		0xBF5E: "\u75CC",
		0xBF5F: "\u75CE",
		0xBF60: "\u75CF",
		0xBF61: "\u75D6",
		0xBF62: "\u75D7",
		0xBF63: "\u75E0",
		0xBF64: "\u75E1",
		0xBF65: "\u75E4",
		0xBF66: "\u75E6",
		0xBF67: "\u75E7",
		0xBF68: "\u75E8",
		0xBF69: "\u75EA",
		0xBF6A: "\u75EB",
		0xBF6B: "\u75EF",
		0xBF6C: "\u75F1",
		0xBF6D: "\u75F9",
		0xBF6E: "\u7600",
		0xBF6F: "\u7602",
		0xBF70: "\u7603",
		0xBF71: "\u7604",
		0xBF72: "\u7605",
		0xBF73: "\u7606",
		0xBF74: "\u7608",
		0xBF75: "\u760A",
		0xBF76: "\u760C",
		0xBF77: "\u760F",
		0xBF78: "\u7610",
		0xBF79: "\u7615",
		0xBF7A: "\u7616",
		0xBF7B: "\u7617",
		0xBF7C: "\u7618",
		0xBF7D: "\u7619",
		0xBF7E: "\u761B",
		0xC021: "\u761C",
		0xC022: "\u761D",
		0xC023: "\u761E",
		0xC024: "\u7625",
		0xC025: "\u7626",
		0xC026: "\u7629",
		0xC027: "\u762A",
		0xC028: "\u762B",
		0xC029: "\u762D",
		0xC02A: "\u762E",
		0xC02B: "\u7632",
		0xC02C: "\u7633",
		0xC02D: "\u7635",
		0xC02E: "\u7638",
		0xC02F: "\u763A",
		0xC030: "\u763C",
		0xC031: "\u763E",
		0xC032: "\u763F",
		0xC033: "\u7640",
		0xC034: "\u7643",
		0xC035: "\u7649",
		0xC036: "\u764D",
		0xC037: "\u7654",
		0xC038: "\u7659",
		0xC039: "\u765E",
		0xC03A: "\u765F",
		0xC03B: "\u7663",
		0xC03C: "\u7664",
		0xC03D: "\u7665",
		0xC03E: "\u766B",
		0xC03F: "\u766D",
		0xC040: "\u766E",
		0xC041: "\u766F",
		0xC042: "\u7671",
		0xC043: "\u7681",
		0xC044: "\u7682",
		0xC045: "\u7691",
		0xC046: "\u7695",
		0xC047: "\u769C",
		0xC048: "\u769D",
		0xC049: "\u769E",
		0xC04A: "\u76A0",
		0xC04B: "\u76A4",
		0xC04C: "\u76A6",
		0xC04D: "\u76AD",
		0xC04E: "\u76B1",
		0xC04F: "\u76B2",
		0xC050: "\u76BB",
		0xC051: "\u76C5",
		0xC052: "\u76C7",
		0xC053: "\u76C9",
		0xC054: "\u76CB",
		0xC055: "\u76CC",
		0xC056: "\u76CE",
		0xC057: "\u76CF",
		0xC058: "\u76D0",
		0xC059: "\u76D1",
		0xC05A: "\u76D4",
		0xC05B: "\u76D8",
		0xC05C: "\u76E6",
		0xC05D: "\u76E9",
		0xC05E: "\u76EC",
		0xC05F: "\u76ED",
		0xC060: "\u76EF",
		0xC061: "\u76F1",
		0xC062: "\u76F9",
		0xC063: "\u76FC",
		0xC064: "\u770A",
		0xC065: "\u770D",
		0xC066: "\u770E",
		0xC067: "\u7718",
		0xC068: "\u7719",
		0xC069: "\u771A",
		0xC06A: "\u771C",
		0xC06B: "\u7721",
		0xC06C: "\u7722",
		0xC06D: "\u7728",
		0xC06E: "\u772C",
		0xC06F: "\u772D",
		0xC070: "\u772F",
		0xC071: "\u7735",
		0xC072: "\u7736",
		0xC073: "\u773D",
		0xC074: "\u7741",
		0xC075: "\u7743",
		0xC076: "\u7745",
		0xC077: "\u7746",
		0xC078: "\u774A",
		0xC079: "\u774E",
		0xC07A: "\u774F",
		0xC07B: "\u7750",
		0xC07C: "\u7751",
		0xC07D: "\u7752",
		0xC07E: "\u7756",
		0xC121: "\u775E",
		0xC122: "\u7760",
		0xC123: "\u7762",
		0xC124: "\u7769",
		0xC125: "\u776C",
		0xC126: "\u777A",
		0xC127: "\u777D",
		0xC128: "\u7780",
		0xC129: "\u7784",
		0xC12A: "\u7785",
		0xC12B: "\u7787",
		0xC12C: "\u778C",
		0xC12D: "\u778D",
		0xC12E: "\u7792",
		0xC12F: "\u7796",
		0xC130: "\u7798",
		0xC131: "\u779C",
		0xC132: "\u779F",
		0xC133: "\u77A2",
		0xC134: "\u77A7",
		0xC135: "\u77A9",
		0xC136: "\u77AA",
		0xC137: "\u77AB",
		0xC138: "\u77B5",
		0xC139: "\u77D1",
		0xC13A: "\u77D3",
		0xC13B: "\u77D9",
		0xC13C: "\u77DE",
		0xC13D: "\u77DF",
		0xC13E: "\u77EB",
		0xC13F: "\u77EC",
		0xC140: "\u77F0",
		0xC141: "\u77F1",
		0xC142: "\u77F4",
		0xC143: "\u77F6",
		0xC144: "\u77F8",
		0xC145: "\u77FB",
		0xC146: "\u77FD",
		0xC147: "\u77FE",
		0xC148: "\u77FF",
		0xC149: "\u7800",
		0xC14A: "\u7801",
		0xC14B: "\u7804",
		0xC14C: "\u7806",
		0xC14D: "\u7809",
		0xC14E: "\u780D",
		0xC14F: "\u7811",
		0xC150: "\u7816",
		0xC151: "\u7817",
		0xC152: "\u7818",
		0xC153: "\u781A",
		0xC154: "\u781C",
		0xC155: "\u781D",
		0xC156: "\u781F",
		0xC157: "\u7822",
		0xC158: "\u7823",
		0xC159: "\u7829",
		0xC15A: "\u782B",
		0xC15B: "\u782C",
		0xC15C: "\u782D",
		0xC15D: "\u782E",
		0xC15E: "\u7830",
		0xC15F: "\u7837",
		0xC160: "\u7838",
		0xC161: "\u7839",
		0xC162: "\u783B",
		0xC163: "\u783C",
		0xC164: "\u783E",
		0xC165: "\u7840",
		0xC166: "\u7841",
		0xC167: "\u7843",
		0xC168: "\u7847",
		0xC169: "\u784A",
		0xC16A: "\u784B",
		0xC16B: "\u784C",
		0xC16C: "\u784E",
		0xC16D: "\u784F",
		0xC16E: "\u7850",
		0xC16F: "\u7852",
		0xC170: "\u7855",
		0xC171: "\u7856",
		0xC172: "\u7857",
		0xC173: "\u7859",
		0xC174: "\u785A",
		0xC175: "\u785C",
		0xC176: "\u7860",
		0xC177: "\u7864",
		0xC178: "\u7868",
		0xC179: "\u786A",
		0xC17A: "\u786D",
		0xC17B: "\u786E",
		0xC17C: "\u7875",
		0xC17D: "\u7876",
		0xC17E: "\u7877",
		0xC221: "\u7889",
		0xC222: "\u788F",
		0xC223: "\u7894",
		0xC224: "\u7898",
		0xC225: "\u789B",
		0xC226: "\u789C",
		0xC227: "\u789F",
		0xC228: "\u78A1",
		0xC229: "\u78A5",
		0xC22A: "\u78A8",
		0xC22B: "\u78AB",
		0xC22C: "\u78AD",
		0xC22D: "\u78B0",
		0xC22E: "\u78B1",
		0xC22F: "\u78B2",
		0xC230: "\u78B3",
		0xC231: "\u78B4",
		0xC232: "\u78B6",
		0xC233: "\u78B9",
		0xC234: "\u78BB",
		0xC235: "\u78C9",
		0xC236: "\u78CE",
		0xC237: "\u78CF",
		0xC238: "\u78D5",
		0xC239: "\u78D6",
		0xC23A: "\u78D9",
		0xC23B: "\u78DC",
		0xC23C: "\u78DF",
		0xC23D: "\u78E0",
		0xC23E: "\u78E1",
		0xC23F: "\u78E3",
		0xC240: "\u78F2",
		0xC241: "\u78F7",
		0xC242: "\u78F9",
		0xC243: "\u78FA",
		0xC244: "\u78FB",
		0xC245: "\u78FE",
		0xC246: "\u7903",
		0xC247: "\u7904",
		0xC248: "\u7905",
		0xC249: "\u7906",
		0xC24A: "\u790C",
		0xC24B: "\u7913",
		0xC24C: "\u791E",
		0xC24D: "\u7924",
		0xC24E: "\u7927",
		0xC24F: "\u7929",
		0xC250: "\u792E",
		0xC251: "\u7931",
		0xC252: "\u7933",
		0xC253: "\u7934",
		0xC254: "\u7935",
		0xC255: "\u793B",
		0xC256: "\u793D",
		0xC257: "\u7943",
		0xC258: "\u7946",
		0xC259: "\u794A",
		0xC25A: "\u794E",
		0xC25B: "\u794F",
		0xC25C: "\u7954",
		0xC25D: "\u7958",
		0xC25E: "\u795B",
		0xC25F: "\u795C",
		0xC260: "\u7967",
		0xC261: "\u796B",
		0xC262: "\u796F",
		0xC263: "\u7972",
		0xC264: "\u7978",
		0xC265: "\u797C",
		0xC266: "\u797E",
		0xC267: "\u798B",
		0xC268: "\u7995",
		0xC269: "\u7996",
		0xC26A: "\u7998",
		0xC26B: "\u799A",
		0xC26C: "\u799B",
		0xC26D: "\u79A4",
		0xC26E: "\u79A8",
		0xC26F: "\u79A9",
		0xC270: "\u79AB",
		0xC271: "\u79B1",
		0xC272: "\u79BB",
		0xC273: "\u79BC",
		0xC274: "\u79C3",
		0xC275: "\u79C6",
		0xC276: "\u79C8",
		0xC277: "\u79CA",
		0xC278: "\u79CD",
		0xC279: "\u79D4",
		0xC27A: "\u79EB",
		0xC27B: "\u79ED",
		0xC27C: "\u79EF",
		0xC27D: "\u79F8",
		0xC27E: "\u79FD",
		0xC321: "\u79FE",
		0xC322: "\u7A02",
		0xC323: "\u7A03",
		0xC324: "\u7A06",
		0xC325: "\u7A09",
		0xC326: "\u7A16",
		0xC327: "\u7A1E",
		0xC328: "\u7A23",
		0xC329: "\u7A28",
		0xC32A: "\u7A2C",
		0xC32B: "\u7A2D",
		0xC32C: "\u7A33",
		0xC32D: "\u7A39",
		0xC32E: "\u7A44",
		0xC32F: "\u7A45",
		0xC330: "\u7A47",
		0xC331: "\u7A4A",
		0xC332: "\u7A51",
		0xC333: "\u7A5C",
		0xC334: "\u7A5F",
		0xC335: "\u7A60",
		0xC336: "\u7A64",
		0xC337: "\u7A6D",
		0xC338: "\u7A6E",
		0xC339: "\u7A75",
		0xC33A: "\u7A77",
		0xC33B: "\u7A78",
		0xC33C: "\u7A80",
		0xC33D: "\u7A85",
		0xC33E: "\u7A86",
		0xC33F: "\u7A8A",
		0xC340: "\u7A8C",
		0xC341: "\u7A8D",
		0xC342: "\u7A8E",
		0xC343: "\u7A91",
		0xC344: "\u7A9C",
		0xC345: "\u7A9D",
		0xC346: "\u7AA0",
		0xC347: "\u7AA3",
		0xC348: "\u7AA5",
		0xC349: "\u7AA6",
		0xC34A: "\u7AA8",
		0xC34B: "\u7AAC",
		0xC34C: "\u7AAD",
		0xC34D: "\u7AB3",
		0xC34E: "\u7AB8",
		0xC34F: "\u7ABB",
		0xC350: "\u7ABE",
		0xC351: "\u7AD1",
		0xC352: "\u7AD6",
		0xC353: "\u7ADE",
		0xC354: "\u7AFD",
		0xC355: "\u7AFE",
		0xC356: "\u7B03",
		0xC357: "\u7B13",
		0xC358: "\u7B14",
		0xC359: "\u7B15",
		0xC35A: "\u7B22",
		0xC35B: "\u7B24",
		0xC35C: "\u7B2A",
		0xC35D: "\u7B2B",
		0xC35E: "\u7B2D",
		0xC35F: "\u7B2E",
		0xC360: "\u7B31",
		0xC361: "\u7B34",
		0xC362: "\u7B38",
		0xC363: "\u7B3A",
		0xC364: "\u7B3C",
		0xC365: "\u7B3E",
		0xC366: "\u7B40",
		0xC367: "\u7B47",
		0xC368: "\u7B4A",
		0xC369: "\u7B58",
		0xC36A: "\u7B5A",
		0xC36B: "\u7B5B",
		0xC36C: "\u7B5C",
		0xC36D: "\u7B5E",
		0xC36E: "\u7B60",
		0xC36F: "\u7B62",
		0xC370: "\u7B66",
		0xC371: "\u7B69",
		0xC372: "\u7B6D",
		0xC373: "\u7B6F",
		0xC374: "\u7B72",
		0xC375: "\u7B77",
		0xC376: "\u7B79",
		0xC377: "\u7B7B",
		0xC378: "\u7B7C",
		0xC379: "\u7B7E",
		0xC37A: "\u7B80",
		0xC37B: "\u7B84",
		0xC37C: "\u7B85",
		0xC37D: "\u7B8E",
		0xC37E: "\u7B90",
		0xC421: "\u7B91",
		0xC422: "\u7B93",
		0xC423: "\u7BA0",
		0xC424: "\u7BA2",
		0xC425: "\u7BA6",
		0xC426: "\u7BA7",
		0xC427: "\u7BA8",
		0xC428: "\u7BA9",
		0xC429: "\u7BAB",
		0xC42A: "\u7BAC",
		0xC42B: "\u7BB2",
		0xC42C: "\u7BB5",
		0xC42D: "\u7BB7",
		0xC42E: "\u7BC5",
		0xC42F: "\u7BCA",
		0xC430: "\u7BD1",
		0xC431: "\u7BD3",
		0xC432: "\u7BD8",
		0xC433: "\u7BD9",
		0xC434: "\u7BDA",
		0xC435: "\u7BDB",
		0xC436: "\u7BE1",
		0xC437: "\u7BE8",
		0xC438: "\u7BEA",
		0xC439: "\u7BEE",
		0xC43A: "\u7BF0",
		0xC43B: "\u7BF1",
		0xC43C: "\u7BF2",
		0xC43D: "\u7BF8",
		0xC43E: "\u7BF9",
		0xC43F: "\u7BFC",
		0xC440: "\u7BFE",
		0xC441: "\u7C01",
		0xC442: "\u7C03",
		0xC443: "\u7C06",
		0xC444: "\u7C09",
		0xC445: "\u7C0B",
		0xC446: "\u7C0C",
		0xC447: "\u7C0E",
		0xC448: "\u7C0F",
		0xC449: "\u7C15",
		0xC44A: "\u7C16",
		0xC44B: "\u7C1E",
		0xC44C: "\u7C20",
		0xC44D: "\u7C26",
		0xC44E: "\u7C28",
		0xC44F: "\u7C30",
		0xC450: "\u7C34",
		0xC451: "\u7C39",
		0xC452: "\u7C3B",
		0xC453: "\u7C41",
		0xC454: "\u7C46",
		0xC455: "\u7C48",
		0xC456: "\u7C4B",
		0xC457: "\u7C51",
		0xC458: "\u7C59",
		0xC459: "\u7C5C",
		0xC45A: "\u7C5D",
		0xC45B: "\u7C62",
		0xC45C: "\u7C67",
		0xC45D: "\u7C69",
		0xC45E: "\u7C6A",
		0xC45F: "\u7C6E",
		0xC460: "\u7C6F",
		0xC461: "\u7C70",
		0xC462: "\u7C72",
		0xC463: "\u7C74",
		0xC464: "\u7C78",
		0xC465: "\u7C79",
		0xC466: "\u7C7B",
		0xC467: "\u7C7C",
		0xC468: "\u7C7D",
		0xC469: "\u7C91",
		0xC46A: "\u7C94",
		0xC46B: "\u7C9C",
		0xC46C: "\u7C9D",
		0xC46D: "\u7C9E",
		0xC46E: "\u7CAA",
		0xC46F: "\u7CAC",
		0xC470: "\u7CBC",
		0xC471: "\u7CC1",
		0xC472: "\u7CC7",
		0xC473: "\u7CC8",
		0xC474: "\u7CC9",
		0xC475: "\u7CCC",
		0xC476: "\u7CCD",
		0xC477: "\u7CD5",
		0xC478: "\u7CD7",
		0xC479: "\u7CD9",
		0xC47A: "\u7CDD",
		0xC47B: "\u7CE1",
		0xC47C: "\u7CE8",
		0xC47D: "\u7CF0",
		0xC47E: "\u7CF1",
		0xC521: "\u7CF5",
		0xC522: "\u7D07",
		0xC523: "\u7D25",
		0xC524: "\u7D27",
		0xC525: "\u7D4D",
		0xC526: "\u7D53",
		0xC527: "\u7D5A",
		0xC528: "\u7D5C",
		0xC529: "\u7D77",
		0xC52A: "\u7D91",
		0xC52B: "\u7DA6",
		0xC52C: "\u7DC9",
		0xC52D: "\u7DCD",
		0xC52E: "\u7DD0",
		0xC52F: "\u7DE5",
		0xC530: "\u7DEA",
		0xC531: "\u7DF1",
		0xC532: "\u7DFA",
		0xC533: "\u7E02",
		0xC534: "\u7E08",
		0xC535: "\u7E10",
		0xC536: "\u7E1A",
		0xC537: "\u7E20",
		0xC538: "\u7E27",
		0xC539: "\u7E34",
		0xC53A: "\u7E36",
		0xC53B: "\u7E44",
		0xC53C: "\u7E47",
		0xC53D: "\u7E50",
		0xC53E: "\u7E60",
		0xC53F: "\u7E61",
		0xC540: "\u7E68",
		0xC541: "\u7E6B",
		0xC542: "\u7E8A",
		0xC543: "\u7E8D",
		0xC544: "\u7E91",
		0xC545: "\u7E95",
		0xC546: "\u7E9A",
		0xC547: "\u7E9F",
		0xC548: "\u7EA0",
		0xC549: "\u7EA1",
		0xC54A: "\u7EA2",
		0xC54B: "\u7EA3",
		0xC54C: "\u7EA4",
		0xC54D: "\u7EA5",
		0xC54E: "\u7EA6",
		0xC54F: "\u7EA7",
		0xC550: "\u7EA8",
		0xC551: "\u7EA9",
		0xC552: "\u7EAA",
		0xC553: "\u7EAB",
		0xC554: "\u7EAC",
		0xC555: "\u7EAD",
		0xC556: "\u7EAE",
		0xC557: "\u7EAF",
		0xC558: "\u7EB0",
		0xC559: "\u7EB1",
		0xC55A: "\u7EB2",
		0xC55B: "\u7EB3",
		0xC55C: "\u7EB4",
		0xC55D: "\u7EB5",
		0xC55E: "\u7EB6",
		0xC55F: "\u7EB7",
		0xC560: "\u7EB8",
		0xC561: "\u7EB9",
		0xC562: "\u7EBA",
		0xC563: "\u7EBB",
		0xC564: "\u7EBC",
		0xC565: "\u7EBD",
		0xC566: "\u7EBE",
		0xC567: "\u7EBF",
		0xC568: "\u7EC0",
		0xC569: "\u7EC1",
		0xC56A: "\u7EC2",
		0xC56B: "\u7EC3",
		0xC56C: "\u7EC4",
		0xC56D: "\u7EC5",
		0xC56E: "\u7EC6",
		0xC56F: "\u7EC7",
		0xC570: "\u7EC8",
		0xC571: "\u7EC9",
		0xC572: "\u7ECA",
		0xC573: "\u7ECB",
		0xC574: "\u7ECC",
		0xC575: "\u7ECD",
		0xC576: "\u7ECE",
		0xC577: "\u7ECF",
		0xC578: "\u7ED0",
		0xC579: "\u7ED1",
		0xC57A: "\u7ED2",
		0xC57B: "\u7ED3",
		0xC57C: "\u7ED4",
		0xC57D: "\u7ED5",
		0xC57E: "\u7ED7",
		0xC621: "\u7ED8",
		0xC622: "\u7ED9",
		0xC623: "\u7EDA",
		0xC624: "\u7EDB",
		0xC625: "\u7EDC",
		0xC626: "\u7EDD",
		0xC627: "\u7EDE",
		0xC628: "\u7EDF",
		0xC629: "\u7EE0",
		0xC62A: "\u7EE1",
		0xC62B: "\u7EE2",
		0xC62C: "\u7EE3",
		0xC62D: "\u7EE4",
		0xC62E: "\u7EE5",
		0xC62F: "\u7EE6",
		0xC630: "\u7EE7",
		0xC631: "\u7EE8",
		0xC632: "\u7EE9",
		0xC633: "\u7EEA",
		0xC634: "\u7EEB",
		0xC635: "\u7EED",
		0xC636: "\u7EEE",
		0xC637: "\u7EEF",
		0xC638: "\u7EF0",
		0xC639: "\u7EF1",
		0xC63A: "\u7EF2",
		0xC63B: "\u7EF3",
		0xC63C: "\u7EF4",
		0xC63D: "\u7EF5",
		0xC63E: "\u7EF6",
		0xC63F: "\u7EF7",
		0xC640: "\u7EF8",
		0xC641: "\u7EF9",
		0xC642: "\u7EFA",
		0xC643: "\u7EFB",
		0xC644: "\u7EFC",
		0xC645: "\u7EFD",
		0xC646: "\u7EFE",
		0xC647: "\u7EFF",
		0xC648: "\u7F00",
		0xC649: "\u7F01",
		0xC64A: "\u7F02",
		0xC64B: "\u7F03",
		0xC64C: "\u7F04",
		0xC64D: "\u7F05",
		0xC64E: "\u7F06",
		0xC64F: "\u7F07",
		0xC650: "\u7F08",
		0xC651: "\u7F09",
		0xC652: "\u7F0A",
		0xC653: "\u7F0B",
		0xC654: "\u7F0C",
		0xC655: "\u7F0E",
		0xC656: "\u7F0F",
		0xC657: "\u7F10",
		0xC658: "\u7F11",
		0xC659: "\u7F12",
		0xC65A: "\u7F13",
		0xC65B: "\u7F14",
		0xC65C: "\u7F15",
		0xC65D: "\u7F16",
		0xC65E: "\u7F17",
		0xC65F: "\u7F18",
		0xC660: "\u7F19",
		0xC661: "\u7F1A",
		0xC662: "\u7F1B",
		0xC663: "\u7F1C",
		0xC664: "\u7F1D",
		0xC665: "\u7F1E",
		0xC666: "\u7F1F",
		0xC667: "\u7F20",
		0xC668: "\u7F21",
		0xC669: "\u7F22",
		0xC66A: "\u7F23",
		0xC66B: "\u7F24",
		0xC66C: "\u7F25",
		0xC66D: "\u7F26",
		0xC66E: "\u7F27",
		0xC66F: "\u7F28",
		0xC670: "\u7F29",
		0xC671: "\u7F2A",
		0xC672: "\u7F2B",
		0xC673: "\u7F2C",
		0xC674: "\u7F2D",
		0xC675: "\u7F2E",
		0xC676: "\u7F2F",
		0xC677: "\u7F30",
		0xC678: "\u7F31",
		0xC679: "\u7F32",
		0xC67A: "\u7F33",
		0xC67B: "\u7F34",
		0xC67C: "\u7F35",
		0xC67D: "\u7F3D",
		0xC67E: "\u7F3E",
		0xC721: "\u7F3F",
		0xC722: "\u7F42",
		0xC723: "\u7F43",
		0xC724: "\u7F44",
		0xC725: "\u7F46",
		0xC726: "\u7F47",
		0xC727: "\u7F48",
		0xC728: "\u7F4B",
		0xC729: "\u7F4F",
		0xC72A: "\u7F52",
		0xC72B: "\u7F57",
		0xC72C: "\u7F5A",
		0xC72D: "\u7F5B",
		0xC72E: "\u7F5D",
		0xC72F: "\u7F61",
		0xC730: "\u7F62",
		0xC731: "\u7F63",
		0xC732: "\u7F65",
		0xC733: "\u7F66",
		0xC734: "\u7F6D",
		0xC735: "\u7F71",
		0xC736: "\u7F73",
		0xC737: "\u7F74",
		0xC738: "\u7F76",
		0xC739: "\u7F7B",
		0xC73A: "\u7F7D",
		0xC73B: "\u7F7E",
		0xC73C: "\u7F7F",
		0xC73D: "\u7F81",
		0xC73E: "\u7F8B",
		0xC73F: "\u7F91",
		0xC740: "\u7F95",
		0xC741: "\u7F96",
		0xC742: "\u7F9F",
		0xC743: "\u7FA1",
		0xC744: "\u7FA2",
		0xC745: "\u7FA5",
		0xC746: "\u7FA7",
		0xC747: "\u7FB0",
		0xC748: "\u7FB1",
		0xC749: "\u7FB4",
		0xC74A: "\u7FBC",
		0xC74B: "\u7FBF",
		0xC74C: "\u7FC0",
		0xC74D: "\u7FC3",
		0xC74E: "\u7FC4",
		0xC74F: "\u7FCE",
		0xC750: "\u7FD8",
		0xC751: "\u7FD9",
		0xC752: "\u7FDA",
		0xC753: "\u7FDB",
		0xC754: "\u7FDF",
		0xC755: "\u7FE5",
		0xC756: "\u7FEE",
		0xC757: "\u7FEF",
		0xC758: "\u7FF1",
		0xC759: "\u7FFA",
		0xC75A: "\u7FFD",
		0xC75B: "\u7FFE",
		0xC75C: "\u8007",
		0xC75D: "\u8008",
		0xC75E: "\u800D",
		0xC75F: "\u8011",
		0xC760: "\u8014",
		0xC761: "\u8016",
		0xC762: "\u8020",
		0xC763: "\u8022",
		0xC764: "\u8025",
		0xC765: "\u8026",
		0xC766: "\u8027",
		0xC767: "\u8029",
		0xC768: "\u802A",
		0xC769: "\u802C",
		0xC76A: "\u802E",
		0xC76B: "\u8030",
		0xC76C: "\u8031",
		0xC76D: "\u8032",
		0xC76E: "\u8035",
		0xC76F: "\u8037",
		0xC770: "\u8038",
		0xC771: "\u803C",
		0xC772: "\u8042",
		0xC773: "\u8043",
		0xC774: "\u804B",
		0xC775: "\u804C",
		0xC776: "\u804D",
		0xC777: "\u8054",
		0xC778: "\u805D",
		0xC779: "\u8069",
		0xC77A: "\u806A",
		0xC77B: "\u8071",
		0xC77C: "\u807C",
		0xC77D: "\u8083",
		0xC77E: "\u8088",
		0xC821: "\u808A",
		0xC822: "\u808E",
		0xC823: "\u808F",
		0xC824: "\u8090",
		0xC825: "\u809C",
		0xC826: "\u809F",
		0xC827: "\u80A0",
		0xC828: "\u80A4",
		0xC829: "\u80A7",
		0xC82A: "\u80A8",
		0xC82B: "\u80AB",
		0xC82C: "\u80AE",
		0xC82D: "\u80B7",
		0xC82E: "\u80B8",
		0xC82F: "\u80BC",
		0xC830: "\u80BD",
		0xC831: "\u80BE",
		0xC832: "\u80BF",
		0xC833: "\u80C0",
		0xC834: "\u80C1",
		0xC835: "\u80C2",
		0xC836: "\u80C8",
		0xC837: "\u80CD",
		0xC838: "\u80D0",
		0xC839: "\u80D4",
		0xC83A: "\u80D7",
		0xC83B: "\u80DC",
		0xC83C: "\u80E0",
		0xC83D: "\u80E7",
		0xC83E: "\u80E8",
		0xC83F: "\u80E9",
		0xC840: "\u80EA",
		0xC841: "\u80EB",
		0xC842: "\u80EC",
		0xC843: "\u80ED",
		0xC844: "\u80EE",
		0xC845: "\u80F0",
		0xC846: "\u80F2",
		0xC847: "\u80F3",
		0xC848: "\u80F5",
		0xC849: "\u80F6",
		0xC84A: "\u80F7",
		0xC84B: "\u80F9",
		0xC84C: "\u80FA",
		0xC84D: "\u80FE",
		0xC84E: "\u8103",
		0xC84F: "\u8104",
		0xC850: "\u810D",
		0xC851: "\u810E",
		0xC852: "\u810F",
		0xC853: "\u8110",
		0xC854: "\u8111",
		0xC855: "\u8112",
		0xC856: "\u8113",
		0xC857: "\u8114",
		0xC858: "\u8116",
		0xC859: "\u8117",
		0xC85A: "\u8118",
		0xC85B: "\u811D",
		0xC85C: "\u811E",
		0xC85D: "\u8121",
		0xC85E: "\u8122",
		0xC85F: "\u8126",
		0xC860: "\u812C",
		0xC861: "\u8130",
		0xC862: "\u8132",
		0xC863: "\u8136",
		0xC864: "\u8138",
		0xC865: "\u813A",
		0xC866: "\u8148",
		0xC867: "\u814A",
		0xC868: "\u814C",
		0xC869: "\u8152",
		0xC86A: "\u8156",
		0xC86B: "\u8158",
		0xC86C: "\u8159",
		0xC86D: "\u815A",
		0xC86E: "\u8160",
		0xC86F: "\u8161",
		0xC870: "\u8164",
		0xC871: "\u8167",
		0xC872: "\u8169",
		0xC873: "\u816D",
		0xC874: "\u816F",
		0xC875: "\u8173",
		0xC876: "\u8177",
		0xC877: "\u817B",
		0xC878: "\u817C",
		0xC879: "\u817D",
		0xC87A: "\u817E",
		0xC87B: "\u8181",
		0xC87C: "\u8186",
		0xC87D: "\u8187",
		0xC87E: "\u818B",
		0xC921: "\u818D",
		0xC922: "\u8191",
		0xC923: "\u8197",
		0xC924: "\u8198",
		0xC925: "\u8199",
		0xC926: "\u819B",
		0xC927: "\u819E",
		0xC928: "\u81A6",
		0xC929: "\u81AA",
		0xC92A: "\u81BB",
		0xC92B: "\u81C1",
		0xC92C: "\u81C3",
		0xC92D: "\u81CA",
		0xC92E: "\u81CC",
		0xC92F: "\u81CF",
		0xC930: "\u81D5",
		0xC931: "\u81D7",
		0xC932: "\u81DB",
		0xC933: "\u81DC",
		0xC934: "\u81DD",
		0xC935: "\u81DE",
		0xC936: "\u81EC",
		0xC937: "\u81EF",
		0xC938: "\u81F2",
		0xC939: "\u81FF",
		0xC93A: "\u8200",
		0xC93B: "\u8203",
		0xC93C: "\u8204",
		0xC93D: "\u8206",
		0xC93E: "\u820B",
		0xC93F: "\u8214",
		0xC940: "\u821D",
		0xC941: "\u8221",
		0xC942: "\u8222",
		0xC943: "\u8223",
		0xC944: "\u8228",
		0xC945: "\u8230",
		0xC946: "\u8231",
		0xC947: "\u8232",
		0xC948: "\u8234",
		0xC949: "\u823B",
		0xC94A: "\u823E",
		0xC94B: "\u8244",
		0xC94C: "\u8245",
		0xC94D: "\u8249",
		0xC94E: "\u824B",
		0xC94F: "\u824E",
		0xC950: "\u824F",
		0xC951: "\u8251",
		0xC952: "\u8253",
		0xC953: "\u8255",
		0xC954: "\u8256",
		0xC955: "\u8263",
		0xC956: "\u8270",
		0xC957: "\u8273",
		0xC958: "\u8274",
		0xC959: "\u8279",
		0xC95A: "\u827A",
		0xC95B: "\u827B",
		0xC95C: "\u827D",
		0xC95D: "\u827F",
		0xC95E: "\u8282",
		0xC95F: "\u8283",
		0xC960: "\u8288",
		0xC961: "\u828A",
		0xC962: "\u828E",
		0xC963: "\u828F",
		0xC964: "\u8291",
		0xC965: "\u8297",
		0xC966: "\u8298",
		0xC967: "\u829C",
		0xC968: "\u82A1",
		0xC969: "\u82A3",
		0xC96A: "\u82A4",
		0xC96B: "\u82A7",
		0xC96C: "\u82A8",
		0xC96D: "\u82A9",
		0xC96E: "\u82AA",
		0xC96F: "\u82AE",
		0xC970: "\u82B0",
		0xC971: "\u82B4",
		0xC972: "\u82B7",
		0xC973: "\u82BC",
		0xC974: "\u82BE",
		0xC975: "\u82C1",
		0xC976: "\u82C4",
		0xC977: "\u82C6",
		0xC978: "\u82C7",
		0xC979: "\u82C8",
		0xC97A: "\u82CA",
		0xC97B: "\u82CB",
		0xC97C: "\u82CC",
		0xC97D: "\u82CD",
		0xC97E: "\u82CE",
		0xCA21: "\u82CF",
		0xCA22: "\u82D5",
		0xCA23: "\u82D8",
		0xCA24: "\u82E0",
		0xCA25: "\u82E2",
		0xCA26: "\u82E4",
		0xCA27: "\u82EF",
		0xCA28: "\u82F2",
		0xCA29: "\u82F6",
		0xCA2A: "\u82F7",
		0xCA2B: "\u82FD",
		0xCA2C: "\u82FE",
		0xCA2D: "\u8300",
		0xCA2E: "\u8301",
		0xCA2F: "\u8307",
		0xCA30: "\u8308",
		0xCA31: "\u830C",
		0xCA32: "\u830F",
		0xCA33: "\u8311",
		0xCA34: "\u8313",
		0xCA35: "\u8314",
		0xCA36: "\u8315",
		0xCA37: "\u831A",
		0xCA38: "\u831B",
		0xCA39: "\u831D",
		0xCA3A: "\u8327",
		0xCA3B: "\u832C",
		0xCA3C: "\u832D",
		0xCA3D: "\u8333",
		0xCA3E: "\u833A",
		0xCA3F: "\u833C",
		0xCA40: "\u8343",
		0xCA41: "\u8344",
		0xCA42: "\u8346",
		0xCA43: "\u8347",
		0xCA44: "\u834D",
		0xCA45: "\u8351",
		0xCA46: "\u8354",
		0xCA47: "\u8359",
		0xCA48: "\u835A",
		0xCA49: "\u835B",
		0xCA4A: "\u835C",
		0xCA4B: "\u835E",
		0xCA4C: "\u835F",
		0xCA4D: "\u8360",
		0xCA4E: "\u8361",
		0xCA4F: "\u8363",
		0xCA50: "\u8364",
		0xCA51: "\u8365",
		0xCA52: "\u8366",
		0xCA53: "\u8367",
		0xCA54: "\u8368",
		0xCA55: "\u8369",
		0xCA56: "\u836A",
		0xCA57: "\u836B",
		0xCA58: "\u836C",
		0xCA59: "\u836D",
		0xCA5A: "\u836E",
		0xCA5B: "\u836F",
		0xCA5C: "\u8378",
		0xCA5D: "\u837D",
		0xCA5E: "\u8386",
		0xCA5F: "\u8392",
		0xCA60: "\u8395",
		0xCA61: "\u8398",
		0xCA62: "\u8399",
		0xCA63: "\u839B",
		0xCA64: "\u839C",
		0xCA65: "\u839D",
		0xCA66: "\u83A7",
		0xCA67: "\u83A9",
		0xCA68: "\u83B0",
		0xCA69: "\u83B2",
		0xCA6A: "\u83B3",
		0xCA6B: "\u83B4",
		0xCA6C: "\u83B6",
		0xCA6D: "\u83B7",
		0xCA6E: "\u83B8",
		0xCA6F: "\u83B9",
		0xCA70: "\u83BA",
		0xCA71: "\u83BC",
		0xCA72: "\u83C0",
		0xCA73: "\u83C2",
		0xCA74: "\u83C7",
		0xCA75: "\u83C9",
		0xCA76: "\u83CD",
		0xCA77: "\u83CF",
		0xCA78: "\u83D1",
		0xCA79: "\u83D4",
		0xCA7A: "\u83DD",
		0xCA7B: "\u83E1",
		0xCA7C: "\u83E2",
		0xCA7D: "\u83E5",
		0xCA7E: "\u83EA",
		0xCB21: "\u83F6",
		0xCB22: "\u83F8",
		0xCB23: "\u83F9",
		0xCB24: "\u83FC",
		0xCB25: "\u83FE",
		0xCB26: "\u8401",
		0xCB27: "\u8406",
		0xCB28: "\u840A",
		0xCB29: "\u840F",
		0xCB2A: "\u8411",
		0xCB2B: "\u8418",
		0xCB2C: "\u841A",
		0xCB2D: "\u841C",
		0xCB2E: "\u841D",
		0xCB2F: "\u8424",
		0xCB30: "\u8425",
		0xCB31: "\u8426",
		0xCB32: "\u8427",
		0xCB33: "\u8428",
		0xCB34: "\u842E",
		0xCB35: "\u8439",
		0xCB36: "\u8445",
		0xCB37: "\u8451",
		0xCB38: "\u8453",
		0xCB39: "\u8456",
		0xCB3A: "\u8459",
		0xCB3B: "\u845A",
		0xCB3C: "\u845C",
		0xCB3D: "\u8460",
		0xCB3E: "\u8467",
		0xCB3F: "\u8473",
		0xCB40: "\u8476",
		0xCB41: "\u8478",
		0xCB42: "\u8487",
		0xCB43: "\u8488",
		0xCB44: "\u8489",
		0xCB45: "\u848C",
		0xCB46: "\u848D",
		0xCB47: "\u848E",
		0xCB48: "\u8497",
		0xCB49: "\u849E",
		0xCB4A: "\u84A8",
		0xCB4B: "\u84AF",
		0xCB4C: "\u84B1",
		0xCB4D: "\u84B4",
		0xCB4E: "\u84BA",
		0xCB4F: "\u84BD",
		0xCB50: "\u84C0",
		0xCB51: "\u84C2",
		0xCB52: "\u84C7",
		0xCB53: "\u84CF",
		0xCB54: "\u84D3",
		0xCB55: "\u84DD",
		0xCB56: "\u84DF",
		0xCB57: "\u84E0",
		0xCB58: "\u84E1",
		0xCB59: "\u84E3",
		0xCB5A: "\u84E5",
		0xCB5B: "\u84E6",
		0xCB5C: "\u84E7",
		0xCB5D: "\u84E8",
		0xCB5E: "\u84EF",
		0xCB5F: "\u84F0",
		0xCB60: "\u84F1",
		0xCB61: "\u84FA",
		0xCB62: "\u84FD",
		0xCB63: "\u8502",
		0xCB64: "\u850A",
		0xCB65: "\u850C",
		0xCB66: "\u851E",
		0xCB67: "\u8523",
		0xCB68: "\u8527",
		0xCB69: "\u852B",
		0xCB6A: "\u8532",
		0xCB6B: "\u8534",
		0xCB6C: "\u8537",
		0xCB6D: "\u8538",
		0xCB6E: "\u8539",
		0xCB6F: "\u853A",
		0xCB70: "\u853B",
		0xCB71: "\u853C",
		0xCB72: "\u853E",
		0xCB73: "\u853F",
		0xCB74: "\u8545",
		0xCB75: "\u8552",
		0xCB76: "\u8553",
		0xCB77: "\u8556",
		0xCB78: "\u8559",
		0xCB79: "\u855E",
		0xCB7A: "\u8564",
		0xCB7B: "\u8570",
		0xCB7C: "\u8572",
		0xCB7D: "\u8574",
		0xCB7E: "\u8579",
		0xCC21: "\u857A",
		0xCC22: "\u857B",
		0xCC23: "\u8581",
		0xCC24: "\u8585",
		0xCC25: "\u8586",
		0xCC26: "\u858C",
		0xCC27: "\u858F",
		0xCC28: "\u859F",
		0xCC29: "\u85A2",
		0xCC2A: "\u85B0",
		0xCC2B: "\u85B4",
		0xCC2C: "\u85B7",
		0xCC2D: "\u85B8",
		0xCC2E: "\u85BF",
		0xCC2F: "\u85C2",
		0xCC30: "\u85CA",
		0xCC31: "\u85CB",
		0xCC32: "\u85CE",
		0xCC33: "\u85D1",
		0xCC34: "\u85D3",
		0xCC35: "\u85E0",
		0xCC36: "\u85E6",
		0xCC37: "\u85E8",
		0xCC38: "\u85ED",
		0xCC39: "\u85F6",
		0xCC3A: "\u85FF",
		0xCC3B: "\u8600",
		0xCC3C: "\u8604",
		0xCC3D: "\u8605",
		0xCC3E: "\u8610",
		0xCC3F: "\u8611",
		0xCC40: "\u8618",
		0xCC41: "\u861E",
		0xCC42: "\u8627",
		0xCC43: "\u8629",
		0xCC44: "\u8635",
		0xCC45: "\u8638",
		0xCC46: "\u863A",
		0xCC47: "\u863C",
		0xCC48: "\u863D",
		0xCC49: "\u8646",
		0xCC4A: "\u864F",
		0xCC4B: "\u8651",
		0xCC4C: "\u8652",
		0xCC4D: "\u8653",
		0xCC4E: "\u8661",
		0xCC4F: "\u8662",
		0xCC50: "\u8663",
		0xCC51: "\u866C",
		0xCC52: "\u866E",
		0xCC53: "\u866F",
		0xCC54: "\u8675",
		0xCC55: "\u8678",
		0xCC56: "\u867A",
		0xCC57: "\u867C",
		0xCC58: "\u867D",
		0xCC59: "\u867E",
		0xCC5A: "\u867F",
		0xCC5B: "\u8680",
		0xCC5C: "\u8681",
		0xCC5D: "\u8682",
		0xCC5E: "\u8684",
		0xCC5F: "\u868D",
		0xCC60: "\u8698",
		0xCC61: "\u869B",
		0xCC62: "\u869C",
		0xCC63: "\u869D",
		0xCC64: "\u86A1",
		0xCC65: "\u86A6",
		0xCC66: "\u86A7",
		0xCC67: "\u86A8",
		0xCC68: "\u86AC",
		0xCC69: "\u86B1",
		0xCC6A: "\u86B4",
		0xCC6B: "\u86BA",
		0xCC6C: "\u86C0",
		0xCC6D: "\u86C1",
		0xCC6E: "\u86CA",
		0xCC6F: "\u86CF",
		0xCC70: "\u86D0",
		0xCC71: "\u86D1",
		0xCC72: "\u86D5",
		0xCC73: "\u86D8",
		0xCC74: "\u86F0",
		0xCC75: "\u86F1",
		0xCC76: "\u86F2",
		0xCC77: "\u86F3",
		0xCC78: "\u86F4",
		0xCC79: "\u86FA",
		0xCC7A: "\u8707",
		0xCC7B: "\u870B",
		0xCC7C: "\u870E",
		0xCC7D: "\u8710",
		0xCC7E: "\u8713",
		0xCD21: "\u8715",
		0xCD22: "\u8717",
		0xCD23: "\u871E",
		0xCD24: "\u8721",
		0xCD25: "\u8722",
		0xCD26: "\u8723",
		0xCD27: "\u8728",
		0xCD28: "\u872E",
		0xCD29: "\u872F",
		0xCD2A: "\u8730",
		0xCD2B: "\u8731",
		0xCD2C: "\u8739",
		0xCD2D: "\u873A",
		0xCD2E: "\u873D",
		0xCD2F: "\u873E",
		0xCD30: "\u8740",
		0xCD31: "\u8742",
		0xCD32: "\u8743",
		0xCD33: "\u8744",
		0xCD34: "\u8747",
		0xCD35: "\u8748",
		0xCD36: "\u8758",
		0xCD37: "\u8761",
		0xCD38: "\u8764",
		0xCD39: "\u8765",
		0xCD3A: "\u8770",
		0xCD3B: "\u8771",
		0xCD3C: "\u8772",
		0xCD3D: "\u877B",
		0xCD3E: "\u877C",
		0xCD3F: "\u877D",
		0xCD40: "\u877E",
		0xCD41: "\u8783",
		0xCD42: "\u8784",
		0xCD43: "\u8785",
		0xCD44: "\u8788",
		0xCD45: "\u8789",
		0xCD46: "\u878B",
		0xCD47: "\u8793",
		0xCD48: "\u8797",
		0xCD49: "\u8798",
		0xCD4A: "\u8799",
		0xCD4B: "\u87A0",
		0xCD4C: "\u87A3",
		0xCD4D: "\u87A8",
		0xCD4E: "\u87AC",
		0xCD4F: "\u87AD",
		0xCD50: "\u87AE",
		0xCD51: "\u87B2",
		0xCD52: "\u87B5",
		0xCD53: "\u87BE",
		0xCD54: "\u87BF",
		0xCD55: "\u87C1",
		0xCD56: "\u87C8",
		0xCD57: "\u87CA",
		0xCD58: "\u87CE",
		0xCD59: "\u87CF",
		0xCD5A: "\u87D1",
		0xCD5B: "\u87DB",
		0xCD5C: "\u87DF",
		0xCD5D: "\u87E2",
		0xCD5E: "\u87E3",
		0xCD5F: "\u87E5",
		0xCD60: "\u87EA",
		0xCD61: "\u87EB",
		0xCD62: "\u87EC",
		0xCD63: "\u87EE",
		0xCD64: "\u87FF",
		0xCD65: "\u8801",
		0xCD66: "\u8803",
		0xCD67: "\u8806",
		0xCD68: "\u880A",
		0xCD69: "\u880B",
		0xCD6A: "\u8810",
		0xCD6B: "\u8813",
		0xCD6C: "\u8814",
		0xCD6D: "\u8819",
		0xCD6E: "\u881A",
		0xCD6F: "\u881B",
		0xCD70: "\u881F",
		0xCD71: "\u8828",
		0xCD72: "\u882D",
		0xCD73: "\u882E",
		0xCD74: "\u8832",
		0xCD75: "\u8835",
		0xCD76: "\u883C",
		0xCD77: "\u8843",
		0xCD78: "\u8845",
		0xCD79: "\u8847",
		0xCD7A: "\u884A",
		0xCD7B: "\u884B",
		0xCD7C: "\u884E",
		0xCD7D: "\u884F",
		0xCD7E: "\u8854",
		0xCE21: "\u8855",
		0xCE22: "\u8856",
		0xCE23: "\u885A",
		0xCE24: "\u8860",
		0xCE25: "\u8864",
		0xCE26: "\u8865",
		0xCE27: "\u8869",
		0xCE28: "\u886C",
		0xCE29: "\u886E",
		0xCE2A: "\u8879",
		0xCE2B: "\u8884",
		0xCE2C: "\u8885",
		0xCE2D: "\u8886",
		0xCE2E: "\u888E",
		0xCE2F: "\u8898",
		0xCE30: "\u889C",
		0xCE31: "\u88A0",
		0xCE32: "\u88AA",
		0xCE33: "\u88AD",
		0xCE34: "\u88AF",
		0xCE35: "\u88BA",
		0xCE36: "\u88BC",
		0xCE37: "\u88C0",
		0xCE38: "\u88C6",
		0xCE39: "\u88C8",
		0xCE3A: "\u88C9",
		0xCE3B: "\u88CB",
		0xCE3C: "\u88CC",
		0xCE3D: "\u88CE",
		0xCE3E: "\u88D2",
		0xCE3F: "\u88E2",
		0xCE40: "\u88E3",
		0xCE41: "\u88E4",
		0xCE42: "\u88E5",
		0xCE43: "\u88E9",
		0xCE44: "\u88EF",
		0xCE45: "\u88F0",
		0xCE46: "\u88F1",
		0xCE47: "\u8903",
		0xCE48: "\u890E",
		0xCE49: "\u890F",
		0xCE4A: "\u8915",
		0xCE4B: "\u8918",
		0xCE4C: "\u8919",
		0xCE4D: "\u891A",
		0xCE4E: "\u891B",
		0xCE4F: "\u891F",
		0xCE50: "\u8921",
		0xCE51: "\u8926",
		0xCE52: "\u8927",
		0xCE53: "\u892F",
		0xCE54: "\u8930",
		0xCE55: "\u8934",
		0xCE56: "\u8935",
		0xCE57: "\u893E",
		0xCE58: "\u8946",
		0xCE59: "\u894F",
		0xCE5A: "\u8955",
		0xCE5B: "\u895C",
		0xCE5C: "\u895D",
		0xCE5D: "\u896B",
		0xCE5E: "\u896C",
		0xCE5F: "\u896E",
		0xCE60: "\u8975",
		0xCE61: "\u8976",
		0xCE62: "\u897B",
		0xCE63: "\u8982",
		0xCE64: "\u8985",
		0xCE65: "\u8994",
		0xCE66: "\u89B0",
		0xCE67: "\u89B5",
		0xCE68: "\u89BB",
		0xCE69: "\u89BC",
		0xCE6A: "\u89C1",
		0xCE6B: "\u89C2",
		0xCE6C: "\u89C3",
		0xCE6D: "\u89C4",
		0xCE6E: "\u89C5",
		0xCE6F: "\u89C6",
		0xCE70: "\u89C7",
		0xCE71: "\u89C8",
		0xCE72: "\u89C9",
		0xCE73: "\u89CA",
		0xCE74: "\u89CB",
		0xCE75: "\u89CC",
		0xCE76: "\u89CD",
		0xCE77: "\u89CE",
		0xCE78: "\u89CF",
		0xCE79: "\u89D0",
		0xCE7A: "\u89D1",
		0xCE7B: "\u89D4",
		0xCE7C: "\u89D5",
		0xCE7D: "\u89D6",
		0xCE7E: "\u89DE",
		0xCF21: "\u89E5",
		0xCF22: "\u89EB",
		0xCF23: "\u89ED",
		0xCF24: "\u89EF",
		0xCF25: "\u89F1",
		0xCF26: "\u89F3",
		0xCF27: "\u89F6",
		0xCF28: "\u89FA",
		0xCF29: "\u89FF",
		0xCF2A: "\u8A04",
		0xCF2B: "\u8A07",
		0xCF2C: "\u8A11",
		0xCF2D: "\u8A1A",
		0xCF2E: "\u8A22",
		0xCF2F: "\u8A26",
		0xCF30: "\u8A3E",
		0xCF31: "\u8A58",
		0xCF32: "\u8A5D",
		0xCF33: "\u8A5F",
		0xCF34: "\u8A67",
		0xCF35: "\u8A79",
		0xCF36: "\u8A7E",
		0xCF37: "\u8A8A",
		0xCF38: "\u8A92",
		0xCF39: "\u8A96",
		0xCF3A: "\u8AD0",
		0xCF3B: "\u8AD3",
		0xCF3C: "\u8AF6",
		0xCF3D: "\u8B05",
		0xCF3E: "\u8B37",
		0xCF3F: "\u8B3F",
		0xCF40: "\u8B4A",
		0xCF41: "\u8B4D",
		0xCF42: "\u8B59",
		0xCF43: "\u8B6D",
		0xCF44: "\u8B78",
		0xCF45: "\u8B7E",
		0xCF46: "\u8B81",
		0xCF47: "\u8B85",
		0xCF48: "\u8B86",
		0xCF49: "\u8B8B",
		0xCF4A: "\u8B94",
		0xCF4B: "\u8B9B",
		0xCF4C: "\u8B9C",
		0xCF4D: "\u8B9E",
		0xCF4E: "\u8B9F",
		0xCF4F: "\u8BA1",
		0xCF50: "\u8BA2",
		0xCF51: "\u8BA3",
		0xCF52: "\u8BA4",
		0xCF53: "\u8BA5",
		0xCF54: "\u8BA6",
		0xCF55: "\u8BA7",
		0xCF56: "\u8BA8",
		0xCF57: "\u8BA9",
		0xCF58: "\u8BAA",
		0xCF59: "\u8BAB",
		0xCF5A: "\u8BAD",
		0xCF5B: "\u8BAE",
		0xCF5C: "\u8BAF",
		0xCF5D: "\u8BB0",
		0xCF5E: "\u8BB1",
		0xCF5F: "\u8BB2",
		0xCF60: "\u8BB3",
		0xCF61: "\u8BB4",
		0xCF62: "\u8BB5",
		0xCF63: "\u8BB6",
		0xCF64: "\u8BB7",
		0xCF65: "\u8BB8",
		0xCF66: "\u8BB9",
		0xCF67: "\u8BBA",
		0xCF68: "\u8BBB",
		0xCF69: "\u8BBC",
		0xCF6A: "\u8BBD",
		0xCF6B: "\u8BBE",
		0xCF6C: "\u8BBF",
		0xCF6D: "\u8BC0",
		0xCF6E: "\u8BC1",
		0xCF6F: "\u8BC2",
		0xCF70: "\u8BC3",
		0xCF71: "\u8BC4",
		0xCF72: "\u8BC5",
		0xCF73: "\u8BC6",
		0xCF74: "\u8BC7",
		0xCF75: "\u8BC8",
		0xCF76: "\u8BC9",
		0xCF77: "\u8BCA",
		0xCF78: "\u8BCB",
		0xCF79: "\u8BCC",
		0xCF7A: "\u8BCD",
		0xCF7B: "\u8BCE",
		0xCF7C: "\u8BCF",
		0xCF7D: "\u8BD0",
		0xCF7E: "\u8BD1",
		0xD021: "\u8BD2",
		0xD022: "\u8BD3",
		0xD023: "\u8BD4",
		0xD024: "\u8BD5",
		0xD025: "\u8BD6",
		0xD026: "\u8BD7",
		0xD027: "\u8BD8",
		0xD028: "\u8BD9",
		0xD029: "\u8BDA",
		0xD02A: "\u8BDB",
		0xD02B: "\u8BDC",
		0xD02C: "\u8BDD",
		0xD02D: "\u8BDE",
		0xD02E: "\u8BDF",
		0xD02F: "\u8BE0",
		0xD030: "\u8BE1",
		0xD031: "\u8BE2",
		0xD032: "\u8BE3",
		0xD033: "\u8BE4",
		0xD034: "\u8BE5",
		0xD035: "\u8BE6",
		0xD036: "\u8BE7",
		0xD037: "\u8BE8",
		0xD038: "\u8BE9",
		0xD039: "\u8BEA",
		0xD03A: "\u8BEB",
		0xD03B: "\u8BEC",
		0xD03C: "\u8BED",
		0xD03D: "\u8BEE",
		0xD03E: "\u8BEF",
		0xD03F: "\u8BF0",
		0xD040: "\u8BF1",
		0xD041: "\u8BF2",
		0xD042: "\u8BF3",
		0xD043: "\u8BF4",
		0xD044: "\u8BF5",
		0xD045: "\u8BF7",
		0xD046: "\u8BF8",
		0xD047: "\u8BF9",
		0xD048: "\u8BFA",
		0xD049: "\u8BFB",
		0xD04A: "\u8BFC",
		0xD04B: "\u8BFD",
		0xD04C: "\u8BFE",
		0xD04D: "\u8BFF",
		0xD04E: "\u8C00",
		0xD04F: "\u8C01",
		0xD050: "\u8C02",
		0xD051: "\u8C03",
		0xD052: "\u8C04",
		0xD053: "\u8C05",
		0xD054: "\u8C06",
		0xD055: "\u8C07",
		0xD056: "\u8C08",
		0xD057: "\u8C0A",
		0xD058: "\u8C0B",
		0xD059: "\u8C0C",
		0xD05A: "\u8C0D",
		0xD05B: "\u8C0E",
		0xD05C: "\u8C0F",
		0xD05D: "\u8C10",
		0xD05E: "\u8C11",
		0xD05F: "\u8C12",
		0xD060: "\u8C13",
		0xD061: "\u8C14",
		0xD062: "\u8C15",
		0xD063: "\u8C16",
		0xD064: "\u8C17",
		0xD065: "\u8C18",
		0xD066: "\u8C19",
		0xD067: "\u8C1A",
		0xD068: "\u8C1B",
		0xD069: "\u8C1C",
		0xD06A: "\u8C1D",
		0xD06B: "\u8C1E",
		0xD06C: "\u8C1F",
		0xD06D: "\u8C20",
		0xD06E: "\u8C21",
		0xD06F: "\u8C22",
		0xD070: "\u8C23",
		0xD071: "\u8C24",
		0xD072: "\u8C25",
		0xD073: "\u8C26",
		0xD074: "\u8C27",
		0xD075: "\u8C28",
		0xD076: "\u8C29",
		0xD077: "\u8C2A",
		0xD078: "\u8C2B",
		0xD079: "\u8C2C",
		0xD07A: "\u8C2D",
		0xD07B: "\u8C2E",
		0xD07C: "\u8C2F",
		0xD07D: "\u8C30",
		0xD07E: "\u8C31",
		0xD121: "\u8C32",
		0xD122: "\u8C33",
		0xD123: "\u8C34",
		0xD124: "\u8C35",
		0xD125: "\u8C36",
		0xD126: "\u8C40",
		0xD127: "\u8C47",
		0xD128: "\u8C49",
		0xD129: "\u8C54",
		0xD12A: "\u8C57",
		0xD12B: "\u8C58",
		0xD12C: "\u8C5D",
		0xD12D: "\u8C68",
		0xD12E: "\u8C6D",
		0xD12F: "\u8C6E",
		0xD130: "\u8C73",
		0xD131: "\u8C86",
		0xD132: "\u8C90",
		0xD133: "\u8C93",
		0xD134: "\u8C9B",
		0xD135: "\u8CC9",
		0xD136: "\u8CD4",
		0xD137: "\u8CE1",
		0xD138: "\u8CEC",
		0xD139: "\u8CEE",
		0xD13A: "\u8CF8",
		0xD13B: "\u8D17",
		0xD13C: "\u8D1B",
		0xD13D: "\u8D1C",
		0xD13E: "\u8D1D",
		0xD13F: "\u8D1E",
		0xD140: "\u8D1F",
		0xD141: "\u8D21",
		0xD142: "\u8D22",
		0xD143: "\u8D23",
		0xD144: "\u8D24",
		0xD145: "\u8D25",
		0xD146: "\u8D26",
		0xD147: "\u8D27",
		0xD148: "\u8D28",
		0xD149: "\u8D29",
		0xD14A: "\u8D2A",
		0xD14B: "\u8D2B",
		0xD14C: "\u8D2C",
		0xD14D: "\u8D2D",
		0xD14E: "\u8D2E",
		0xD14F: "\u8D2F",
		0xD150: "\u8D30",
		0xD151: "\u8D31",
		0xD152: "\u8D32",
		0xD153: "\u8D33",
		0xD154: "\u8D34",
		0xD155: "\u8D35",
		0xD156: "\u8D36",
		0xD157: "\u8D37",
		0xD158: "\u8D38",
		0xD159: "\u8D39",
		0xD15A: "\u8D3A",
		0xD15B: "\u8D3B",
		0xD15C: "\u8D3C",
		0xD15D: "\u8D3D",
		0xD15E: "\u8D3E",
		0xD15F: "\u8D3F",
		0xD160: "\u8D40",
		0xD161: "\u8D41",
		0xD162: "\u8D42",
		0xD163: "\u8D43",
		0xD164: "\u8D44",
		0xD165: "\u8D45",
		0xD166: "\u8D46",
		0xD167: "\u8D47",
		0xD168: "\u8D48",
		0xD169: "\u8D49",
		0xD16A: "\u8D4A",
		0xD16B: "\u8D4B",
		0xD16C: "\u8D4C",
		0xD16D: "\u8D4D",
		0xD16E: "\u8D4E",
		0xD16F: "\u8D4F",
		0xD170: "\u8D50",
		0xD171: "\u8D51",
		0xD172: "\u8D52",
		0xD173: "\u8D53",
		0xD174: "\u8D54",
		0xD175: "\u8D55",
		0xD176: "\u8D56",
		0xD177: "\u8D57",
		0xD178: "\u8D58",
		0xD179: "\u8D59",
		0xD17A: "\u8D5A",
		0xD17B: "\u8D5B",
		0xD17C: "\u8D5C",
		0xD17D: "\u8D5D",
		0xD17E: "\u8D5E",
		0xD221: "\u8D60",
		0xD222: "\u8D61",
		0xD223: "\u8D62",
		0xD224: "\u8D63",
		0xD225: "\u8D69",
		0xD226: "\u8D6A",
		0xD227: "\u8D6F",
		0xD228: "\u8D75",
		0xD229: "\u8D76",
		0xD22A: "\u8D78",
		0xD22B: "\u8D82",
		0xD22C: "\u8D84",
		0xD22D: "\u8D8B",
		0xD22E: "\u8D91",
		0xD22F: "\u8D94",
		0xD230: "\u8D95",
		0xD231: "\u8D9F",
		0xD232: "\u8DA6",
		0xD233: "\u8DAF",
		0xD234: "\u8DB1",
		0xD235: "\u8DB4",
		0xD236: "\u8DB5",
		0xD237: "\u8DB8",
		0xD238: "\u8DBB",
		0xD239: "\u8DBC",
		0xD23A: "\u8DBF",
		0xD23B: "\u8DC0",
		0xD23C: "\u8DC3",
		0xD23D: "\u8DC4",
		0xD23E: "\u8DC5",
		0xD23F: "\u8DC6",
		0xD240: "\u8DCE",
		0xD241: "\u8DD0",
		0xD242: "\u8DD1",
		0xD243: "\u8DD4",
		0xD244: "\u8DD5",
		0xD245: "\u8DD7",
		0xD246: "\u8DDE",
		0xD247: "\u8DE2",
		0xD248: "\u8DE4",
		0xD249: "\u8DE5",
		0xD24A: "\u8DE9",
		0xD24B: "\u8DEC",
		0xD24C: "\u8DF0",
		0xD24D: "\u8DF2",
		0xD24E: "\u8DF4",
		0xD24F: "\u8DF6",
		0xD250: "\u8DF7",
		0xD251: "\u8DF8",
		0xD252: "\u8DF9",
		0xD253: "\u8DFA",
		0xD254: "\u8DFB",
		0xD255: "\u8DFD",
		0xD256: "\u8E05",
		0xD257: "\u8E06",
		0xD258: "\u8E0C",
		0xD259: "\u8E12",
		0xD25A: "\u8E14",
		0xD25B: "\u8E16",
		0xD25C: "\u8E20",
		0xD25D: "\u8E21",
		0xD25E: "\u8E22",
		0xD25F: "\u8E23",
		0xD260: "\u8E26",
		0xD261: "\u8E27",
		0xD262: "\u8E29",
		0xD263: "\u8E2B",
		0xD264: "\u8E2C",
		0xD265: "\u8E2E",
		0xD266: "\u8E2F",
		0xD267: "\u8E31",
		0xD268: "\u8E33",
		0xD269: "\u8E36",
		0xD26A: "\u8E38",
		0xD26B: "\u8E39",
		0xD26C: "\u8E3A",
		0xD26D: "\u8E3D",
		0xD26E: "\u8E40",
		0xD26F: "\u8E41",
		0xD270: "\u8E45",
		0xD271: "\u8E4B",
		0xD272: "\u8E4D",
		0xD273: "\u8E4E",
		0xD274: "\u8E4F",
		0xD275: "\u8E51",
		0xD276: "\u8E52",
		0xD277: "\u8E53",
		0xD278: "\u8E5A",
		0xD279: "\u8E5C",
		0xD27A: "\u8E61",
		0xD27B: "\u8E62",
		0xD27C: "\u8E66",
		0xD27D: "\u8E69",
		0xD27E: "\u8E6C",
		0xD321: "\u8E6D",
		0xD322: "\u8E6F",
		0xD323: "\u8E70",
		0xD324: "\u8E75",
		0xD325: "\u8E77",
		0xD326: "\u8E7A",
		0xD327: "\u8E7B",
		0xD328: "\u8E7D",
		0xD329: "\u8E7E",
		0xD32A: "\u8E7F",
		0xD32B: "\u8E82",
		0xD32C: "\u8E83",
		0xD32D: "\u8E88",
		0xD32E: "\u8E89",
		0xD32F: "\u8E8F",
		0xD330: "\u8E90",
		0xD331: "\u8E92",
		0xD332: "\u8E95",
		0xD333: "\u8E97",
		0xD334: "\u8E9A",
		0xD335: "\u8E9C",
		0xD336: "\u8E9E",
		0xD337: "\u8EA5",
		0xD338: "\u8EAD",
		0xD339: "\u8EB2",
		0xD33A: "\u8EB3",
		0xD33B: "\u8EB6",
		0xD33C: "\u8EBA",
		0xD33D: "\u8EC0",
		0xD33E: "\u8EC3",
		0xD33F: "\u8EE4",
		0xD340: "\u8F2D",
		0xD341: "\u8F66",
		0xD342: "\u8F67",
		0xD343: "\u8F68",
		0xD344: "\u8F69",
		0xD345: "\u8F6A",
		0xD346: "\u8F6B",
		0xD347: "\u8F6C",
		0xD348: "\u8F6D",
		0xD349: "\u8F6E",
		0xD34A: "\u8F6F",
		0xD34B: "\u8F70",
		0xD34C: "\u8F71",
		0xD34D: "\u8F72",
		0xD34E: "\u8F73",
		0xD34F: "\u8F74",
		0xD350: "\u8F75",
		0xD351: "\u8F76",
		0xD352: "\u8F77",
		0xD353: "\u8F78",
		0xD354: "\u8F79",
		0xD355: "\u8F7A",
		0xD356: "\u8F7B",
		0xD357: "\u8F7C",
		0xD358: "\u8F7D",
		0xD359: "\u8F7E",
		0xD35A: "\u8F7F",
		0xD35B: "\u8F80",
		0xD35C: "\u8F81",
		0xD35D: "\u8F82",
		0xD35E: "\u8F83",
		0xD35F: "\u8F84",
		0xD360: "\u8F85",
		0xD361: "\u8F86",
		0xD362: "\u8F87",
		0xD363: "\u8F88",
		0xD364: "\u8F89",
		0xD365: "\u8F8A",
		0xD366: "\u8F8B",
		0xD367: "\u8F8C",
		0xD368: "\u8F8D",
		0xD369: "\u8F8E",
		0xD36A: "\u8F8F",
		0xD36B: "\u8F90",
		0xD36C: "\u8F91",
		0xD36D: "\u8F92",
		0xD36E: "\u8F93",
		0xD36F: "\u8F94",
		0xD370: "\u8F95",
		0xD371: "\u8F96",
		0xD372: "\u8F97",
		0xD373: "\u8F98",
		0xD374: "\u8F99",
		0xD375: "\u8F9A",
		0xD376: "\u8FA0",
		0xD377: "\u8FA4",
		0xD378: "\u8FA6",
		0xD379: "\u8FA9",
		0xD37A: "\u8FAB",
		0xD37B: "\u8FB3",
		0xD37C: "\u8FB5",
		0xD37D: "\u8FB9",
		0xD37E: "\u8FBD",
		0xD421: "\u8FBE",
		0xD422: "\u8FC1",
		0xD423: "\u8FC6",
		0xD424: "\u8FC7",
		0xD425: "\u8FC8",
		0xD426: "\u8FCD",
		0xD427: "\u8FD0",
		0xD428: "\u8FD2",
		0xD429: "\u8FD3",
		0xD42A: "\u8FD5",
		0xD42B: "\u8FD8",
		0xD42C: "\u8FD9",
		0xD42D: "\u8FDB",
		0xD42E: "\u8FDC",
		0xD42F: "\u8FDD",
		0xD430: "\u8FDE",
		0xD431: "\u8FDF",
		0xD432: "\u8FE4",
		0xD433: "\u8FE8",
		0xD434: "\u8FEE",
		0xD435: "\u8FFB",
		0xD436: "\u9002",
		0xD437: "\u9004",
		0xD438: "\u9009",
		0xD439: "\u900A",
		0xD43A: "\u9012",
		0xD43B: "\u901B",
		0xD43C: "\u9026",
		0xD43D: "\u9029",
		0xD43E: "\u902D",
		0xD43F: "\u902F",
		0xD440: "\u9034",
		0xD441: "\u9037",
		0xD442: "\u903B",
		0xD443: "\u9044",
		0xD444: "\u9046",
		0xD445: "\u904C",
		0xD446: "\u9057",
		0xD447: "\u905B",
		0xD448: "\u905D",
		0xD449: "\u9062",
		0xD44A: "\u9074",
		0xD44B: "\u9079",
		0xD44C: "\u9085",
		0xD44D: "\u9088",
		0xD44E: "\u908B",
		0xD44F: "\u9090",
		0xD450: "\u9093",
		0xD451: "\u9095",
		0xD452: "\u9097",
		0xD453: "\u9098",
		0xD454: "\u9099",
		0xD455: "\u909B",
		0xD456: "\u909D",
		0xD457: "\u90A0",
		0xD458: "\u90A1",
		0xD459: "\u90A2",
		0xD45A: "\u90AC",
		0xD45B: "\u90AE",
		0xD45C: "\u90B0",
		0xD45D: "\u90B2",
		0xD45E: "\u90B3",
		0xD45F: "\u90B4",
		0xD460: "\u90B6",
		0xD461: "\u90B9",
		0xD462: "\u90BA",
		0xD463: "\u90BB",
		0xD464: "\u90BD",
		0xD465: "\u90BE",
		0xD466: "\u90C3",
		0xD467: "\u90C4",
		0xD468: "\u90C5",
		0xD469: "\u90C7",
		0xD46A: "\u90C8",
		0xD46B: "\u90CB",
		0xD46C: "\u90CF",
		0xD46D: "\u90D0",
		0xD46E: "\u90D1",
		0xD46F: "\u90D3",
		0xD470: "\u90D5",
		0xD471: "\u90D7",
		0xD472: "\u90DA",
		0xD473: "\u90DC",
		0xD474: "\u90DD",
		0xD475: "\u90DF",
		0xD476: "\u90E6",
		0xD477: "\u90E7",
		0xD478: "\u90EA",
		0xD479: "\u90EB",
		0xD47A: "\u90EF",
		0xD47B: "\u90F0",
		0xD47C: "\u90F3",
		0xD47D: "\u90F4",
		0xD47E: "\u90F8",
		0xD521: "\u90FE",
		0xD522: "\u90FF",
		0xD523: "\u9100",
		0xD524: "\u9103",
		0xD525: "\u9104",
		0xD526: "\u9105",
		0xD527: "\u9106",
		0xD528: "\u9109",
		0xD529: "\u910C",
		0xD52A: "\u910F",
		0xD52B: "\u9114",
		0xD52C: "\u9116",
		0xD52D: "\u9117",
		0xD52E: "\u9118",
		0xD52F: "\u911A",
		0xD530: "\u911C",
		0xD531: "\u911E",
		0xD532: "\u9120",
		0xD533: "\u9122",
		0xD534: "\u9123",
		0xD535: "\u9127",
		0xD536: "\u9129",
		0xD537: "\u912F",
		0xD538: "\u9131",
		0xD539: "\u9134",
		0xD53A: "\u9136",
		0xD53B: "\u9139",
		0xD53C: "\u913A",
		0xD53D: "\u913E",
		0xD53E: "\u9142",
		0xD53F: "\u9143",
		0xD540: "\u9146",
		0xD541: "\u9148",
		0xD542: "\u914F",
		0xD543: "\u9150",
		0xD544: "\u9155",
		0xD545: "\u9157",
		0xD546: "\u915A",
		0xD547: "\u915D",
		0xD548: "\u915E",
		0xD549: "\u9161",
		0xD54A: "\u9164",
		0xD54B: "\u9166",
		0xD54C: "\u9167",
		0xD54D: "\u916E",
		0xD54E: "\u916F",
		0xD54F: "\u9170",
		0xD550: "\u9171",
		0xD551: "\u9174",
		0xD552: "\u9176",
		0xD553: "\u9179",
		0xD554: "\u917A",
		0xD555: "\u917D",
		0xD556: "\u917E",
		0xD557: "\u917F",
		0xD558: "\u9181",
		0xD559: "\u9183",
		0xD55A: "\u9184",
		0xD55B: "\u9185",
		0xD55C: "\u918A",
		0xD55D: "\u918C",
		0xD55E: "\u9191",
		0xD55F: "\u9196",
		0xD560: "\u919A",
		0xD561: "\u919B",
		0xD562: "\u91A1",
		0xD563: "\u91A3",
		0xD564: "\u91A8",
		0xD565: "\u91AC",
		0xD566: "\u91AD",
		0xD567: "\u91AE",
		0xD568: "\u91B0",
		0xD569: "\u91B1",
		0xD56A: "\u91B2",
		0xD56B: "\u91BB",
		0xD56C: "\u91BC",
		0xD56D: "\u91BD",
		0xD56E: "\u91BE",
		0xD56F: "\u91BF",
		0xD570: "\u91C3",
		0xD571: "\u91C4",
		0xD572: "\u91C5",
		0xD573: "\u91CA",
		0xD574: "\u91EC",
		0xD575: "\u9204",
		0xD576: "\u9238",
		0xD577: "\u9274",
		0xD578: "\u928E",
		0xD579: "\u9297",
		0xD57A: "\u92AE",
		0xD57B: "\u92B2",
		0xD57C: "\u92C6",
		0xD57D: "\u92C8",
		0xD57E: "\u92DC",
		0xD621: "\u92EC",
		0xD622: "\u92F9",
		0xD623: "\u930D",
		0xD624: "\u9327",
		0xD625: "\u9336",
		0xD626: "\u9338",
		0xD627: "\u933E",
		0xD628: "\u934A",
		0xD629: "\u935A",
		0xD62A: "\u936A",
		0xD62B: "\u936B",
		0xD62C: "\u938B",
		0xD62D: "\u938F",
		0xD62E: "\u9392",
		0xD62F: "\u93A2",
		0xD630: "\u93A3",
		0xD631: "\u93A9",
		0xD632: "\u93C7",
		0xD633: "\u93CA",
		0xD634: "\u93DA",
		0xD635: "\u93DF",
		0xD636: "\u93E6",
		0xD637: "\u93F5",
		0xD638: "\u93FA",
		0xD639: "\u93FD",
		0xD63A: "\u940B",
		0xD63B: "\u9412",
		0xD63C: "\u941D",
		0xD63D: "\u9425",
		0xD63E: "\u943E",
		0xD63F: "\u944C",
		0xD640: "\u9455",
		0xD641: "\u9464",
		0xD642: "\u946B",
		0xD643: "\u9472",
		0xD644: "\u9479",
		0xD645: "\u9482",
		0xD646: "\u9485",
		0xD647: "\u9486",
		0xD648: "\u9487",
		0xD649: "\u9488",
		0xD64A: "\u9489",
		0xD64B: "\u948A",
		0xD64C: "\u948B",
		0xD64D: "\u948C",
		0xD64E: "\u948D",
		0xD64F: "\u948E",
		0xD650: "\u948F",
		0xD651: "\u9490",
		0xD652: "\u9492",
		0xD653: "\u9493",
		0xD654: "\u9494",
		0xD655: "\u9495",
		0xD656: "\u9496",
		0xD657: "\u9497",
		0xD658: "\u9498",
		0xD659: "\u9499",
		0xD65A: "\u949A",
		0xD65B: "\u949B",
		0xD65C: "\u949C",
		0xD65D: "\u949D",
		0xD65E: "\u949E",
		0xD65F: "\u949F",
		0xD660: "\u94A0",
		0xD661: "\u94A1",
		0xD662: "\u94A2",
		0xD663: "\u94A3",
		0xD664: "\u94A4",
		0xD665: "\u94A5",
		0xD666: "\u94A6",
		0xD667: "\u94A7",
		0xD668: "\u94A8",
		0xD669: "\u94A9",
		0xD66A: "\u94AA",
		0xD66B: "\u94AB",
		0xD66C: "\u94AC",
		0xD66D: "\u94AD",
		0xD66E: "\u94AE",
		0xD66F: "\u94AF",
		0xD670: "\u94B0",
		0xD671: "\u94B1",
		0xD672: "\u94B2",
		0xD673: "\u94B3",
		0xD674: "\u94B4",
		0xD675: "\u94B5",
		0xD676: "\u94B6",
		0xD677: "\u94B7",
		0xD678: "\u94B9",
		0xD679: "\u94BA",
		0xD67A: "\u94BB",
		0xD67B: "\u94BC",
		0xD67C: "\u94BD",
		0xD67D: "\u94BE",
		0xD67E: "\u94BF",
		0xD721: "\u94C0",
		0xD722: "\u94C1",
		0xD723: "\u94C2",
		0xD724: "\u94C3",
		0xD725: "\u94C4",
		0xD726: "\u94C5",
		0xD727: "\u94C6",
		0xD728: "\u94C8",
		0xD729: "\u94C9",
		0xD72A: "\u94CA",
		0xD72B: "\u94CB",
		0xD72C: "\u94CC",
		0xD72D: "\u94CD",
		0xD72E: "\u94CE",
		0xD72F: "\u94CF",
		0xD730: "\u94D0",
		0xD731: "\u94D1",
		0xD732: "\u94D2",
		0xD733: "\u94D3",
		0xD734: "\u94D5",
		0xD735: "\u94D6",
		0xD736: "\u94D7",
		0xD737: "\u94D8",
		0xD738: "\u94D9",
		0xD739: "\u94DA",
		0xD73A: "\u94DB",
		0xD73B: "\u94DC",
		0xD73C: "\u94DD",
		0xD73D: "\u94DE",
		0xD73E: "\u94DF",
		0xD73F: "\u94E0",
		0xD740: "\u94E1",
		0xD741: "\u94E2",
		0xD742: "\u94E3",
		0xD743: "\u94E4",
		0xD744: "\u94E5",
		0xD745: "\u94E6",
		0xD746: "\u94E7",
		0xD747: "\u94E8",
		0xD748: "\u94E9",
		0xD749: "\u94EA",
		0xD74A: "\u94EB",
		0xD74B: "\u94EC",
		0xD74C: "\u94ED",
		0xD74D: "\u94EE",
		0xD74E: "\u94EF",
		0xD74F: "\u94F0",
		0xD750: "\u94F1",
		0xD751: "\u94F2",
		0xD752: "\u94F3",
		0xD753: "\u94F4",
		0xD754: "\u94F5",
		0xD755: "\u94F6",
		0xD756: "\u94F7",
		0xD757: "\u94F8",
		0xD758: "\u94F9",
		0xD759: "\u94FA",
		0xD75A: "\u94FB",
		0xD75B: "\u94FC",
		0xD75C: "\u94FD",
		0xD75D: "\u94FE",
		0xD75E: "\u94FF",
		0xD75F: "\u9500",
		0xD760: "\u9501",
		0xD761: "\u9502",
		0xD762: "\u9503",
		0xD763: "\u9504",
		0xD764: "\u9505",
		0xD765: "\u9506",
		0xD766: "\u9507",
		0xD767: "\u9508",
		0xD768: "\u9509",
		0xD769: "\u950A",
		0xD76A: "\u950B",
		0xD76B: "\u950C",
		0xD76C: "\u950E",
		0xD76D: "\u950F",
		0xD76E: "\u9510",
		0xD76F: "\u9511",
		0xD770: "\u9512",
		0xD771: "\u9513",
		0xD772: "\u9514",
		0xD773: "\u9515",
		0xD774: "\u9516",
		0xD775: "\u9517",
		0xD776: "\u9518",
		0xD777: "\u9519",
		0xD778: "\u951A",
		0xD779: "\u951B",
		0xD77A: "\u951C",
		0xD77B: "\u951D",
		0xD77C: "\u951E",
		0xD77D: "\u951F",
		0xD77E: "\u9521",
		0xD821: "\u9522",
		0xD822: "\u9523",
		0xD823: "\u9524",
		0xD824: "\u9525",
		0xD825: "\u9526",
		0xD826: "\u9527",
		0xD827: "\u9528",
		0xD828: "\u9529",
		0xD829: "\u952A",
		0xD82A: "\u952B",
		0xD82B: "\u952C",
		0xD82C: "\u952D",
		0xD82D: "\u952E",
		0xD82E: "\u952F",
		0xD82F: "\u9530",
		0xD830: "\u9531",
		0xD831: "\u9532",
		0xD832: "\u9533",
		0xD833: "\u9534",
		0xD834: "\u9535",
		0xD835: "\u9536",
		0xD836: "\u9537",
		0xD837: "\u9538",
		0xD838: "\u9539",
		0xD839: "\u953B",
		0xD83A: "\u953C",
		0xD83B: "\u953D",
		0xD83C: "\u953E",
		0xD83D: "\u953F",
		0xD83E: "\u9540",
		0xD83F: "\u9541",
		0xD840: "\u9542",
		0xD841: "\u9543",
		0xD842: "\u9544",
		0xD843: "\u9545",
		0xD844: "\u9546",
		0xD845: "\u9547",
		0xD846: "\u9548",
		0xD847: "\u9549",
		0xD848: "\u954A",
		0xD849: "\u954B",
		0xD84A: "\u954C",
		0xD84B: "\u954D",
		0xD84C: "\u954E",
		0xD84D: "\u954F",
		0xD84E: "\u9550",
		0xD84F: "\u9551",
		0xD850: "\u9552",
		0xD851: "\u9553",
		0xD852: "\u9554",
		0xD853: "\u9555",
		0xD854: "\u9556",
		0xD855: "\u9557",
		0xD856: "\u9558",
		0xD857: "\u955A",
		0xD858: "\u955B",
		0xD859: "\u955C",
		0xD85A: "\u955D",
		0xD85B: "\u955E",
		0xD85C: "\u9560",
		0xD85D: "\u9561",
		0xD85E: "\u9562",
		0xD85F: "\u9563",
		0xD860: "\u9564",
		0xD861: "\u9565",
		0xD862: "\u9566",
		0xD863: "\u9567",
		0xD864: "\u9568",
		0xD865: "\u9569",
		0xD866: "\u956A",
		0xD867: "\u956B",
		0xD868: "\u956C",
		0xD869: "\u956D",
		0xD86A: "\u956F",
		0xD86B: "\u9570",
		0xD86C: "\u9571",
		0xD86D: "\u9572",
		0xD86E: "\u9573",
		0xD86F: "\u9574",
		0xD870: "\u9575",
		0xD871: "\u9576",
		0xD872: "\u957F",
		0xD873: "\u9586",
		0xD874: "\u958E",
		0xD875: "\u9592",
		0xD876: "\u95A9",
		0xD877: "\u95C6",
		0xD878: "\u95C8",
		0xD879: "\u95D3",
		0xD87A: "\u95DA",
		0xD87B: "\u95DD",
		0xD87C: "\u95DE",
		0xD87D: "\u95E8",
		0xD87E: "\u95E9",
		0xD921: "\u95EA",
		0xD922: "\u95EC",
		0xD923: "\u95ED",
		0xD924: "\u95EE",
		0xD925: "\u95EF",
		0xD926: "\u95F0",
		0xD927: "\u95F1",
		0xD928: "\u95F2",
		0xD929: "\u95F3",
		0xD92A: "\u95F4",
		0xD92B: "\u95F5",
		0xD92C: "\u95F6",
		0xD92D: "\u95F7",
		0xD92E: "\u95F8",
		0xD92F: "\u95F9",
		0xD930: "\u95FA",
		0xD931: "\u95FB",
		0xD932: "\u95FC",
		0xD933: "\u95FD",
		0xD934: "\u95FE",
		0xD935: "\u95FF",
		0xD936: "\u9600",
		0xD937: "\u9601",
		0xD938: "\u9602",
		0xD939: "\u9603",
		0xD93A: "\u9604",
		0xD93B: "\u9605",
		0xD93C: "\u9606",
		0xD93D: "\u9607",
		0xD93E: "\u9608",
		0xD93F: "\u9609",
		0xD940: "\u960A",
		0xD941: "\u960B",
		0xD942: "\u960C",
		0xD943: "\u960D",
		0xD944: "\u960E",
		0xD945: "\u960F",
		0xD946: "\u9610",
		0xD947: "\u9611",
		0xD948: "\u9612",
		0xD949: "\u9613",
		0xD94A: "\u9614",
		0xD94B: "\u9615",
		0xD94C: "\u9616",
		0xD94D: "\u9617",
		0xD94E: "\u9618",
		0xD94F: "\u9619",
		0xD950: "\u961A",
		0xD951: "\u961B",
		0xD952: "\u961D",
		0xD953: "\u961F",
		0xD954: "\u9622",
		0xD955: "\u9627",
		0xD956: "\u962C",
		0xD957: "\u9631",
		0xD958: "\u9633",
		0xD959: "\u9634",
		0xD95A: "\u9635",
		0xD95B: "\u9636",
		0xD95C: "\u963C",
		0xD95D: "\u963D",
		0xD95E: "\u9641",
		0xD95F: "\u9645",
		0xD960: "\u9646",
		0xD961: "\u9647",
		0xD962: "\u9648",
		0xD963: "\u9649",
		0xD964: "\u9652",
		0xD965: "\u9654",
		0xD966: "\u9655",
		0xD967: "\u9657",
		0xD968: "\u9658",
		0xD969: "\u9661",
		0xD96A: "\u9667",
		0xD96B: "\u9668",
		0xD96C: "\u9669",
		0xD96D: "\u9674",
		0xD96E: "\u967F",
		0xD96F: "\u9682",
		0xD970: "\u9684",
		0xD971: "\u9690",
		0xD972: "\u9691",
		0xD973: "\u9696",
		0xD974: "\u969E",
		0xD975: "\u969F",
		0xD976: "\u96A9",
		0xD977: "\u96B3",
		0xD978: "\u96BD",
		0xD979: "\u96BE",
		0xD97A: "\u96CF",
		0xD97B: "\u96D2",
		0xD97C: "\u96DD",
		0xD97D: "\u96DE",
		0xD97E: "\u96E0",
		0xDA21: "\u96E9",
		0xDA22: "\u96EF",
		0xDA23: "\u96F1",
		0xDA24: "\u96F3",
		0xDA25: "\u96FE",
		0xDA26: "\u9701",
		0xDA27: "\u9702",
		0xDA28: "\u9705",
		0xDA29: "\u9709",
		0xDA2A: "\u971B",
		0xDA2B: "\u9721",
		0xDA2C: "\u9722",
		0xDA2D: "\u9728",
		0xDA2E: "\u972B",
		0xDA2F: "\u972D",
		0xDA30: "\u9734",
		0xDA31: "\u9736",
		0xDA32: "\u9753",
		0xDA33: "\u975B",
		0xDA34: "\u9763",
		0xDA35: "\u9765",
		0xDA36: "\u976A",
		0xDA37: "\u976C",
		0xDA38: "\u9770",
		0xDA39: "\u9773",
		0xDA3A: "\u9776",
		0xDA3B: "\u9777",
		0xDA3C: "\u9778",
		0xDA3D: "\u977D",
		0xDA3E: "\u977F",
		0xDA3F: "\u9780",
		0xDA40: "\u9789",
		0xDA41: "\u978C",
		0xDA42: "\u9791",
		0xDA43: "\u9792",
		0xDA44: "\u9793",
		0xDA45: "\u9794",
		0xDA46: "\u979A",
		0xDA47: "\u979B",
		0xDA48: "\u979D",
		0xDA49: "\u979E",
		0xDA4A: "\u979F",
		0xDA4B: "\u97A1",
		0xDA4C: "\u97A5",
		0xDA4D: "\u97A7",
		0xDA4E: "\u97AC",
		0xDA4F: "\u97AF",
		0xDA50: "\u97B2",
		0xDA51: "\u97B5",
		0xDA52: "\u97B6",
		0xDA53: "\u97B9",
		0xDA54: "\u97BD",
		0xDA55: "\u97BE",
		0xDA56: "\u97BF",
		0xDA57: "\u97C1",
		0xDA58: "\u97C2",
		0xDA59: "\u97C9",
		0xDA5A: "\u97CC",
		0xDA5B: "\u97CD",
		0xDA5C: "\u97D4",
		0xDA5D: "\u97D9",
		0xDA5E: "\u97DB",
		0xDA5F: "\u97DE",
		0xDA60: "\u97E1",
		0xDA61: "\u97E4",
		0xDA62: "\u97E6",
		0xDA63: "\u97E7",
		0xDA64: "\u97E8",
		0xDA65: "\u97E9",
		0xDA66: "\u97EA",
		0xDA67: "\u97EB",
		0xDA68: "\u97EC",
		0xDA69: "\u980A",
		0xDA6A: "\u9830",
		0xDA6B: "\u9853",
		0xDA6C: "\u9856",
		0xDA6D: "\u9857",
		0xDA6E: "\u9862",
		0xDA6F: "\u9865",
		0xDA70: "\u986C",
		0xDA71: "\u9875",
		0xDA72: "\u9876",
		0xDA73: "\u9877",
		0xDA74: "\u9878",
		0xDA75: "\u9879",
		0xDA76: "\u987A",
		0xDA77: "\u987B",
		0xDA78: "\u987C",
		0xDA79: "\u987D",
		0xDA7A: "\u987E",
		0xDA7B: "\u987F",
		0xDA7C: "\u9880",
		0xDA7D: "\u9881",
		0xDA7E: "\u9882",
		0xDB21: "\u9883",
		0xDB22: "\u9884",
		0xDB23: "\u9885",
		0xDB24: "\u9886",
		0xDB25: "\u9887",
		0xDB26: "\u9888",
		0xDB27: "\u9889",
		0xDB28: "\u988A",
		0xDB29: "\u988B",
		0xDB2A: "\u988C",
		0xDB2B: "\u988D",
		0xDB2C: "\u988E",
		0xDB2D: "\u988F",
		0xDB2E: "\u9890",
		0xDB2F: "\u9891",
		0xDB30: "\u9893",
		0xDB31: "\u9894",
		0xDB32: "\u9896",
		0xDB33: "\u9897",
		0xDB34: "\u9898",
		0xDB35: "\u9899",
		0xDB36: "\u989A",
		0xDB37: "\u989B",
		0xDB38: "\u989C",
		0xDB39: "\u989D",
		0xDB3A: "\u989E",
		0xDB3B: "\u989F",
		0xDB3C: "\u98A0",
		0xDB3D: "\u98A1",
		0xDB3E: "\u98A2",
		0xDB3F: "\u98A3",
		0xDB40: "\u98A4",
		0xDB41: "\u98A5",
		0xDB42: "\u98A6",
		0xDB43: "\u98A7",
		0xDB44: "\u98B3",
		0xDB45: "\u98BA",
		0xDB46: "\u98C7",
		0xDB47: "\u98C8",
		0xDB48: "\u98CE",
		0xDB49: "\u98CF",
		0xDB4A: "\u98D0",
		0xDB4B: "\u98D1",
		0xDB4C: "\u98D2",
		0xDB4D: "\u98D3",
		0xDB4E: "\u98D4",
		0xDB4F: "\u98D5",
		0xDB50: "\u98D6",
		0xDB51: "\u98D7",
		0xDB52: "\u98D8",
		0xDB53: "\u98D9",
		0xDB54: "\u98DE",
		0xDB55: "\u98E1",
		0xDB56: "\u98E5",
		0xDB57: "\u98E6",
		0xDB58: "\u98E7",
		0xDB59: "\u98E8",
		0xDB5A: "\u98F1",
		0xDB5B: "\u9901",
		0xDB5C: "\u9908",
		0xDB5D: "\u990D",
		0xDB5E: "\u991A",
		0xDB5F: "\u9926",
		0xDB60: "\u9927",
		0xDB61: "\u9931",
		0xDB62: "\u9933",
		0xDB63: "\u9935",
		0xDB64: "\u993A",
		0xDB65: "\u993B",
		0xDB66: "\u993C",
		0xDB67: "\u9946",
		0xDB68: "\u994D",
		0xDB69: "\u9954",
		0xDB6A: "\u995C",
		0xDB6B: "\u995D",
		0xDB6C: "\u995E",
		0xDB6D: "\u9960",
		0xDB6E: "\u9964",
		0xDB6F: "\u9965",
		0xDB70: "\u9966",
		0xDB71: "\u9967",
		0xDB72: "\u9968",
		0xDB73: "\u9969",
		0xDB74: "\u996A",
		0xDB75: "\u996B",
		0xDB76: "\u996C",
		0xDB77: "\u996D",
		0xDB78: "\u996E",
		0xDB79: "\u996F",
		0xDB7A: "\u9970",
		0xDB7B: "\u9971",
		0xDB7C: "\u9972",
		0xDB7D: "\u9973",
		0xDB7E: "\u9974",
		0xDC21: "\u9975",
		0xDC22: "\u9976",
		0xDC23: "\u9977",
		0xDC24: "\u9978",
		0xDC25: "\u9979",
		0xDC26: "\u997A",
		0xDC27: "\u997B",
		0xDC28: "\u997C",
		0xDC29: "\u997D",
		0xDC2A: "\u997E",
		0xDC2B: "\u997F",
		0xDC2C: "\u9980",
		0xDC2D: "\u9981",
		0xDC2E: "\u9982",
		0xDC2F: "\u9983",
		0xDC30: "\u9984",
		0xDC31: "\u9985",
		0xDC32: "\u9986",
		0xDC33: "\u9987",
		0xDC34: "\u9988",
		0xDC35: "\u9989",
		0xDC36: "\u998A",
		0xDC37: "\u998B",
		0xDC38: "\u998C",
		0xDC39: "\u998D",
		0xDC3A: "\u998E",
		0xDC3B: "\u998F",
		0xDC3C: "\u9990",
		0xDC3D: "\u9991",
		0xDC3E: "\u9992",
		0xDC3F: "\u9993",
		0xDC40: "\u9994",
		0xDC41: "\u9995",
		0xDC42: "\u99C3",
		0xDC43: "\u9A04",
		0xDC44: "\u9A0C",
		0xDC45: "\u9A10",
		0xDC46: "\u9A17",
		0xDC47: "\u9A20",
		0xDC48: "\u9A27",
		0xDC49: "\u9A32",
		0xDC4A: "\u9A36",
		0xDC4B: "\u9A4A",
		0xDC4C: "\u9A4C",
		0xDC4D: "\u9A4E",
		0xDC4E: "\u9A58",
		0xDC4F: "\u9A6C",
		0xDC50: "\u9A6D",
		0xDC51: "\u9A6E",
		0xDC52: "\u9A6F",
		0xDC53: "\u9A70",
		0xDC54: "\u9A71",
		0xDC55: "\u9A72",
		0xDC56: "\u9A73",
		0xDC57: "\u9A74",
		0xDC58: "\u9A75",
		0xDC59: "\u9A76",
		0xDC5A: "\u9A77",
		0xDC5B: "\u9A78",
		0xDC5C: "\u9A79",
		0xDC5D: "\u9A7A",
		0xDC5E: "\u9A7B",
		0xDC5F: "\u9A7C",
		0xDC60: "\u9A7D",
		0xDC61: "\u9A7E",
		0xDC62: "\u9A7F",
		0xDC63: "\u9A80",
		0xDC64: "\u9A81",
		0xDC65: "\u9A82",
		0xDC66: "\u9A83",
		0xDC67: "\u9A84",
		0xDC68: "\u9A85",
		0xDC69: "\u9A86",
		0xDC6A: "\u9A87",
		0xDC6B: "\u9A88",
		0xDC6C: "\u9A89",
		0xDC6D: "\u9A8A",
		0xDC6E: "\u9A8B",
		0xDC6F: "\u9A8C",
		0xDC70: "\u9A8D",
		0xDC71: "\u9A8E",
		0xDC72: "\u9A8F",
		0xDC73: "\u9A90",
		0xDC74: "\u9A91",
		0xDC75: "\u9A92",
		0xDC76: "\u9A93",
		0xDC77: "\u9A95",
		0xDC78: "\u9A96",
		0xDC79: "\u9A97",
		0xDC7A: "\u9A98",
		0xDC7B: "\u9A99",
		0xDC7C: "\u9A9A",
		0xDC7D: "\u9A9B",
		0xDC7E: "\u9A9C",
		0xDD21: "\u9A9D",
		0xDD22: "\u9A9E",
		0xDD23: "\u9A9F",
		0xDD24: "\u9AA0",
		0xDD25: "\u9AA1",
		0xDD26: "\u9AA2",
		0xDD27: "\u9AA3",
		0xDD28: "\u9AA4",
		0xDD29: "\u9AA5",
		0xDD2A: "\u9AA6",
		0xDD2B: "\u9AA7",
		0xDD2C: "\u9AAB",
		0xDD2D: "\u9AAF",
		0xDD2E: "\u9AB1",
		0xDD2F: "\u9AB3",
		0xDD30: "\u9AB4",
		0xDD31: "\u9AB6",
		0xDD32: "\u9AB7",
		0xDD33: "\u9ABA",
		0xDD34: "\u9ABE",
		0xDD35: "\u9AC1",
		0xDD36: "\u9AC2",
		0xDD37: "\u9AC3",
		0xDD38: "\u9AC5",
		0xDD39: "\u9AC6",
		0xDD3A: "\u9AC8",
		0xDD3B: "\u9ACB",
		0xDD3C: "\u9ACC",
		0xDD3D: "\u9ACE",
		0xDD3E: "\u9AD2",
		0xDD3F: "\u9AD5",
		0xDD40: "\u9AD6",
		0xDD41: "\u9AE0",
		0xDD42: "\u9AE1",
		0xDD43: "\u9AE4",
		0xDD44: "\u9AE5",
		0xDD45: "\u9AE7",
		0xDD46: "\u9AE9",
		0xDD47: "\u9AF2",
		0xDD48: "\u9AF3",
		0xDD49: "\u9AF9",
		0xDD4A: "\u9AFD",
		0xDD4B: "\u9B01",
		0xDD4C: "\u9B03",
		0xDD4D: "\u9B05",
		0xDD4E: "\u9B07",
		0xDD4F: "\u9B08",
		0xDD50: "\u9B0B",
		0xDD51: "\u9B0C",
		0xDD52: "\u9B0D",
		0xDD53: "\u9B0E",
		0xDD54: "\u9B0F",
		0xDD55: "\u9B10",
		0xDD56: "\u9B11",
		0xDD57: "\u9B12",
		0xDD58: "\u9B13",
		0xDD59: "\u9B19",
		0xDD5A: "\u9B21",
		0xDD5B: "\u9B26",
		0xDD5C: "\u9B2C",
		0xDD5D: "\u9B34",
		0xDD5E: "\u9B36",
		0xDD5F: "\u9B46",
		0xDD60: "\u9B47",
		0xDD61: "\u9B48",
		0xDD62: "\u9B49",
		0xDD63: "\u9B4A",
		0xDD64: "\u9B4C",
		0xDD65: "\u9B5E",
		0xDD66: "\u9B84",
		0xDD67: "\u9B8C",
		0xDD68: "\u9B9E",
		0xDD69: "\u9BB7",
		0xDD6A: "\u9BC8",
		0xDD6B: "\u9BD7",
		0xDD6C: "\u9BE5",
		0xDD6D: "\u9BF7",
		0xDD6E: "\u9BFE",
		0xDD6F: "\u9C16",
		0xDD70: "\u9C23",
		0xDD71: "\u9C37",
		0xDD72: "\u9C42",
		0xDD73: "\u9C4D",
		0xDD74: "\u9C4F",
		0xDD75: "\u9C53",
		0xDD76: "\u9C56",
		0xDD77: "\u9C58",
		0xDD78: "\u9C5B",
		0xDD79: "\u9C5F",
		0xDD7A: "\u9C64",
		0xDD7B: "\u9C68",
		0xDD7C: "\u9C6D",
		0xDD7D: "\u9C6E",
		0xDD7E: "\u9C77",
		0xDE21: "\u9C7A",
		0xDE22: "\u9C7C",
		0xDE23: "\u9C7D",
		0xDE24: "\u9C7E",
		0xDE25: "\u9C7F",
		0xDE26: "\u9C80",
		0xDE27: "\u9C81",
		0xDE28: "\u9C82",
		0xDE29: "\u9C83",
		0xDE2A: "\u9C85",
		0xDE2B: "\u9C86",
		0xDE2C: "\u9C87",
		0xDE2D: "\u9C88",
		0xDE2E: "\u9C89",
		0xDE2F: "\u9C8A",
		0xDE30: "\u9C8B",
		0xDE31: "\u9C8C",
		0xDE32: "\u9C8D",
		0xDE33: "\u9C8E",
		0xDE34: "\u9C8F",
		0xDE35: "\u9C90",
		0xDE36: "\u9C91",
		0xDE37: "\u9C92",
		0xDE38: "\u9C94",
		0xDE39: "\u9C95",
		0xDE3A: "\u9C96",
		0xDE3B: "\u9C97",
		0xDE3C: "\u9C98",
		0xDE3D: "\u9C99",
		0xDE3E: "\u9C9A",
		0xDE3F: "\u9C9B",
		0xDE40: "\u9C9C",
		0xDE41: "\u9C9D",
		0xDE42: "\u9C9E",
		0xDE43: "\u9C9F",
		0xDE44: "\u9CA0",
		0xDE45: "\u9CA1",
		0xDE46: "\u9CA2",
		0xDE47: "\u9CA3",
		0xDE48: "\u9CA4",
		0xDE49: "\u9CA5",
		0xDE4A: "\u9CA6",
		0xDE4B: "\u9CA7",
		0xDE4C: "\u9CA8",
		0xDE4D: "\u9CA9",
		0xDE4E: "\u9CAA",
		0xDE4F: "\u9CAB",
		0xDE50: "\u9CAC",
		0xDE51: "\u9CAD",
		0xDE52: "\u9CAE",
		0xDE53: "\u9CAF",
		0xDE54: "\u9CB0",
		0xDE55: "\u9CB1",
		0xDE56: "\u9CB2",
		0xDE57: "\u9CB3",
		0xDE58: "\u9CB4",
		0xDE59: "\u9CB5",
		0xDE5A: "\u9CB7",
		0xDE5B: "\u9CB8",
		0xDE5C: "\u9CB9",
		0xDE5D: "\u9CBA",
		0xDE5E: "\u9CBB",
		0xDE5F: "\u9CBC",
		0xDE60: "\u9CBD",
		0xDE61: "\u9CBE",
		0xDE62: "\u9CBF",
		0xDE63: "\u9CC0",
		0xDE64: "\u9CC1",
		0xDE65: "\u9CC2",
		0xDE66: "\u9CC3",
		0xDE67: "\u9CC4",
		0xDE68: "\u9CC5",
		0xDE69: "\u9CC6",
		0xDE6A: "\u9CC7",
		0xDE6B: "\u9CC8",
		0xDE6C: "\u9CC9",
		0xDE6D: "\u9CCA",
		0xDE6E: "\u9CCC",
		0xDE6F: "\u9CCD",
		0xDE70: "\u9CCE",
		0xDE71: "\u9CCF",
		0xDE72: "\u9CD0",
		0xDE73: "\u9CD1",
		0xDE74: "\u9CD2",
		0xDE75: "\u9CD3",
		0xDE76: "\u9CD4",
		0xDE77: "\u9CD5",
		0xDE78: "\u9CD6",
		0xDE79: "\u9CD7",
		0xDE7A: "\u9CD8",
		0xDE7B: "\u9CD9",
		0xDE7C: "\u9CDA",
		0xDE7D: "\u9CDB",
		0xDE7E: "\u9CDC",
		0xDF21: "\u9CDD",
		0xDF22: "\u9CDE",
		0xDF23: "\u9CDF",
		0xDF24: "\u9CE0",
		0xDF25: "\u9CE1",
		0xDF26: "\u9CE2",
		0xDF27: "\u9CE3",
		0xDF28: "\u9CE4",
		0xDF29: "\u9D33",
		0xDF2A: "\u9D76",
		0xDF2B: "\u9D7C",
		0xDF2C: "\u9D83",
		0xDF2D: "\u9DAC",
		0xDF2E: "\u9DB5",
		0xDF2F: "\u9DBF",
		0xDF30: "\u9DD7",
		0xDF31: "\u9DE5",
		0xDF32: "\u9DEB",
		0xDF33: "\u9DF0",
		0xDF34: "\u9DF3",
		0xDF35: "\u9E0E",
		0xDF36: "\u9E11",
		0xDF37: "\u9E15",
		0xDF38: "\u9E1D",
		0xDF39: "\u9E1F",
		0xDF3A: "\u9E20",
		0xDF3B: "\u9E21",
		0xDF3C: "\u9E22",
		0xDF3D: "\u9E23",
		0xDF3E: "\u9E25",
		0xDF3F: "\u9E26",
		0xDF40: "\u9E27",
		0xDF41: "\u9E28",
		0xDF42: "\u9E29",
		0xDF43: "\u9E2A",
		0xDF44: "\u9E2B",
		0xDF45: "\u9E2C",
		0xDF46: "\u9E2D",
		0xDF47: "\u9E2E",
		0xDF48: "\u9E2F",
		0xDF49: "\u9E30",
		0xDF4A: "\u9E31",
		0xDF4B: "\u9E32",
		0xDF4C: "\u9E33",
		0xDF4D: "\u9E34",
		0xDF4E: "\u9E35",
		0xDF4F: "\u9E36",
		0xDF50: "\u9E37",
		0xDF51: "\u9E38",
		0xDF52: "\u9E39",
		0xDF53: "\u9E3A",
		0xDF54: "\u9E3B",
		0xDF55: "\u9E3C",
		0xDF56: "\u9E3D",
		0xDF57: "\u9E3E",
		0xDF58: "\u9E3F",
		0xDF59: "\u9E40",
		0xDF5A: "\u9E41",
		0xDF5B: "\u9E42",
		0xDF5C: "\u9E43",
		0xDF5D: "\u9E44",
		0xDF5E: "\u9E45",
		0xDF5F: "\u9E46",
		0xDF60: "\u9E47",
		0xDF61: "\u9E48",
		0xDF62: "\u9E49",
		0xDF63: "\u9E4A",
		0xDF64: "\u9E4B",
		0xDF65: "\u9E4C",
		0xDF66: "\u9E4D",
		0xDF67: "\u9E4E",
		0xDF68: "\u9E4F",
		0xDF69: "\u9E50",
		0xDF6A: "\u9E51",
		0xDF6B: "\u9E52",
		0xDF6C: "\u9E53",
		0xDF6D: "\u9E54",
		0xDF6E: "\u9E55",
		0xDF6F: "\u9E56",
		0xDF70: "\u9E57",
		0xDF71: "\u9E58",
		0xDF72: "\u9E59",
		0xDF73: "\u9E5A",
		0xDF74: "\u9E5B",
		0xDF75: "\u9E5C",
		0xDF76: "\u9E5D",
		0xDF77: "\u9E5E",
		0xDF78: "\u9E5F",
		0xDF79: "\u9E60",
		0xDF7A: "\u9E61",
		0xDF7B: "\u9E62",
		0xDF7C: "\u9E63",
		0xDF7D: "\u9E64",
		0xDF7E: "\u9E65",
		0xE021: "\u9E66",
		0xE022: "\u9E67",
		0xE023: "\u9E68",
		0xE024: "\u9E69",
		0xE025: "\u9E6A",
		0xE026: "\u9E6B",
		0xE027: "\u9E6C",
		0xE028: "\u9E6D",
		0xE029: "\u9E6E",
		0xE02A: "\u9E6F",
		0xE02B: "\u9E70",
		0xE02C: "\u9E71",
		0xE02D: "\u9E72",
		0xE02E: "\u9E73",
		0xE02F: "\u9E74",
		0xE030: "\u9E7A",
		0xE031: "\u9E7B",
		0xE032: "\u9E7C",
		0xE033: "\u9E7E",
		0xE034: "\u9E80",
		0xE035: "\u9E82",
		0xE036: "\u9E85",
		0xE037: "\u9E87",
		0xE038: "\u9E90",
		0xE039: "\u9E96",
		0xE03A: "\u9E9E",
		0xE03B: "\u9EA4",
		0xE03C: "\u9EA8",
		0xE03D: "\u9EAB",
		0xE03E: "\u9EAF",
		0xE03F: "\u9EB5",
		0xE040: "\u9EBD",
		0xE041: "\u9EC1",
		0xE042: "\u9EC7",
		0xE043: "\u9EC9",
		0xE044: "\u9ED1",
		0xE045: "\u9ED5",
		0xE046: "\u9EDF",
		0xE047: "\u9EE1",
		0xE048: "\u9EE2",
		0xE049: "\u9EE6",
		0xE04A: "\u9EE7",
		0xE04B: "\u9EE9",
		0xE04C: "\u9EEA",
		0xE04D: "\u9EEE",
		0xE04E: "\u9EF0",
		0xE04F: "\u9EF2",
		0xE050: "\u9EF5",
		0xE051: "\u9EFE",
		0xE052: "\u9EFF",
		0xE053: "\u9F02",
		0xE054: "\u9F03",
		0xE055: "\u9F09",
		0xE056: "\u9F0B",
		0xE057: "\u9F0D",
		0xE058: "\u9F10",
		0xE059: "\u9F12",
		0xE05A: "\u9F17",
		0xE05B: "\u9F19",
		0xE05C: "\u9F1F",
		0xE05D: "\u9F22",
		0xE05E: "\u9F25",
		0xE05F: "\u9F26",
		0xE060: "\u9F27",
		0xE061: "\u9F29",
		0xE062: "\u9F2B",
		0xE063: "\u9F2F",
		0xE064: "\u9F31",
		0xE065: "\u9F34",
		0xE066: "\u9F37",
		0xE067: "\u9F39",
		0xE068: "\u9F3D",
		0xE069: "\u9F41",
		0xE06A: "\u9F46",
		0xE06B: "\u9F47",
		0xE06C: "\u9F49",
		0xE06D: "\u9F50",
		0xE06E: "\u9F51",
		0xE06F: "\u9F5A",
		0xE070: "\u9F69",
		0xE071: "\u9F7F",
		0xE072: "\u9F80",
		0xE073: "\u9F81",
		0xE074: "\u9F82",
		0xE075: "\u9F83",
		0xE076: "\u9F84",
		0xE077: "\u9F85",
		0xE078: "\u9F86",
		0xE079: "\u9F87",
		0xE07A: "\u9F88",
		0xE07B: "\u9F89",
		0xE07C: "\u9F8A",
		0xE07D: "\u9F8B",
		0xE07E: "\u9F8C",
		0xE121: "\u9F90",
		0xE122: "\u9F94",
		0xE123: "\u9F99",
		0xE124: "\u9F9A",
		0xE125: "\u9F9B",
		0xE126: "\u9F9F",
		0xE127: "\u9FA2",
		0xE621: "\u9B21",
		0xE93A: "\uFA1F",
		0xE93B: "\u9C36",
		0xE93C: "\u33A5",
		0xF121: "\u2460",
		0xF122: "\u2461",
		0xF123: "\u2462",
		0xF124: "\u2463",
		0xF125: "\u2464",
		0xF126: "\u2465",
		0xF127: "\u2466",
		0xF128: "\u2467",
		0xF129: "\u2468",
		0xF12A: "\u2469",
		0xF12B: "\u246A",
		0xF12C: "\u246B",
		0xF12D: "\u246C",
		0xF12E: "\u246D",
		0xF12F: "\u246E",
		0xF130: "\u246F",
		0xF131: "\u2470",
		0xF132: "\u2471",
		0xF133: "\u2472",
		0xF134: "\u2473",
	}
}
